@@ -1,4 +1,4 @@
-/* $Id: blast_format.hpp 446084 2014-09-11 11:52:44Z ivanov $
+/* $Id: blast_format.hpp 504527 2016-06-16 03:56:08Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -47,22 +47,44 @@ Author: Jason Papadopoulos
 #include <objtools/align_format/tabular.hpp>
 #include <objtools/align_format/showalign.hpp>
 #include <objtools/align_format/showdefline.hpp>
+#include <objtools/align_format/taxFormat.hpp>
 #include <algo/blast/format/blastfmtutil.hpp>
 #include <algo/blast/blastinput/blast_input.hpp>
 #include <algo/blast/blastinput/blast_args.hpp>
 #include <algo/blast/api/local_db_adapter.hpp>
 #include <algo/blast/api/blast_seqinfosrc.hpp>
+#include <algo/blast/format/sam.hpp>
+#include <objects/blast/blast__.hpp>
+
 
 BEGIN_NCBI_SCOPE
-
+USING_SCOPE(align_format);
 class CCmdLineBlastXML2ReportData;
 
 /// This class formats the BLAST results for command line applications
 class NCBI_XBLASTFORMAT_EXPORT CBlastFormat
 {
 public:
-    /// The line length of pairwise blast output
-    static const int kFormatLineLength = 68;
+
+    ///igblast clone info
+    struct SClone {
+        string na;
+        string chain_type;
+        string aa;
+        string v_gene;
+        string d_gene;
+        string j_gene;
+        string seqid;
+        double identity;
+        string productive;
+    };
+
+    
+    enum {
+        /// The line length of pairwise blast output
+        kFormatLineLength = 68,
+        kMinTaxFormatLineLength = 100
+    };
 
     /// Constructor
     /// @param opts BLAST options used in the search [in]
@@ -104,6 +126,8 @@ public:
     /// tabular/comma-separated value output format. An empty string implies to
     /// use the default value when applicable. [in]
     /// @param domain_db_adapter Domain database for Delta-blast [in]
+    /// @param cmdline Actual command-line for SAM [in]
+    /// @param subjectTag Identifier for set of subjects [in]
     CBlastFormat(const blast::CBlastOptions& opts, 
                  blast::CLocalDbAdapter& db_adapter,
                  blast::CFormattingArgs::EOutputFormat format_type, 
@@ -123,7 +147,9 @@ public:
                  bool is_megablast = false,
                  bool is_indexed = false,
                  const blast::CIgBlastOptions * ig_opts = NULL,
-                 const blast::CLocalDbAdapter* domain_db_adapter = NULL);
+                 const blast::CLocalDbAdapter* domain_db_adapter = NULL,
+                 const string & cmdline = kEmptyStr,
+		 const string& subjectTag = kEmptyStr);
 
     /// Constructor
     /// @param opts BLAST options used in the search [in]
@@ -163,7 +189,8 @@ public:
                  bool is_html = false, 
                  bool is_remote_search = false,
                  const string& custom_output_format = kEmptyStr,
-                 bool is_vdb = false);
+                 bool is_vdb = false,
+                 const string & cmdline = kEmptyStr);
 
     /// Class destructor
     ~CBlastFormat();
@@ -201,6 +228,8 @@ public:
     /// @param result position index if index >= 0 [in]
     void PrintOneResultSet(blast::CIgBlastResults& results,
                            CConstRef<blast::CBlastQueryVector> queries, 
+                           SClone& clone_info,
+                           bool fill_clone_info,
                            int index = -1);
 
     /// Print all alignment information for aa PHI-BLAST run.
@@ -227,7 +256,8 @@ public:
     void WriteArchive(blast::IQueryFactory& queries,
                            blast::CBlastOptionsHandle& options_handle,
                            const blast::CSearchResultSet& results,
-                           unsigned int num_iters = 0);
+                           unsigned int num_iters = 0,
+                           const list<CRef<objects::CBlast4_error> >& msg = list<CRef<objects::CBlast4_error> > () );
 
     /// Writes out the pssm and results as an "archive" format
     /// @param pssm 	psiblast pssm
@@ -237,7 +267,8 @@ public:
     void WriteArchive(objects::CPssmWithParameters & pssm,
                       blast::CBlastOptionsHandle& options_handle,
                       const blast::CSearchResultSet& results,
-                      unsigned int num_iters = 0);
+                      unsigned int num_iters = 0,
+                      const list<CRef<objects::CBlast4_error> > & msg = list<CRef<objects::CBlast4_error> >());
 
     /// Print the footer of the blast report
     /// @param options Options used for performing the blast search [in]
@@ -267,6 +298,9 @@ public:
 
     /// Set Alignment Length
     void SetLineLength(size_t len) {m_LineLength = len;}
+
+    static void PrintArchive(CRef<objects::CBlast4_archive> archive,
+            					  CNcbiOstream& out);
 private:
     /// Format type
     blast::CFormattingArgs::EOutputFormat m_FormatType;
@@ -294,6 +328,10 @@ private:
     CRef<CScope> m_Scope;       
     /** True if we are formatting for BLAST2Sequences */
     bool m_IsBl2Seq;            
+    /// True if it is m_IsBl2Seq in dbscan mode
+    bool m_IsDbScan;
+    /// Tag for subject sequences (e.g., name of input file)
+    string m_SubjectTag;
     /// True if this object is formatting the results of a remote search
     bool m_IsRemoteSearch;
     /// Used to count number of searches formatted. 
@@ -346,6 +384,11 @@ private:
 
     IOS_BASE::iostate m_OrigExceptionMask;
 
+    /// Pointer to the SAM formatting object
+    auto_ptr<CBlast_SAM_Formatter> m_SamFormatter;
+
+    string m_Cmdline;
+
     /// Output the ancillary data for one query that was searched
     /// @param summary The ancillary data to report [in]
     void x_PrintOneQueryFooter(const blast::CBlastAncillaryData& summary);
@@ -407,7 +450,9 @@ private:
 
    /// Prints IgTabular report for one query
    /// @param results Results for one query or Phi-blast iteration [in]
-   void x_PrintIgTabularReport(const blast::CIgBlastResults& results);
+   void x_PrintIgTabularReport(const blast::CIgBlastResults& results,
+                               SClone& clone_info,
+                               bool fill_clone_info);
 
    /// Replace the query with its reversed-compliement
    /// @param results Ig Blast results [in, out]
@@ -435,6 +480,9 @@ private:
    void x_GenerateXML2MasterFile(void);
    void x_GenerateJSONMasterFile(void);
    void x_WriteXML2(CCmdLineBlastXML2ReportData & report_data);
+
+   void x_InitSAMFormatter();
+   void x_PrintTaxReport(const blast::CSearchResults& results);
 };
 
 END_NCBI_SCOPE
