@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMISC__HPP
 #define CORELIB___NCBIMISC__HPP
 
-/*  $Id: ncbimisc.hpp 257775 2011-03-16 14:46:26Z rafanovi $
+/*  $Id: ncbimisc.hpp 500279 2016-05-03 17:12:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,38 +35,27 @@
 /// Miscellaneous common-use basic types and functionality
 
 
-#include <corelib/ncbistl.hpp>
+#include <corelib/ncbidbg.hpp>
+#include <stdlib.h>
 #ifdef HAVE_SYS_TYPES_H
 #  include <sys/types.h>
 #endif
-#ifdef NCBI_COMPILER_ICC
-// Preemptively pull in <cctype>, which breaks if we've already
-// repointed is* at NCBI_is*.
-#  include <cctype>
-#else
-#  include <ctype.h>
-#endif
 
-#if !defined(HAVE_NULLPTR)
+#if !defined(HAVE_NULLPTR)  &&  !defined(nullptr)
 #  define nullptr NULL
 #endif
 
-#if defined(_DEBUG)  &&  !defined(NCBI_NO_STRICT_CTYPE_ARGS)
-#  define NCBI_STRICT_CTYPE_ARGS
-#endif
-
 
 /** @addtogroup AppFramework
  *
  * @{
  */
 
+
 #ifndef NCBI_ESWITCH_DEFINED
 #define NCBI_ESWITCH_DEFINED
 
-#ifdef __cplusplus
 extern "C" {
-#endif
 
 /*
  * ATTENTION!   Do not change this enumeration!
@@ -76,33 +65,48 @@ extern "C" {
  * type, please apply equivalent changes to both definitions.
  */
 
-/** Aux. enum to set/unset/default various features
+/** Aux. enum to set/unset/default various features.
  */
-typedef enum {
+typedef enum ENcbiSwitch {
     eOff = 0,
     eOn,
     eDefault
 } ESwitch;
 
-#ifdef __cplusplus
-}  /* extern "C" */
-#endif
+} // extern "C"
 
-#endif  /* !defined(NCBI_ESWITCH_DEFINED) */
+#endif //!NCBI_ESWITCH_DEFINED
 
 
-BEGIN_NCBI_SCOPE
+#ifndef NCBI_EOWNERSHIP_DEFINED
+#define NCBI_EOWNERSHIP_DEFINED
+
+extern "C" {
+
+/*
+ * ATTENTION!   Do not change this enumeration!
+ *
+ * It must always be kept in sync with its plain C counterpart defined in
+ * "connect/ncbi_types.h". If you absolutely(sic!) need to alter this
+ * type, please apply equivalent changes to both definitions.
+ */
+
+/** Ownership relations between objects.
+ *
+ * Can be used to define or transfer ownership of objects.
+ * For example, specify if a CSocket object owns its underlying SOCK object.
+ */
+typedef enum ENcbiOwnership {
+    eNoOwnership,       /** No ownership assumed                    */
+    eTakeOwnership      /** An object can take ownership of another */
+} EOwnership;
+
+} // extern "C"
+
+#endif //!NCBI_EOWNERSHIP_DEFINED
 
 
-/// Which type of ownership between objects.
-///
-/// Can be used to specify ownership relationship between objects.
-/// For example, specify if a CSocket object owns the underlying
-/// SOCK object. 
-enum EOwnership {
-    eNoOwnership,       ///< No ownership relationship
-    eTakeOwnership      ///< An object can take ownership of another
-};
+BEGIN_NCBI_NAMESPACE;
 
 
 /// Whether a value is nullable.
@@ -112,7 +116,7 @@ enum ENullable {
 };
 
 
-/// Whether a value is nullable.
+/// Signedness of a value.
 enum ESign {
     eNegative = -1,     ///< Value is negative
     eZero     =  0,     ///< Value is zero
@@ -156,34 +160,27 @@ enum EInterruptOnSignal {
 /////////////////////////////////////////////////////////////////////////////
 
 
-/// Macro to hide all operators with bool argument which may be used
-/// unintentionally when second argument is of class having operator bool().
-/// All methods are simply declared private without body definition.
-#define HIDE_SAFE_BOOL_OPERATORS()                      \
-    private:                                            \
-    void operator<(bool) const;                         \
-    void operator>(bool) const;                         \
-    void operator<=(bool) const;                        \
-    void operator>=(bool) const;                        \
-    void operator==(bool) const;                        \
-    void operator!=(bool) const;                        \
-    void operator+(bool) const;                         \
-    void operator-(bool) const;                         \
-    public:
-
-
-/// Low level macro for declaring bool operator.
-#define DECLARE_SAFE_BOOL_METHOD(Expr)                  \
-    operator bool(void) const {                         \
-        return (Expr);                                  \
-    }
+/// Low level macro for declaring safe bool operator.
+#define DECLARE_SAFE_BOOL_METHOD(Expr)                                  \
+    struct SSafeBoolTag {                                               \
+        void SafeBoolTrue(SSafeBoolTag*) {}                             \
+    };                                                                  \
+    typedef void (SSafeBoolTag::*TBoolType)(SSafeBoolTag*);             \
+    operator TBoolType() const {                                        \
+        return (Expr)? &SSafeBoolTag::SafeBoolTrue: 0;                  \
+    }                                                                   \
+    private:                                                            \
+    bool operator==(TBoolType) const;                                   \
+    bool operator!=(TBoolType) const;                                   \
+    public:                                                             \
+    friend struct SSafeBoolTag
 
 
 /// Declaration of safe bool operator from boolean expression.
 /// Actual operator declaration will be:
-///    operator bool(void) const;
+///    operator TBoolType(void) const;
+/// where TBoolType is a typedef convertible to bool (member pointer).
 #define DECLARE_OPERATOR_BOOL(Expr)             \
-    HIDE_SAFE_BOOL_OPERATORS()                  \
     DECLARE_SAFE_BOOL_METHOD(Expr)
 
 
@@ -200,6 +197,16 @@ enum EInterruptOnSignal {
 #define DECLARE_OPERATOR_BOOL_REF(Ref)          \
     DECLARE_OPERATOR_BOOL((Ref).NotNull())
 
+/// Override the DECLARE_OPERATOR_BOOL, etc.
+/// from an ancestor class.  This is needed because sometimes you can't just 
+/// use DECLARE_OPERATOR_BOOL due to cases such as the ancestor class
+/// being privately inherited from.
+#define OVERRIDE_OPERATOR_BOOL(TAncestorClass, NewBoolExpr)     \
+    using TAncestorClass::TBoolType;                                    \
+    using TAncestorClass::SSafeBoolTag;                                 \
+    operator TBoolType() const {                                \
+        return (NewBoolExpr)? & SSafeBoolTag::SafeBoolTrue : 0;        \
+    }
 
 /// Template used for empty base class optimization.
 /// See details in the August '97 "C++ Issue" of Dr. Dobb's Journal
@@ -276,6 +283,41 @@ private:
 };
 
 
+/// Template used to replace bool type arguments with some strict equivalent.
+/// This allow to prevent compiler to do an implicit casts from other types
+/// to bool. "TEnum" should be an enumerated type with two values:
+///   - negative (FALSE/OFF) should have value 0
+//    - positive (TRUE/ON) have value 1.
+
+template <class TEnum>
+class CBoolEnum
+{
+public:
+    // Constructors
+    CBoolEnum(bool  value) : m_Value(value) {}
+    CBoolEnum(TEnum value) : m_Value(value ? true : false) {}
+
+    /// Operator bool
+    operator bool() const   { return m_Value; }
+    /// Operator enum
+    operator TEnum () const { return TEnum(m_Value); }
+
+private:
+    bool m_Value;
+
+private:
+    // Disable implicit conversions from/to other types
+    CBoolEnum(char*);
+    CBoolEnum(int);
+    CBoolEnum(unsigned int);
+    template <class T> CBoolEnum(T);
+
+    operator int() const;
+    operator unsigned int() const;
+};
+
+
+
 #ifdef HAVE_NO_AUTO_PTR
 
 
@@ -420,18 +462,18 @@ struct CDeleter
 /// Define an "auto_ptr" like class that can be used inside STL containers.
 ///
 /// The Standard auto_ptr template from STL doesn't allow the auto_ptr to be
-/// put in STL containers (list, vector, map etc.). The reason for this is
+/// put in STL containers (list, vector, map etc.).  The reason for this is
 /// the absence of copy constructor and assignment operator.
 /// We decided that it would be useful to have an analog of STL's auto_ptr
 /// without this restriction - AutoPtr.
 ///
-/// Due to nature of AutoPtr its copy constructor and assignment operator
-/// modify the state of the source AutoPtr object as it transfers ownership
-/// to the target AutoPtr object. Also, we added possibility to redefine the
-/// way pointer will be deleted: the second argument of template allows
+/// Due to the nature of AutoPtr its copy constructor and assignment operator
+/// modify the state of the source AutoPtr object as it transfers the ownership
+/// to the target AutoPtr object.  Also, we added possibility to redefine the
+/// way pointer will be deleted:  the second argument of the template allows
 /// pointers from "malloc" in AutoPtr, or you can use "ArrayDeleter" (see
 /// above) to properly delete an array of objects using "delete[]" instead
-/// of "delete". By default, the internal pointer will be deleted by C++
+/// of "delete".  By default, the internal pointer is deleted by the C++
 /// "delete" operator.
 ///
 /// @sa
@@ -447,26 +489,22 @@ public:
     /// Constructor.
     AutoPtr(element_type* p = 0)
         : m_Ptr(p), m_Data(true)
-    {
-    }
+    { }
 
     /// Constructor.
     AutoPtr(element_type* p, const deleter_type& deleter)
         : m_Ptr(p), m_Data(deleter, true)
-    {
-    }
+    { }
 
     /// Constructor, own the pointed object if ownership == eTakeOwnership
     AutoPtr(element_type* p, EOwnership ownership)
         : m_Ptr(p), m_Data(ownership != eNoOwnership)
-    {
-    }
+    { }
 
     /// Constructor, own the pointed object if ownership == eTakeOwnership
     AutoPtr(element_type* p, const deleter_type& deleter, EOwnership ownership)
         : m_Ptr(p), m_Data(deleter, ownership != eNoOwnership)
-    {
-    }
+    { }
 
     /// Copy constructor.
     AutoPtr(const AutoPtr<X, Del>& p)
@@ -520,8 +558,8 @@ public:
         return m_Ptr;
     }
 
-    /// Reset will delete old pointer, set content to new value,
-    /// and accept ownership upon the new pointer.
+    /// Reset will delete the old pointer (if owned), set content to the new
+    /// value, and assume the ownership upon the new pointer by default.
     void reset(element_type* p = 0, EOwnership ownership = eTakeOwnership)
     {
         if ( m_Ptr != p ) {
@@ -530,15 +568,17 @@ public:
             }
             m_Ptr   = p;
         }
-        m_Data.second() = p != 0 && ownership == eTakeOwnership;
+        m_Data.second() = ownership != eNoOwnership;
     }
 
     void Swap(AutoPtr<X, Del>& a)
     {
-        swap(m_Ptr, a.m_Ptr);
+        swap(m_Ptr,  a.m_Ptr);
         swap(m_Data, a.m_Data);
     }
 
+    bool IsOwned(void) const { return m_Ptr  &&  m_Data.second(); }
+
 private:
     element_type* m_Ptr;                  ///< Internal pointer representation.
     mutable pair_base_member<deleter_type, bool> m_Data; ///< State info.
@@ -558,8 +598,8 @@ private:
 /// "AutoPtr" like class for using with arrays
 ///
 /// vector<> template comes with a performance penalty, since it always
-/// initializes its content. This template is not a vector replacement,
-/// it's a version of AutoPtr<> tuned for array pointers. For convenience
+/// initializes its content.  This template is not a vector replacement,
+/// it's a version of AutoPtr<> tuned for array pointers.  For convenience
 /// it defines array style access operator [] and size based contructor.
 ///
 /// @sa AutoPtr
@@ -578,16 +618,15 @@ public:
     /// @note In this case you should use ArrayDeleter<> or compatible
     explicit AutoArray(size_t size)
         : m_Ptr(new element_type[size]), m_Data(true)
-    {}
+    { }
 
     explicit AutoArray(element_type* p = 0)
         : m_Ptr(p), m_Data(true)
-    {}
+    { }
 
     AutoArray(element_type* p, const deleter_type& deleter)
         : m_Ptr(p), m_Data(deleter, true)
-    {
-    }
+    { }
 
     AutoArray(const AutoArray<X, Del>& p)
         : m_Ptr(0), m_Data(p.m_Data)
@@ -617,11 +656,12 @@ public:
         reset(p);
         return *this;
     }
+
     /// Bool operator for use in if() clause.
     DECLARE_OPERATOR_BOOL_PTR(m_Ptr);
 
     /// Get pointer.
-    element_type* get       (void) const { return  m_Ptr; }
+    element_type* get (void) const { return  m_Ptr; }
 
     /// Release will release ownership of pointer to caller.
     element_type* release(void)
@@ -630,26 +670,28 @@ public:
         return m_Ptr;
     }
 
-    /// array style dereference (returns value)
+    /// Array style dereference (returns value)
     const element_type& operator[](size_t pos) const { return m_Ptr[pos]; }
 
-    /// array style dereference (returns reference)
-    element_type& operator[](size_t pos) { return m_Ptr[pos]; }
+    /// Array style dereference (returns reference)
+    element_type&       operator[](size_t pos)       { return m_Ptr[pos]; }
 
-    /// Reset will delete old pointer, set content to new value,
-    /// and accept ownership upon the new pointer.
+    /// Reset will delete the old pointer, set content to the new value,
+    /// and assume the ownership upon the new pointer.
     void reset(element_type* p = 0)
     {
-        if (m_Ptr  &&  m_Data.second()) {
-            m_Data.first().Delete(release());
+        if (m_Ptr != p) {
+            if (m_Ptr  &&  m_Data.second()) {
+                m_Data.first().Delete(release());
+            }
+            m_Ptr  = p;
         }
-        m_Ptr   = p;
         m_Data.second() = true;
     }
 
     void Swap(AutoPtr<X, Del>& a)
     {
-        swap(m_Ptr, a.m_Ptr);
+        swap(m_Ptr,  a.m_Ptr);
         swap(m_Data, a.m_Data);
     }
 
@@ -661,12 +703,112 @@ private:
     }
 
 private:
-    element_type*  m_Ptr;
+    element_type* m_Ptr;
     mutable pair_base_member<deleter_type, bool> m_Data; ///< State info.
 };
 
 
 
+/////////////////////////////////////////////////////////////////////////////
+///
+/// CNullable --
+///
+/// A value whith 'unassigned' state.
+///
+
+
+/// Define "null" pointer value.
+enum ENull {
+    null = 0
+};
+
+
+// Default callback for null value - throws CCoreException.
+NCBI_XNCBI_EXPORT NCBI_NORETURN void g_ThrowOnNull(void);
+
+// Default callback template.
+template <class TValue>
+struct SThrowOnNull
+{
+    TValue operator()(void) const
+    {
+        g_ThrowOnNull();
+    }
+};
+
+
+/// Template class allowing to store a value or null (unassigned) state.
+/// TNullToValue functor can be used to perform an action when the value
+/// is requested from a null object. By default CCoreException is thrown.
+/// To perform other actions (e.g. provide a default value) the functor
+/// must define 'TValue operator()(void) const' method.
+template <class TValue, class TNullToValue = SThrowOnNull<TValue> >
+class CNullable
+{
+public:
+    /// Create an empty nullable.
+    CNullable(ENull = null)
+        : m_IsNull(true) {}
+    /// Initialize nullable with a specific value.
+    CNullable(TValue value)
+        : m_IsNull(false), m_Value(value) {}
+
+    /// Check if the object is unassigned.
+    bool IsNull(void) const
+    {
+        return m_IsNull;
+    }
+
+    /// Get nullable value.
+    /// If NULL, then call TNullToValue and use the value return by the latter.
+    /// @attention  The default implementation of TNullToValue (g_ThrowOnNull)
+    ///             throws an exception! 
+    operator TValue(void) const
+    {
+        return m_IsNull ? TNullToValue()() : m_Value;
+    }
+
+    /// Get a const reference to the current value. If NULL, the reference is
+    /// a copy of the default value or throw (depending on TNullToValue
+    /// implementation). In any case the IsNull state is unchanged.
+    const TValue& GetValue(void) const
+    {
+        if ( m_IsNull ) {
+            const_cast<TValue&>(m_Value) = TNullToValue()();
+        }
+        return m_Value;
+    }
+
+    /// Get a non-const reference to the value. If NULL, try to initialize this
+    /// instance with the default value or throw.
+    TValue& SetValue(void)
+    {
+        if ( m_IsNull ) {
+            *this = TNullToValue()();
+        }
+        return m_Value;
+    }
+
+    /// Assign a value to the nullable.
+    CNullable& operator= (TValue value)
+    {
+        m_IsNull = false; m_Value = value;
+        return *this;
+    }
+    
+    /// Reset nullable to unassigned state.
+    CNullable& operator= (ENull  null_value)
+    {
+        m_IsNull = true;
+        return *this;
+    }
+
+private:
+    bool   m_IsNull;
+    TValue m_Value;
+};
+
+
 // "min" and "max" templates
 //
 
@@ -708,205 +850,102 @@ extern char* strdup(const char* str);
 
 
 
-// ctype hacks
-//
-
-#ifdef NCBI_STRICT_CTYPE_ARGS
-
-END_NCBI_SCOPE
-
-#define NCBI_CTYPEFAKEBODY \
-  { return See_the_standard_on_proper_argument_type_for_ctype_macros(c); }
-
-#ifdef isalpha
-inline int NCBI_isalpha(unsigned char c) { return isalpha(c); }
-inline int NCBI_isalpha(int           c) { return isalpha(c); }
-template<class C>
-inline int NCBI_isalpha(C c) NCBI_CTYPEFAKEBODY
-#undef  isalpha
-#define isalpha NCBI_isalpha
-#endif
-
-#ifdef isalnum
-inline int NCBI_isalnum(unsigned char c) { return isalnum(c); }
-inline int NCBI_isalnum(int           c) { return isalnum(c); }
-template<class C>
-inline int NCBI_isalnum(C c) NCBI_CTYPEFAKEBODY
-#undef  isalnum
-#define isalnum NCBI_isalnum
-#endif
-
-#ifdef isascii
-inline int NCBI_isascii(unsigned char c) { return isascii(c); }
-inline int NCBI_isascii(int           c) { return isascii(c); }
-template<class C>
-inline int NCBI_isascii(C c) NCBI_CTYPEFAKEBODY
-#undef  isascii
-#define isascii NCBI_isascii
-#endif
-
-#ifdef isblank
-inline int NCBI_isblank(unsigned char c) { return isblank(c); }
-inline int NCBI_isblank(int           c) { return isblank(c); }
-template<class C>
-inline int NCBI_isblank(C c) NCBI_CTYPEFAKEBODY
-#undef  isblank
-#define isblank NCBI_isblank
-#endif
-
-#ifdef iscntrl
-inline int NCBI_iscntrl(unsigned char c) { return iscntrl(c); }
-inline int NCBI_iscntrl(int           c) { return iscntrl(c); }
-template<class C>
-inline int NCBI_iscntrl(C c) NCBI_CTYPEFAKEBODY
-#undef  iscntrl
-#define iscntrl NCBI_iscntrl
-#endif
-
-#ifdef isdigit
-inline int NCBI_isdigit(unsigned char c) { return isdigit(c); }
-inline int NCBI_isdigit(int           c) { return isdigit(c); }
-template<class C>
-inline int NCBI_isdigit(C c) NCBI_CTYPEFAKEBODY
-#undef  isdigit
-#define isdigit NCBI_isdigit
-#endif
-
-#ifdef isgraph
-inline int NCBI_isgraph(unsigned char c) { return isgraph(c); }
-inline int NCBI_isgraph(int           c) { return isgraph(c); }
-template<class C>
-inline int NCBI_isgraph(C c) NCBI_CTYPEFAKEBODY
-#undef  isgraph
-#define isgraph NCBI_isgraph
-#endif
-
-#ifdef islower
-inline int NCBI_islower(unsigned char c) { return islower(c); }
-inline int NCBI_islower(int           c) { return islower(c); }
-template<class C>
-inline int NCBI_islower(C c) NCBI_CTYPEFAKEBODY
-#undef  islower
-#define islower NCBI_islower
-#endif
-
-#ifdef isprint
-inline int NCBI_isprint(unsigned char c) { return isprint(c); }
-inline int NCBI_isprint(int           c) { return isprint(c); }
-template<class C>
-inline int NCBI_isprint(C c) NCBI_CTYPEFAKEBODY
-#undef  isprint
-#define isprint NCBI_isprint
-#endif
-
-#ifdef ispunct
-inline int NCBI_ispunct(unsigned char c) { return ispunct(c); }
-inline int NCBI_ispunct(int           c) { return ispunct(c); }
-template<class C>
-inline int NCBI_ispunct(C c) NCBI_CTYPEFAKEBODY
-#undef  ispunct
-#define ispunct NCBI_ispunct
-#endif
-
-#ifdef isspace
-inline int NCBI_isspace(unsigned char c) { return isspace(c); }
-inline int NCBI_isspace(int           c) { return isspace(c); }
-template<class C>
-inline int NCBI_isspace(C c) NCBI_CTYPEFAKEBODY
-#undef  isspace
-#define isspace NCBI_isspace
-#endif
-
-#ifdef isupper
-inline int NCBI_isupper(unsigned char c) { return isupper(c); }
-inline int NCBI_isupper(int           c) { return isupper(c); }
-template<class C>
-inline int NCBI_isupper(C c) NCBI_CTYPEFAKEBODY
-#undef  isupper
-#define isupper NCBI_isupper
-#endif
-
-#ifdef isxdigit
-inline int NCBI_isxdigit(unsigned char c) { return isxdigit(c); }
-inline int NCBI_isxdigit(int           c) { return isxdigit(c); }
-template<class C>
-inline int NCBI_isxdigit(C c) NCBI_CTYPEFAKEBODY
-#undef  isxdigit
-#define isxdigit NCBI_isxdigit
-#endif
-
-#ifdef toascii
-inline int NCBI_toascii(unsigned char c) { return toascii(c); }
-inline int NCBI_toascii(int           c) { return toascii(c); }
-template<class C>
-inline int NCBI_toascii(C c) NCBI_CTYPEFAKEBODY
-#undef  toascii
-#define toascii NCBI_toascii
-#endif
-
-#ifdef tolower
-inline int NCBI_tolower(unsigned char c) { return tolower(c); }
-inline int NCBI_tolower(int           c) { return tolower(c); }
-template<class C>
-inline int NCBI_tolower(C c) NCBI_CTYPEFAKEBODY
-#undef  tolower
-#define tolower NCBI_tolower
-#endif
-
-#ifdef toupper
-inline int NCBI_toupper(unsigned char c) { return toupper(c); }
-inline int NCBI_toupper(int           c) { return toupper(c); }
-template<class C>
-inline int NCBI_toupper(C c) NCBI_CTYPEFAKEBODY
-#undef  toupper
-#define toupper NCBI_toupper
-#endif
-
-#undef NCBI_CTYPEFAKEBODY
-
-BEGIN_NCBI_SCOPE
-
-#endif // NCBI_STRICT_CTYPE_ARGS
-
-
-
 //  ITERATE
 //  NON_CONST_ITERATE
+//  REVERSE_ITERATE
+//  NON_CONST_REVERSE_ITERATE
 //  ERASE_ITERATE
 //
-// Useful macro to write 'for' statements with the STL container iterator as
+// Useful macros to write 'for' statements with the STL container iterator as
 // a variable.
 //
 
+// *ITERATE helper to enforce constness of the container reference
+template<typename Type>
+inline const Type& s_ITERATE_ConstRef(const Type& obj)
+{
+    return obj;
+}
+#define ITERATE_CONST(Cont) NCBI_NS_NCBI::s_ITERATE_ConstRef(Cont)
+
+// *ITERATE helper to verify that the container isn't a temporary object
+#ifdef _DEBUG
+template<typename Type>
+inline bool s_ITERATE_SameObject(const Type& obj1, const Type& obj2)
+{
+    return &obj1 == &obj2;
+}
+# define ITERATE_BEGIN(Cont, Begin)                                     \
+    (NCBI_ASSERT_EXPR(NCBI_NS_NCBI::s_ITERATE_SameObject(Cont, Cont),   \
+                      "rvalue container in *ITERATE"), (Cont).Begin())
+#else
+# define ITERATE_BEGIN(Cont, Begin) ((Cont).Begin())
+#endif
+
+// *ITERATE helper macro to declare iterator variable
+#if 0 && defined(NCBI_HAVE_CXX11)
+# define ITERATE_VAR(Type) auto
+#else
+# define ITERATE_VAR(Type) Type
+#endif
+
 /// ITERATE macro to sequence through container elements.
-#define ITERATE(Type, Var, Cont) \
-    for ( Type::const_iterator Var = (Cont).begin(), NCBI_NAME2(Var,_end) = (Cont).end();  Var != NCBI_NAME2(Var,_end);  ++Var )
+#define ITERATE(Type, Var, Cont)                                        \
+    for ( ITERATE_VAR(Type::const_iterator)                             \
+              Var = ITERATE_BEGIN(ITERATE_CONST(Cont), begin),          \
+              NCBI_NAME2(Var,_end) = ITERATE_CONST(Cont).end();         \
+          Var != NCBI_NAME2(Var,_end);  ++Var )
 
 /// Non constant version of ITERATE macro.
-#define NON_CONST_ITERATE(Type, Var, Cont) \
-    for ( Type::iterator Var = (Cont).begin();  Var != (Cont).end();  ++Var )
+#define NON_CONST_ITERATE(Type, Var, Cont)                              \
+    for ( ITERATE_VAR(Type::iterator) Var = ITERATE_BEGIN(Cont, begin); \
+          Var != (Cont).end();  ++Var )
 
-/// Non constant version with ability to erase current element, if container permits.
-/// Use only on containers, for which erase do not ruin other iterators into the container
-/// e.g., map, list, but NOT vector.
+/// ITERATE macro to reverse sequence through container elements.
+#define REVERSE_ITERATE(Type, Var, Cont)                                \
+    for ( ITERATE_VAR(Type::const_reverse_iterator)                     \
+              Var = ITERATE_BEGIN(ITERATE_CONST(Cont), rbegin),         \
+              NCBI_NAME2(Var,_end) = ITERATE_CONST(Cont).rend();        \
+          Var != NCBI_NAME2(Var,_end);  ++Var )
+
+/// Non constant version of REVERSE_ITERATE macro.
+#define NON_CONST_REVERSE_ITERATE(Type, Var, Cont)                      \
+    for ( ITERATE_VAR(Type::reverse_iterator)                           \
+              Var = ITERATE_BEGIN(Cont, rbegin);                        \
+          Var != (Cont).rend();  ++Var )
+
+/// Non-constant version with ability to erase current element, if container
+/// permits. Use only on containers, for which erase do not ruin other
+/// iterators into the container, e.g. map, list, but NOT vector.
 /// See also VECTOR_ERASE
 #define ERASE_ITERATE(Type, Var, Cont) \
-    for ( Type::iterator Var, NCBI_NAME2(Var,_next) = (Cont).begin();   \
+    for ( ITERATE_VAR(Type::iterator) Var = ITERATE_BEGIN(Cont, begin), \
+              NCBI_NAME2(Var,_next) = Var;                              \
           (Var = NCBI_NAME2(Var,_next)) != (Cont).end() &&              \
               (++NCBI_NAME2(Var,_next), true); )
+
 /// Use this macro inside body of ERASE_ITERATE cycle to erase from
 /// vector-like container. Plain erase() call would invalidate Var_next
 /// iterator and would make the cycle controlling code to fail.
 #define VECTOR_ERASE(Var, Cont) (NCBI_NAME2(Var,_next) = (Cont).erase(Var))
 
-/// ITERATE macro to reverse sequence through container elements.
-#define REVERSE_ITERATE(Type, Var, Cont) \
-    for ( Type::const_reverse_iterator Var = (Cont).rbegin(), NCBI_NAME2(Var,_end) = (Cont).rend();  Var != NCBI_NAME2(Var,_end);  ++Var )
+/// The body of the loop will be run with Var equal to false and then true.
+/// The seemlingly excessive complexity of this macro is to get around a couple of limitations:
+/// * A bool only has two states, so it's not possible to represent the complete state space
+///   of (first iteration, second iteration, done) without another variable.
+/// * The variables declared in a for-loop's first part must be of the same type, so
+///   the other variable has to be a bool instead of something more convenient such as
+///   as a loop-counter.
+#define ITERATE_BOTH_BOOL_VALUES(BoolVar) \
+    for( bool BoolVar##BOTH_BOOL_VALUES_DONE##__LINE__ = false, BoolVar = false; ! BoolVar##BOTH_BOOL_VALUES_DONE##__LINE__  ; BoolVar##BOTH_BOOL_VALUES_DONE##__LINE__ = BoolVar, BoolVar = true )
 
-/// Non constant version of REVERSE_ITERATE macro.
-#define NON_CONST_REVERSE_ITERATE(Type, Var, Cont) \
-    for ( Type::reverse_iterator Var = (Cont).rbegin();  Var != (Cont).rend();  ++Var )
+/// idx loops from 0 (inclusive) to up_to (exclusive)
+#define ITERATE_0_IDX(idx, up_to) \
+    for( TSeqPos idx = 0; idx < up_to; ++idx )
 
+/// Just repeat the body of the loop num_iters times
+#define ITERATE_SIMPLE(num_iters) \
+    ITERATE_0_IDX( _dummy_idx_94768308_##__LINE__, num_iters ) // the number has no significance; it is entirely random.
 
 /// Type for sequence locations and lengths.
 ///
@@ -925,6 +964,170 @@ const TSeqPos kInvalidSeqPos = ((TSeqPos) (-1));
 /// Use this typedef rather than its expansion, which may change.
 typedef int TSignedSeqPos;
 
+/// Type for Taxon1-name.taxid
+typedef int TTaxId;
+
+/// Type for sequence GI.
+///
+/// Use this typedef rather than its expansion, which may change.
+
+//#define NCBI_STRICT_GI
+//#define NCBI_INT8_GI
+//#define NCBI_STRICT_ENTREZ_ID
+
+#ifdef NCBI_STRICT_GI
+# define NCBI_INT8_GI
+#endif
+
+#ifdef NCBI_INT8_GI
+
+// Generic id type which needs to be the same size as GI.
+typedef Int8 TIntId;
+typedef Uint8 TUintId;
+
+#ifdef NCBI_STRICT_GI
+
+// Strict mode can be enabled only for Int8 GIs.
+
+class CStrictId64
+{
+public:
+    CStrictId64(void) : m_Gi(0) {}
+
+    bool operator==(const CStrictId64& gi) const { return m_Gi == gi.m_Gi; }
+    bool operator!=(const CStrictId64& gi) const { return m_Gi != gi.m_Gi; }
+    bool operator<(const CStrictId64& gi) const { return m_Gi < gi.m_Gi; }
+    bool operator<=(const CStrictId64& gi) const { return m_Gi <= gi.m_Gi; }
+    bool operator>(const CStrictId64& gi) const { return m_Gi > gi.m_Gi; }
+    bool operator>=(const CStrictId64& gi) const { return m_Gi >= gi.m_Gi; }
+
+    CStrictId64& operator++(void) { m_Gi++; return *this; }
+    CStrictId64 operator++(int) { CStrictId64 tmp = *this; m_Gi++; return tmp; }
+    CStrictId64& operator--(void) { m_Gi--; return *this; }
+    CStrictId64 operator--(int) { CStrictId64 tmp = *this; m_Gi--; return tmp; }
+
+    TIntId operator+(const CStrictId64& gi) const { return m_Gi + gi.m_Gi; }
+    TIntId operator-(const CStrictId64& gi) const { return m_Gi - gi.m_Gi; }
+
+    CStrictId64 operator+(TIntId offset) const { return m_Gi + offset; }
+    CStrictId64 operator-(TIntId offset) const { return m_Gi - offset; }
+#if defined(NCBI_INT8_GI)
+    CStrictId64 operator+(Int4 offset) const { return m_Gi + offset; }
+    CStrictId64 operator-(Int4 offset) const { return m_Gi - offset; }
+#endif
+
+    CStrictId64(TIntId value) : m_Gi(value) {}
+    CStrictId64& operator=(TIntId value) { m_Gi = value; return *this; }
+    operator TIntId(void) const { return m_Gi; }
+    bool operator==(TIntId value) const { return m_Gi == value; }
+
+#if defined(NCBI_INT8_GI) && defined(NCBI_TEST_APPLICATION)
+    CStrictId64(Int4 value) : m_Gi(value) {}
+    CStrictId64& operator=(Int4 value) { m_Gi = value; return *this; }
+    bool operator==(Int4 value) const { return m_Gi == value; }
+#endif
+
+    operator bool(void) const { return m_Gi != 0; }
+
+private:
+#if defined(NCBI_INT8_GI) && !defined(NCBI_TEST_APPLICATION)
+    CStrictId64(Int4);
+    CStrictId64& operator=(Int4);
+    operator Int4(void) const;
+#endif
+    CStrictId64(Int1);
+    CStrictId64(Uint1);
+    CStrictId64(Int2);
+    CStrictId64(Uint2);
+    CStrictId64(Uint4);
+    CStrictId64(Uint8);
+    CStrictId64(float);
+    CStrictId64(double);
+    CStrictId64& operator=(Int1);
+    CStrictId64& operator=(Uint1);
+    CStrictId64& operator=(Int2);
+    CStrictId64& operator=(Uint2);
+    CStrictId64& operator=(Uint4);
+    CStrictId64& operator=(Uint8);
+    CStrictId64& operator=(float);
+    CStrictId64& operator=(double);
+    operator Int1(void) const;
+    operator Uint1(void) const;
+    operator Int2(void) const;
+    operator Uint2(void) const;
+    operator Uint4(void) const;
+    operator Uint8(void) const;
+    operator float(void) const;
+    operator double(void) const;
+
+private:
+    TIntId m_Gi;
+};
+
+/// @deprecated: Use CStrictId64 instead of CStrictGi, or TGi/TEntrezId typedefs.
+NCBI_DEPRECATED typedef CStrictId64 CStrictGi;
+
+
+inline
+CNcbiOstream& operator<<(CNcbiOstream& out, const CStrictId64& gi)
+{
+    return out << TIntId(gi);
+}
+
+inline
+CNcbiIstream& operator>>(CNcbiIstream& in, CStrictId64& gi)
+{
+    TIntId id;
+    in >> id;
+    gi = id;
+    return in;
+}
+
+typedef CStrictId64 TGi;
+
+#else // NCBI_STRICT_GI
+
+typedef TIntId TGi;
+
+#endif // NCBI_STRICT_GI
+
+#else // NCBI_INT8_GI
+
+typedef int TGi;
+typedef Int4 TIntId;
+typedef Uint4 TUintId;
+
+#endif
+
+/// TEntrezId type for entrez ids which require the same strictness as TGi.
+#ifdef NCBI_STRICT_ENTREZ_ID
+# ifndef NCBI_STRICT_GI
+#  undef NCBI_STRICT_ENTREZ_ID
+# endif
+typedef TGi TEntrezId;
+#else
+typedef TIntId TEntrezId;
+#endif
+
+/// a helper template to enforce constness of argument to GI_CONST macro
+template<TIntId gi>
+class CConstGIChecker {
+public:
+    static const TIntId value = gi;
+};
+#define GI_CONST(gi) (TGi(CConstGIChecker<gi>::value))
+#define ZERO_GI GI_CONST(0)
+#define INVALID_GI GI_CONST(-1)
+
+/// Temporary macros to convert TGi to other types (int, unsigned etc.).
+#define GI_TO(T, gi) (static_cast<T>(TIntId(gi)))
+#define GI_FROM(T, value) (TGi(static_cast<TIntId>(value)))
+
+/// Convert gi-compatible int to/from other types.
+#define INT_ID_TO(T, id) (static_cast<T>(id))
+#define INT_ID_FROM(T, value) (static_cast<TIntId>(value))
+
+
 /// Helper address class
 class CRawPointer
 {
@@ -969,9 +1172,9 @@ template <size_t KEmbeddedSize, class TType = char>
 class CFastBuffer
 {
 public:
-    CFastBuffer(size_t size)
-        : m_Size(size),
-          m_Buffer(size <= KEmbeddedSize ? m_EmbeddedBuffer : new TType[size])
+    CFastBuffer(size_t buf_size)
+        : m_Size(buf_size),
+          m_Buffer(buf_size <= KEmbeddedSize ? m_EmbeddedBuffer : new TType[buf_size])
     {}
     ~CFastBuffer() { if (m_Buffer != m_EmbeddedBuffer) delete[] m_Buffer; }
 
@@ -1018,9 +1221,9 @@ private:
 /// @sa NCBI_DEPRECATED_CTOR
 #define NCBI_DEPRECATED_CLASS NCBI_DEPRECATED_CTOR(class)
 
-END_NCBI_SCOPE
+END_NCBI_NAMESPACE;
 
-BEGIN_STD_SCOPE
+BEGIN_STD_NAMESPACE;
 
 template<class T1, class T2>
 inline
@@ -1053,22 +1256,19 @@ size_t ArraySize(const Element (&)[Size])
     return Size;
 }
 
+#ifdef NCBI_STRICT_GI
+template <> struct hash<ncbi::CStrictId64>
+{
+    size_t operator()(const ncbi::CStrictId64 & x) const
+    {
+        return hash<ncbi::TIntId>()((ncbi::TIntId)x);
+    }
+};
+#endif /* NCBI_STRICT_GI */
+
 #endif
 
-END_STD_SCOPE
-
-/// Definition of packed enum type, to save some memory
-#if defined(NCBI_COMPILER_MSVC)
-#  define NCBI_PACKED_ENUM_TYPE(type)  : type
-#  define NCBI_PACKED_ENUM_END()
-#elif (defined(NCBI_COMPILER_GCC) && NCBI_COMPILER_VERSION >= 400)  ||  defined(NCBI_COMPILER_ICC)
-#  define NCBI_PACKED_ENUM_TYPE(type)
-#  define NCBI_PACKED_ENUM_END()        __attribute__((packed))
-#else
-#  define NCBI_PACKED_ENUM_TYPE(type)
-#  define NCBI_PACKED_ENUM_END()
-#endif
-
+END_STD_NAMESPACE;
 
 /* @} */
 
