@@ -1,7 +1,7 @@
 #ifndef ANNOT_COLLECTOR__HPP
 #define ANNOT_COLLECTOR__HPP
 
-/*  $Id: annot_collector.hpp 439336 2014-06-27 16:08:17Z vasilche $
+/*  $Id: annot_collector.hpp 499903 2016-04-28 21:07:18Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -61,6 +61,8 @@ struct SAnnotObject_Index;
 class CSeq_entry_Info;
 class CSeq_annot_Info;
 class CSeq_annot_SNP_Info;
+class CSeq_annot_SortedIter;
+class CSeqTableInfo;
 struct SSNP_Info;
 struct SIdAnnotObjs;
 class CSeq_loc_Conversion;
@@ -79,7 +81,8 @@ public:
         fMapped_Product      = 1<<1,
         fMapped_Seq_point    = 1<<2,
         fMapped_Partial_from = 1<<3,
-        fMapped_Partial_to   = 1<<4
+        fMapped_Partial_to   = 1<<4,
+        fFromOtherTSE        = 1<<5
     };
 
     enum EMappedObjectType {
@@ -123,6 +126,9 @@ public:
     ENa_strand GetMappedStrand(void) const;
     void SetMappedStrand(ENa_strand strand);
 
+    bool IsFromOtherTSE(void) const;
+    void SetFromOtherTSE(bool from_other_tse = true);
+
     const CSeq_loc& GetMappedSeq_loc(void) const;
     const CSeq_id& GetMappedSeq_id(void) const;
     CSeq_loc_Conversion& GetMappedSeq_loc_Conv(void) const;
@@ -173,9 +179,15 @@ class NCBI_XOBJMGR_EXPORT CAnnotObject_Ref
 public:
     typedef CRange<TSeqPos> TRange;
     typedef Int4           TAnnotIndex;
-    enum {
-        kSNPTableBit    = 0x80000000,
-        kAnnotIndexMask = 0x7fffffff
+    enum EAnnotType {
+        fAnnot_NoAnnotInfo    = 1<<0,
+        fAnnot_SNPTable       = 1<<1,
+        fAnnot_SeqTable       = 1<<2,
+
+        eAnnot_Regular        = 0,
+        eAnnot_SNPTable       = fAnnot_NoAnnotInfo | fAnnot_SNPTable,
+        eAnnot_SeqTable       = fAnnot_SeqTable,
+        eAnnot_SortedSeqTable = fAnnot_NoAnnotInfo | fAnnot_SeqTable,
     };
 
     CAnnotObject_Ref(void);
@@ -185,15 +197,34 @@ public:
                      const CSeq_annot_Handle& annot_handle,
                      const SSNP_Info& snp_info,
                      CSeq_loc_Conversion* cvt);
+    CAnnotObject_Ref(const CSeq_annot_Handle& annot_handle,
+                     const CSeq_annot_SortedIter& iter,
+                     CSeq_loc_Conversion* cvt);
 
+    // the annotation has CAnnotObject_Info object and entry in annot index
     bool HasAnnotObject_Info(void) const;
+    // the annotation has CSeq_feat, CAnnotObject_Info, and annot index
     bool IsPlainFeat(void) const;
-    bool IsSNPFeat(void) const;
-    bool IsTableFeat(void) const;
+
+    // the feature is from parsed SNP table (GenBank SNP external annotations)
+    // it doesn't have corresponding CAnnotObject_Info and entry in annot index
+    bool IsSNPTableFeat(void) const;
+    // the feature is from Seq-table
+    // it may or may not have corresponding CAnnotObject_Info and annot index
+    bool IsAnySeqTableFeat(void) const;
+    // the feature is from pre-sorted Seq-table
+    // it doesn't have corresponding CAnnotObject_Info and entry in annot index
+    bool IsSortedSeqTableFeat(void) const;
+
+    // replaced with IsSNPTableFeat() and IsSortedSeqTableFeat()
+    NCBI_DEPRECATED bool IsSNPFeat(void) const;
+    // replaced with IsAnySeqTableFeat() and IsSortedSeqTableFeat()
+    NCBI_DEPRECATED bool IsTableFeat(void) const;
 
     const CSeq_annot_Handle& GetSeq_annot_Handle(void) const;
     const CSeq_annot_Info& GetSeq_annot_Info(void) const;
     const CSeq_annot_SNP_Info& GetSeq_annot_SNP_Info(void) const;
+    const CSeqTableInfo& GetSeqTableInfo(void) const;
     TAnnotIndex GetAnnotIndex(void) const;
 
     const CAnnotObject_Info& GetAnnotObject_Info(void) const;
@@ -208,6 +239,9 @@ public:
 
     CAnnotMapping_Info& GetMappingInfo(void) const;
 
+    bool IsFromOtherTSE(void) const;
+    void SetFromOtherTSE(bool from_other_tse = true);
+
     void ResetLocation(void);
     bool operator<(const CAnnotObject_Ref& ref) const; // sort by object
     bool operator==(const CAnnotObject_Ref& ref) const; // sort by object
@@ -219,8 +253,12 @@ private:
     friend class CAnnot_Collector;
 
     CSeq_annot_Handle          m_Seq_annot;   //  4 or  8
+    mutable CAnnotMapping_Info m_MappingInfo; // 20 or 32
     TAnnotIndex                m_AnnotIndex;  //  4 or  4
-    mutable CAnnotMapping_Info m_MappingInfo; // 16 or 20
+    EAnnotType                 m_AnnotType;   //  4 or  4
+    // total size:
+    //   32 B on 32-bit system (can be packed into 28 B)
+    //   48 B on 64-bit system (can be packed into 40 B)
 };
 
 
@@ -280,7 +318,7 @@ private:
     const TAnnotSet& GetAnnotSet(void) const;
     CScope& GetScope(void) const;
 
-    const SAnnotSelector& GetSelector(void);
+    const SAnnotSelector& GetSelector(void) const;
     bool CanResolveId(const CSeq_id_Handle& idh, const CBioseq_Handle& bh);
 
     void x_Initialize0(const SAnnotSelector& selector);
@@ -362,13 +400,20 @@ private:
                      const CTSE_Handle*     using_tse,
                      bool top_level = false);
 
+    CBioseq_Handle x_GetBioseqHandle(const CSeq_id_Handle& id,
+                                     bool top_level = false) const;
+    // returns if adaptive depth heuristics should be checked for a segment
+    bool x_CheckAdaptive(const CSeq_id_Handle& id) const;
+    bool x_CheckAdaptive(const CBioseq_Handle& bh) const;
+
     // Search annotations within tse before filtering by source location.
     // Called by: x_SearchLoc(), x_SearchMaster()
     // Calls: x_SearchTSE2()
     bool x_SearchTSE(const CTSE_Handle&    tse,
                      const CSeq_id_Handle& id,
                      const CHandleRange&   hr,
-                     CSeq_loc_Conversion*  cvt);
+                     CSeq_loc_Conversion*  cvt,
+                     bool check_adaptive);
 
     // Search annotations within tse after filtering by source location.
     // Called by: x_SearchTSE()
@@ -376,7 +421,8 @@ private:
     bool x_SearchTSE2(const CTSE_Handle&    tse,
                       const CSeq_id_Handle& id,
                       const CHandleRange&   hr,
-                      CSeq_loc_Conversion*  cvt);
+                      CSeq_loc_Conversion*  cvt,
+                      bool check_adaptive);
 
     // Called by: x_SearchTSE2()
     // Calls: x_SearchRange()
@@ -467,6 +513,7 @@ private:
     typedef SAnnotSelector::TMaxSearchSegments TMaxSearchSegments;
     CStopWatch              m_SearchTime;
     TMaxSearchSegments      m_SearchSegments;
+    bool                    m_FromOtherTSE;
 
     friend class CAnnotTypes_CI;
     friend class CMappedFeat;
@@ -730,6 +777,25 @@ void CAnnotMapping_Info::SetProduct(bool product)
 }
 
 
+inline
+bool CAnnotMapping_Info::IsFromOtherTSE(void) const
+{
+    return (m_MappedFlags & fFromOtherTSE) != 0;
+}
+
+
+inline
+void CAnnotMapping_Info::SetFromOtherTSE(bool from_other_tse)
+{
+    if ( from_other_tse ) {
+        m_MappedFlags |= fFromOtherTSE;
+    }
+    else {
+        m_MappedFlags &= ~fFromOtherTSE;
+    }
+}
+
+
 inline
 void CAnnotMapping_Info::SetAnnotObjectRange(const TRange& range, bool product)
 {
@@ -757,7 +823,7 @@ void CAnnotMapping_Info::Swap(CAnnotMapping_Info& info)
 
 inline
 CAnnotObject_Ref::CAnnotObject_Ref(void)
-    : m_AnnotIndex(0)
+    : m_AnnotIndex(0), m_AnnotType(eAnnot_Regular)
 {
 }
 
@@ -765,37 +831,56 @@ CAnnotObject_Ref::CAnnotObject_Ref(void)
 inline
 CAnnotObject_Ref::TAnnotIndex CAnnotObject_Ref::GetAnnotIndex(void) const
 {
-    return m_AnnotIndex & kAnnotIndexMask;
+    return m_AnnotIndex;
 }
 
 
 inline
 bool CAnnotObject_Ref::HasAnnotObject_Info(void) const
 {
-    return (m_AnnotIndex & kSNPTableBit) == 0;
+    return (m_AnnotType & fAnnot_NoAnnotInfo) == 0;
 }
 
 
 inline
 bool CAnnotObject_Ref::IsPlainFeat(void) const
 {
-    return (m_AnnotIndex & kSNPTableBit) == 0 &&
-        GetAnnotObject_Info().IsRegular();
+    return m_AnnotType == eAnnot_Regular;
+}
+
+
+inline
+bool CAnnotObject_Ref::IsSNPTableFeat(void) const
+{
+    return m_AnnotType == eAnnot_SNPTable;
+}
+
+
+inline
+bool CAnnotObject_Ref::IsSortedSeqTableFeat(void) const
+{
+    return m_AnnotType == eAnnot_SortedSeqTable;
+}
+
+
+inline
+bool CAnnotObject_Ref::IsAnySeqTableFeat(void) const
+{
+    return (m_AnnotType & fAnnot_SeqTable) != 0;
 }
 
 
 inline
 bool CAnnotObject_Ref::IsSNPFeat(void) const
 {
-    return (m_AnnotIndex & kSNPTableBit) != 0;
+    return IsSNPTableFeat();
 }
 
 
 inline
 bool CAnnotObject_Ref::IsTableFeat(void) const
 {
-    return (m_AnnotIndex & kSNPTableBit) == 0 &&
-        !GetAnnotObject_Info().IsRegular();
+    return IsAnySeqTableFeat();
 }
 
 
@@ -819,6 +904,9 @@ bool CAnnotObject_Ref::operator<(const CAnnotObject_Ref& ref) const
     if ( m_Seq_annot != ref.m_Seq_annot ) {
         return m_Seq_annot.OrderedBefore(ref.m_Seq_annot);
     }
+    if ( m_AnnotType != ref.m_AnnotType ) {
+        return m_AnnotType < ref.m_AnnotType;
+    }
     return m_AnnotIndex < ref.m_AnnotIndex;
 }
 
@@ -826,16 +914,16 @@ bool CAnnotObject_Ref::operator<(const CAnnotObject_Ref& ref) const
 inline
 bool CAnnotObject_Ref::operator==(const CAnnotObject_Ref& ref) const
 {
-    return (m_Seq_annot == ref.m_Seq_annot &&
-            m_AnnotIndex == ref.m_AnnotIndex);
+    return (m_AnnotIndex == ref.m_AnnotIndex &&
+            m_AnnotType == ref.m_AnnotType &&
+            m_Seq_annot == ref.m_Seq_annot);
 }
 
 
 inline
 bool CAnnotObject_Ref::operator!=(const CAnnotObject_Ref& ref) const
 {
-    return (m_Seq_annot != ref.m_Seq_annot ||
-            m_AnnotIndex != ref.m_AnnotIndex);
+    return !(*this == ref);
 }
 
 
@@ -846,6 +934,20 @@ CAnnotMapping_Info& CAnnotObject_Ref::GetMappingInfo(void) const
 }
 
 
+inline
+bool CAnnotObject_Ref::IsFromOtherTSE(void) const
+{
+    return m_MappingInfo.IsFromOtherTSE();
+}
+
+
+inline
+void CAnnotObject_Ref::SetFromOtherTSE(bool from_other_tse)
+{
+    m_MappingInfo.SetFromOtherTSE(from_other_tse);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // CAnnot_Collector
 /////////////////////////////////////////////////////////////////////////////
@@ -867,7 +969,7 @@ CScope& CAnnot_Collector::GetScope(void) const
 
 
 inline
-const SAnnotSelector& CAnnot_Collector::GetSelector(void)
+const SAnnotSelector& CAnnot_Collector::GetSelector(void) const
 {
     return *m_Selector;
 }
@@ -878,6 +980,7 @@ void CAnnotObject_Ref::Swap(CAnnotObject_Ref& ref)
 {
     m_Seq_annot.Swap(ref.m_Seq_annot);
     swap(m_AnnotIndex, ref.m_AnnotIndex);
+    swap(m_AnnotType, ref.m_AnnotType);
     m_MappingInfo.Swap(ref.m_MappingInfo);
 }
 
