@@ -1,7 +1,7 @@
 #ifndef SEQUENCE__HPP
 #define SEQUENCE__HPP
 
-/*  $Id: sequence.hpp 258061 2011-03-17 15:17:43Z rafanovi $
+/*  $Id: sequence.hpp 499498 2016-04-26 17:24:55Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -40,12 +40,14 @@
 #include <serial/objostr.hpp>
 
 #include <objects/seqfeat/SeqFeatData.hpp>
+#include <objects/seqfeat/Cdregion.hpp>
 #include <objects/seqloc/Na_strand.hpp>
 #include <objects/seqloc/Seq_interval.hpp>
 #include <objects/seqloc/Seq_loc.hpp>
-#include <objects/seq/seq_id_handle.hpp>
+#include <objects/seq/seq_id_mapper.hpp>
 #include <util/strsearch.hpp>
 #include <objmgr/feat_ci.hpp>
+#include <objmgr/bioseq_ci.hpp>
 #include <objmgr/scope.hpp>
 #include <objmgr/util/seq_loc_util.hpp>
 #include <objmgr/util/create_defline.hpp>
@@ -67,6 +69,7 @@ class CSeq_entry;
 class CSeq_entry_handle;
 class CGenetic_code;
 class CMolInfo;
+class CSeq_gap;
 
 BEGIN_SCOPE(sequence)
 
@@ -127,7 +130,7 @@ typedef int EGetIdType;
 /// Id type in the flags is ignored, only VerifyId and ThrowOnError
 /// flags are checked.
 NCBI_XOBJUTIL_EXPORT
-int GetGiForAccession(const string& acc,
+TGi GetGiForAccession(const string& acc,
                       CScope& scope,
                       EGetIdType flags = 0);
 
@@ -137,7 +140,7 @@ int GetGiForAccession(const string& acc,
 /// Id type in the flags is ignored, only VerifyId and ThrowOnError
 /// flags are checked.
 NCBI_XOBJUTIL_EXPORT
-string GetAccessionForGi(int           gi,
+string GetAccessionForGi(TGi           gi,
                          CScope&       scope,
                          EAccessionVersion use_version = eWithAccessionVersion,
                          EGetIdType flags = 0);
@@ -148,7 +151,7 @@ string GetAccessionForGi(int           gi,
 /// Id type in the flags is ignored, only VerifyId and ThrowOnError
 /// flags are checked.
 NCBI_XOBJUTIL_EXPORT
-int GetGiForId(const objects::CSeq_id& id,
+TGi GetGiForId(const objects::CSeq_id& id,
                CScope& scope,
                EGetIdType flags = 0);
 
@@ -216,6 +219,20 @@ NCBI_XOBJUTIL_EXPORT
 CSeq_id_Handle GetId(const CSeq_id_Handle& id, CScope& scope,
                      EGetIdType type = eGetId_Default);
 
+/// Return a selected ID type from a set of Seq-ids
+/// Arguments (except 'id') and behavior is the same as of
+/// GetId(const CSeq_id& id, ...).
+NCBI_XOBJUTIL_EXPORT
+CSeq_id_Handle GetId(const CBioseq::TId& id,
+                     EGetIdType type = eGetId_Default);
+
+/// Return a selected ID type from a Bioseq
+/// Arguments (except 'seq') and behavior is the same as of
+/// GetId(const CBioseq_Handle& seq, ...).
+NCBI_XOBJUTIL_EXPORT
+CSeq_id_Handle GetId(const CBioseq& seq,
+                     EGetIdType type = eGetId_Default);
+
 /* @} */
 
 
@@ -252,6 +269,7 @@ CSeq_id_Handle FindLatestSequence(const CSeq_id_Handle& idh,
 
 /** @name GetTitle
  * Get sequence's title (used in various flat-file formats.)
+ * Deprecated in favor of CDeflineGenerator.
  * @{
  */
 
@@ -266,9 +284,9 @@ enum EGetTitleFlags {
 };
 typedef int TGetTitleFlags;
 
-NCBI_XOBJUTIL_EXPORT
+NCBI_DEPRECATED NCBI_XOBJUTIL_EXPORT
 string GetTitle(const CBioseq_Handle& hnd, TGetTitleFlags flags = 0);
-NCBI_XOBJUTIL_EXPORT
+NCBI_DEPRECATED NCBI_XOBJUTIL_EXPORT
 bool GetTitle(const CBioseq& seq, string* title_ptr, TGetTitleFlags flags = 0);
 
 /* @} */
@@ -318,6 +336,10 @@ enum EBestFeatOpts {
     /// favor longer features over shorter features
     fBestFeat_FavorLonger = 0x04,
 
+    /// Pay no attention to strands when finding the best feat.  This may be
+    /// useful for, e.g., trans-spliced genes.
+    fBestFeat_IgnoreStrand = 0x08,
+
     /// default options: do everything
     fBestFeat_Defaults = 0
 };
@@ -332,6 +354,7 @@ typedef vector<TFeatScore> TFeatScores;
 // function, we allow plugins
 class CGetOverlappingFeaturesPlugin {
 public:
+    virtual ~CGetOverlappingFeaturesPlugin() {}
     virtual void processSAnnotSelector( 
         SAnnotSelector &sel ) = 0;
 
@@ -352,6 +375,7 @@ public:
 
     virtual void processMainLoop( 
         bool &shouldContinueToNextIteration,
+        CRef<CSeq_loc> &cleaned_loc_this_iteration,
         CRef<CSeq_loc> &candidate_feat_loc,
         EOverlapType &overlap_type_this_iteration,
         bool &revert_locations_this_iteration,
@@ -359,6 +383,14 @@ public:
         const CMappedFeat &feat,
         TSeqPos circular_length,
         SAnnotSelector::EOverlapType annot_overlap_type ) = 0;
+
+    virtual void postProcessDiffAmount( 
+        Int8 &cur_diff, 
+        CRef<CSeq_loc> &cleaned_loc_this_iteration, 
+        CRef<CSeq_loc> &candidate_feat_loc, 
+        CScope &scope, 
+        SAnnotSelector &sel, 
+        TSeqPos circular_length ) = 0;
 };
 
 /// Find all features overlapping the location. Features and corresponding
@@ -500,6 +532,10 @@ GetBestOverlappingFeat(const CSeq_feat& feat,
                        TBestFeatOpts opts = fBestFeat_Defaults,
                        CGetOverlappingFeaturesPlugin *plugin = NULL );
 
+NCBI_XOBJUTIL_EXPORT
+CConstRef<CSeq_feat> GetmRNAforCDS(const CSeq_feat& cds, CScope& scope);
+
+
 /// Get the best overlapping feature for a SNP (variation) feature
 /// @param snp_feat
 ///   SNP feature object
@@ -536,8 +572,16 @@ CConstRef<CSeq_feat> GetBestOverlapForSNP(const CSeq_feat& snp_feat,
 
 
 /// Convenience functions for popular overlapping types
+enum ETransSplicing {
+    eTransSplicing_No = 0,
+    eTransSplicing_Yes,
+    eTransSplicing_Auto ///< Ignore overlap strand if the source location
+                        ///< has mixed/both strand.
+};
 NCBI_XOBJUTIL_EXPORT
-CConstRef<CSeq_feat> GetOverlappingGene(const CSeq_loc& loc, CScope& scope);
+CConstRef<CSeq_feat> GetOverlappingGene(
+    const CSeq_loc& loc, CScope& scope,
+    ETransSplicing eTransSplicing = eTransSplicing_Auto);
 
 
 NCBI_XOBJUTIL_EXPORT
@@ -606,6 +650,10 @@ CBioseq_Handle GetParentForPart(const CBioseq_Handle& part);
 /// a CException if there is no org-ref associated with the sequence
 NCBI_XOBJUTIL_EXPORT
 const COrg_ref& GetOrg_ref(const CBioseq_Handle& handle);
+/// Return the pointer to org-ref associated with a given sequence
+/// or null if there is no org-ref associated with the sequence
+NCBI_XOBJUTIL_EXPORT
+const COrg_ref* GetOrg_refOrNull(const CBioseq_Handle& handle);
 
 /// return the tax-id associated with a given sequence.  This will return 0
 /// if no tax-id can be found.
@@ -614,11 +662,17 @@ int GetTaxId(const CBioseq_Handle& handle);
 
 /// Retrieve the MolInfo object for a given bioseq handle.  If the supplied
 /// sequence does not have a MolInfo associated with it, this will return NULL
+NCBI_XOBJUTIL_EXPORT
+const CMolInfo* GetMolInfo(const CBioseq& bioseq);
+
 NCBI_XOBJUTIL_EXPORT
 const CMolInfo* GetMolInfo(const CBioseq_Handle& handle);
 
 /// Retrieve the BioSource object for a given bioseq handle.  If the supplied
 /// sequence does not have a MolInfo associated with it, this will return NULL
+NCBI_XOBJUTIL_EXPORT
+const CBioSource* GetBioSource(const CBioseq& bioseq);
+
 NCBI_XOBJUTIL_EXPORT
 const CBioSource* GetBioSource(const CBioseq_Handle& handle);
 
@@ -628,59 +682,19 @@ const CBioSource* GetBioSource(const CBioseq_Handle& handle);
 ///   return the bioseq
 /// location referes to multiple bioseqs:
 ///   if parts of a segmentd bioseq, returns the segmentd bioseq.
-///   otherwise, return the first bioseq that could be found (first localy then,
-///   if flag is eGetBioseq_All, remote)
+///   otherwise, return the first bioseq that could be found
+///   (first localy then, if flag is eGetBioseq_All, remote)
 NCBI_XOBJUTIL_EXPORT
 CBioseq_Handle GetBioseqFromSeqLoc(const CSeq_loc& loc, CScope& scope,
     CScope::EGetBioseqFlag flag = CScope::eGetBioseq_Loaded);
 
 
-/// Create a constructed bioseq from a location
-/// This function will construct a bioseq from a given location, using a set of
-/// options to control how the bioseq should be represented.  Options here
-/// include:
-///  - create a far-pointing delta-seq or a raw sequence
-///      - specify the depth of recursion for the delta bioseq
-///  - copy all bioseq descriptors
-/// @param bsh
-///   Source bioseq handle
-/// @param from
-///   Starting position of sequence
-/// @param to
-///   Ending position of sequence
-/// @param new_seq_id
-///   Seq-id for the newly created sequence
-/// @param opts
-///   The set of flags controlling how the sequence should be created
-/// @param delta_seq_level
-///   Level of recursion from which to pull the delta-seq components
-///   Level 0 = use the components in the source sequence;
-///   Level 1 = use the components from the components in the source sequence
-/// @return
-///   Newly created bioseq
-
-enum ECreateBioseqFlags {
-    //< Create a delta sequence.  If this is not present, a raw sequence
-    //< is created
-    fBioseq_CreateDelta = 0x01,
-
-    //< Copy all descriptors into the new bioseq
-    fBioseq_CopyDescriptors = 0x02,
-
-    //< Project all annotations onto the new bioseq, and
-    //< create a new annotation for these
-    fBioseq_CopyAnnot = 0x04,
-
-    fBioseq_Defaults = fBioseq_CreateDelta
-};
-typedef int TCreateBioseqFlags;
-
+/// Return protein name from corresponding Prot-ref feature.
+/// Throws exception if the sequence is not a protein,
+/// or if there is no unambiguously best Prot-ref feature,
+/// or if the feature doesn't return non-empty label.
 NCBI_XOBJUTIL_EXPORT
-CRef<CBioseq> CreateBioseqFromBioseq(const CBioseq_Handle& bsh,
-                                     TSeqPos from, TSeqPos to,
-                                     const CSeq_id_Handle& new_seq_id,
-                                     TCreateBioseqFlags opts = fBioseq_Defaults,
-                                     int delta_seq_level = 1);
+string GetProteinName(const CBioseq_Handle& seq);
 
 
 /* @} */
@@ -712,20 +726,35 @@ END_SCOPE(sequence)
 class NCBI_XOBJUTIL_EXPORT CFastaOstream {
 public:
     enum EFlags {
-        fAssembleParts   = 0x01, ///< assemble FAR delta sequences
-        fInstantiateGaps = 0x02, ///< with Ns or Xs as appropriate, vs. one -
-        fSuppressRange   = 0x04, ///< never include location details in defline
-        fReverseStrand   = 0x08, ///< flip the (implicit) location
-        fKeepGTSigns     = 0x10, ///< don't convert '>' to '_' in title
-        fMapMasksUp      = 0x20, ///< honor masks specified at a lower level
-        fMapMasksDown    = 0x40, ///< honor masks specified at a higher level
-        fNoExpensiveOps  = 0x80, ///< don't try too hard to find titles
+        fAssembleParts      = 0x0001, ///< assemble FAR delta sequences; on by dflt
+        fInstantiateGaps    = 0x0002, ///< honor specifed gap mode; on by default
+        fSuppressRange      = 0x0004, ///< never include location details in defline
+        fReverseStrand      = 0x0008, ///< flip the (implicit) location
+        fKeepGTSigns        = 0x0010, ///< don't convert '>' to '_' in title
+        fMapMasksUp         = 0x0020, ///< honor masks specified at a lower level
+        fMapMasksDown       = 0x0040, ///< honor masks specified at a higher level
+        fNoExpensiveOps     = 0x0080, ///< don't try too hard to find titles
+        fShowModifiers      = 0x0100, ///< show key-value pair modifiers (e.g. "[organism=Homo sapiens]")
+        fNoDupCheck         = 0x0200, ///< skip check for duplicate sequence IDs
+        fShowGapModifiers   = 0x0400, ///< show gap key-value pair modifiers (e.g. "[linkage-evidence=map;strobe]"). Only works if gap mode is eGM_count.
+        fKeepUnknGapNomLen  = 0x0800, ///< Keep unknown gap's nominal length.  That is, when a gap has an unknown length but nominal length, use that instead of just making it 100.
+        fShowGapsOfSizeZero = 0x1000, ///< Use this to show gaps of size zero as a lone hyphen at the end of a line.
         // historically misnamed as eFlagName
         eAssembleParts   = fAssembleParts,
         eInstantiateGaps = fInstantiateGaps
     };
     typedef int TFlags; ///< binary OR of EFlags
 
+    /// How to represent gaps with fInstantiateGaps enabled, as it is
+    /// by default.  (Disabling fInstantiateGaps is equivalent to
+    /// requesting eGM_one_dash.)
+    enum EGapMode {
+        eGM_one_dash, ///< A single dash, followed by a line break.
+        eGM_dashes,   ///< Multiple inline dashes.
+        eGM_letters,  ///< Multiple inline Ns or Xs as appropriate (default).
+        eGM_count     ///< >?N or >?unk100, as appropriate.
+    };
+
     CFastaOstream(CNcbiOstream& out);
     virtual ~CFastaOstream();
 
@@ -740,11 +769,13 @@ public:
                                const CSeq_loc* location = 0,
                                const string& custom_title = kEmptyStr);
     virtual void WriteSequence(const CBioseq_Handle& handle,
-                               const CSeq_loc* location = 0);
+                               const CSeq_loc* location = 0, 
+                               CSeq_loc::EOpFlags merge_flags=CSeq_loc::fMerge_All);
 
     /// These versions may set up a temporary object manager scope
     /// In the common case of a raw bioseq, no scope is needed
-    void Write(const CSeq_entry& entry, const CSeq_loc* location = 0);
+    void Write(const CSeq_entry& entry, const CSeq_loc* location = 0,
+               bool no_scope = false);
     void Write(const CBioseq&    seq,   const CSeq_loc* location = 0,
                bool no_scope = false,   const string& custom_title = kEmptyStr);
     void WriteTitle(const CBioseq& seq, const CSeq_loc* location = 0,
@@ -770,25 +801,71 @@ public:
 
     /// Other parameters...
     TSeqPos GetWidth   (void) const    { return m_Width;   }
-    void    SetWidth   (TSeqPos width) { m_Width = width;  }
+    void    SetWidth   (TSeqPos width);
     TFlags  GetAllFlags(void) const    { return m_Flags;   }
     void    SetAllFlags(TFlags flags)  { m_Flags = flags;  }
     void    SetFlag    (EFlags flag)   { m_Flags |=  flag; }
     void    ResetFlag  (EFlags flag)   { m_Flags &= ~flag; }
+    void    SetGapMode (EGapMode mode) { m_GapMode = mode; }
+    EGapMode GetGapMode(void) const    { return m_GapMode; }
 
-private:
+    /// This indicates the text of the modifiers of a gap.
+    struct NCBI_XOBJUTIL_EXPORT SGapModText {
+        /// String representing the gap type.
+        /// Examples: "short_arm", "telomere", etc.
+        string gap_type;
+        /// A vector representing the linkage-evidences of the gap.
+        /// Example linkage-evidences: "align genus", "within clone", etc.
+        vector<string> gap_linkage_evidences;
+
+        // more fields may be added in the future.
+
+        /// This will write the modifiers in
+        /// FASTA format.  (example: "[gap-type=short_arm]")
+        void WriteAllModsAsFasta( CNcbiOstream & out ) const;
+    };
+
+    /// Given a CSeq_gap object, this outputs the
+    /// Gap information
+    ///
+    /// @param seq_gap
+    ///   This is the seq_gap information we're using to figure out
+    ///   the gap mod text
+    /// @param out_gap_mod_text
+    ///   This holds the result.
+    static void
+    GetGapModText(
+        const CSeq_gap & seq_gap,
+        SGapModText & out_gap_mod_text );
+
+protected:
     CNcbiOstream&       m_Out;
     auto_ptr<sequence::CDeflineGenerator> m_Gen;
+
+    virtual void x_WriteSeqIds    ( const CBioseq& bioseq,
+                                    const CSeq_loc* location);
+    virtual void x_WriteModifiers ( const CBioseq_Handle & handle );
+    virtual void x_WriteSeqTitle  ( const CBioseq& bioseq,
+                                    CScope* scope,
+                                    const string& custom_title);
+
+private:
     CConstRef<CSeq_loc> m_SoftMask;
     CConstRef<CSeq_loc> m_HardMask;
     TSeqPos             m_Width;
     TFlags              m_Flags;
+    EGapMode            m_GapMode;
+    TSeq_id_HandleSet   m_PreviousWholeIds;
+    // avoid recomputing for every sequence
+    typedef AutoPtr<char, ArrayDeleter<char> > TCharBuf;
+    TCharBuf            m_Dashes, m_LC_Ns, m_LC_Xs, m_UC_Ns, m_UC_Xs;
 
-    void x_WriteSeqIds  (const CBioseq& bioseq,
-                         const CSeq_loc* location);
-    void x_WriteSeqTitle(const CBioseq& bioseq,
-                         CScope* scope,
-                         const string& custom_title);
+    sequence::CDeflineGenerator::TUserFlags x_GetTitleFlags(void) const;
+
+    void x_PrintStringModIfNotDup(
+        bool *seen, const CTempString & key, const CTempString & value );
+    void x_PrintIntModIfNotDup(
+        bool *seen, const CTempString & key, const int value );
 
     CConstRef<CSeq_loc> x_MapMask(CSeq_loc_Mapper& mapper, const CSeq_loc& mask,
                                   const CSeq_id* base_seq_id, CScope* scope);
@@ -971,6 +1048,12 @@ public:
     static CRef<CBioseq> TranslateToProtein(const CSeq_feat& cds,
                                               CScope& scope);
 
+    static bool ChangeDeltaProteinToRawProtein(CRef<CBioseq> protein);
+
+    /// Find "best" frame for a coding region. "Best" frame has no
+    /// internal stop codons.
+    static CCdregion::EFrame FindBestFrame(const CSeq_feat& cds, CScope& scope);
+
 };
 
 
@@ -1137,8 +1220,447 @@ private:
 };  //  end of CSeqSearch
 
 
+/// This trims ambiguous bases from the start and/or end of 
+/// sequences, using customizable rules.
+class NCBI_XOBJUTIL_EXPORT CSequenceAmbigTrimmer : public CObject
+{
+public:
+
+    /// This enum is used to set what is meant by "ambiguous".
+    enum EMeaningOfAmbig {
+        /// Here, only N for nucleotides and X for amino acids is considered
+        /// ambiguous.
+        eMeaningOfAmbig_OnlyCompletelyUnknown, 
+        /// Here, anything that's not certain is considered
+        /// ambiguous.  That is, anything but A, C, G, T for nucleotides,
+        /// and most amino acids except, for example, B (which can be 
+        /// aspartic acid or asparagine), X (completely ambiguous), etc.
+        eMeaningOfAmbig_AnyAmbig,
+    };
+
+    enum EFlags {
+        fFlags_DoNotTrimBeginning = (1 << 0), ///< 0x01 ("Beginning" as defined by CSeqVector)
+        fFlags_DoNotTrimEnd       = (1 << 1), ///< 0x02 ("End" as defined by CSeqVector)
+
+        fFlags_DoNotTrimSeqGap    = (1 << 2), ///< 0x04 (Seq-gaps are not considered trimmable if this flag is set, only letter gaps (e.g. N's for nucs))
+
+        // we might support this in the future
+        // fFlags_TrimAnnot         = (1 << 3)   ///< 0x08 (Trim annots based on trimmed bioseq location)
+    };
+    typedef int TFlags;
+
+    /// For example, if bases_to_check is 10 and max_bases_allowed_to_be_ambig
+    /// is 5, then on each iteration we check the 10 terminal bases and
+    /// trim off those 10 if there are more than 5 ambiguous bases there.
+    struct NCBI_XOBJUTIL_EXPORT STrimRule {
+        TSignedSeqPos bases_to_check;
+        TSignedSeqPos max_bases_allowed_to_be_ambig;
+    };
+    /// Multiple STrimRules are allowed, which are applied from 
+    /// smallest bases_to_check to largest bases_to_check, and 
+    /// redundant rules are automatically removed.  When a rule is applied,
+    /// we start over at the first sorted rule again.
+    typedef vector<STrimRule> TTrimRuleVec;
+
+    /// This returns a reasonable default for trimming rules.
+    static const TTrimRuleVec & GetDefaultTrimRules(void);
+
+    /// This sets up the parameters for how this trimmer will act
+    ///
+    /// @param eMeaningOfAmbig
+    ///   This indicates exactly what ambiguous means (e.g. just "N" or
+    ///   do all ambiguous symbols count? )
+    /// @param fFlags
+    ///   miscellaneous parameters to control this.  See TFlags.
+    /// @param vecTrimRules
+    ///   This indicates how trimming will occur.  See TTrimRuleVec.
+    /// @param uMinSeqLen
+    ///   Trimming tries to halt if the sequence becomes smaller than this size.
+    ///   It is possible for the resulting sequence to be below the
+    ///   uMinSeqLen size (or even trimmed to nothing), but the trimmer
+    ///   will at least <i>try</i> not to do that.
+    CSequenceAmbigTrimmer(
+        EMeaningOfAmbig eMeaningOfAmbig,
+        TFlags fFlags = 0,
+        const TTrimRuleVec & vecTrimRules = GetDefaultTrimRules(),
+        TSignedSeqPos uMinSeqLen = 50 );
+
+    /// Do-nothing destructor just to allow inheritance.
+    virtual ~CSequenceAmbigTrimmer() { }
+
+    /// This indicates what happened with the trim.
+    /// Error states are indicated by an exception, not EResult.
+    enum EResult {
+        /// Bioseq is now trimmed.
+        eResult_SuccessfullyTrimmed,
+
+        /// Bioseq is left unchanged because it did not need to be trimmed
+        /// at all.  This is NOT an error.
+        eResult_NoTrimNeeded
+    };
+
+    /// This trims the given bioseq, using params
+    /// set in the CSequenceAmbigTrimmer constructor.  It will properly
+    /// handle the annots and descs inside the bioseq, too, if requested.
+    ///
+    /// @param bioseq_handle
+    ///   The bioseq to trim.
+    /// @return
+    ///   This returns how the trimming went.  On error, an exception
+    ///   is thrown and the bioseq may be in an undefined state.
+    virtual EResult DoTrim( CBioseq_Handle &bioseq_handle );
+
+protected:
+    /// This holds the current interpretation for "ambiguous".  For example,
+    /// it indicates whether just 'N' is ambiguous or if any non-ACGT
+    /// letter is ambiguous.  Works for amino acids, too (e.g. 'X' for
+    /// completely unknown, etc.)
+    EMeaningOfAmbig m_eMeaningOfAmbig;
+    /// This holds the flags that affect the behavior of this class.
+    TFlags          m_fFlags;
+    /// This holds the trimming rules that will be applied.
+    /// It should be normalized by the constructor 
+    /// to eliminate dups and to sort it from least to most bases.
+    TTrimRuleVec    m_vecTrimRules;
+    /// When the bioseq gets trimmed down to less than this size,
+    /// we halt the trimming.
+    TSignedSeqPos         m_uMinSeqLen;
+
+    /// Test if a given flag is set.
+    bool x_TestFlag(TFlags fFlag) {
+        return ( ( m_fFlags & fFlag ) != 0 );
+    }
+
+    /// This prepares the vector of trimming rules to be used
+    /// by the trimming algorithm. For example, it eliminate duplicates
+    /// and puts the rules in the correct order.
+    /// 
+    /// @param vecTrimRules
+    ///   Input and output.
+    virtual void x_NormalizeVecTrimRules( TTrimRuleVec & vecTrimRules );
+
+    /// The bioseq is trimmed to size 0.
+    ///
+    /// @param bioseq_handle
+    ///   The bioseq to trim to nothing.
+    /// @returns
+    ///   Works just like the DoTrim return value.
+    virtual EResult x_TrimToNothing( CBioseq_Handle &bioseq_handle );
+
+    // below this point, left/right means positions going numerically upward,
+    // but start/end is relative to direction. That is, 
+    // a negative direction would imply end &lt;= start.
+
+    /// This returns the last good base that won't be trimmed
+    /// (note: last really means "first" when we're starting from the end)
+    ///
+    /// @param seqvec
+    ///   This lets us explore the Bioseq to find out where to trim.
+    /// @param iStartPosInclusive_arg
+    ///   This is the where we start our trimming.  Depending on
+    ///   direction, this could be &lt; or &gt; iEndPosInclusive_arg.
+    /// @param iEndPosInclusive_arg
+    ///   This is where the trimming ends (inclusive).  Analogous to
+    ///   iStartPosInclusive_arg.
+    /// @param iTrimDirection
+    ///   1 to trim from left to right, -1 to trim from right to left.
+    /// @return
+    ///   The last good base (remember: last means "lower number" when we're
+    ///   checking from the end).  If trimming would trim off the entire
+    ///   sequence, it returns a position past the end of the sequence.
+    virtual TSignedSeqPos x_FindWhereToTrim(
+        const CSeqVector & seqvec,
+        const TSignedSeqPos iStartPosInclusive_arg,
+        const TSignedSeqPos iEndPosInclusive_arg,
+        TSignedSeqPos iTrimDirection );
+
+    /// This adjusts in_out_uStartOfGoodBasesSoFar if we're at
+    /// a CSeqMap gap.  It does not notice ambiguous bases that
+    /// are inside a normal sequence.
+    ///
+    /// @param seqvec
+    ///   This is used to access information about the sequence.
+    /// @param in_out_uStartOfGoodBasesSoFar
+    ///   This is the start of where we check for a gap.
+    ///   It will be updated to be past the gap, if a gap is found.
+    /// @param in_out_uRightmostGoodBaseSoFar
+    ///   Analogous to in_out_uLeftmostGoodBaseSoFar.  It's inclusive.
+    /// @param uEndOfGoodBasesSoFar
+    ///   This limits how far this function may search (inclusive) 
+    ///   when looking for
+    ///   the end of a gap segment.
+    /// @param iTrimDirection
+    ///   1 to trim from left to right, -1 to trim from right to left.
+    /// @param uChunkSize
+    ///   The gap size that we chop off must be a multiple of uChunkSize.
+    ///   We will chop off less if we would go more than 1 past the
+    ///   uEndOfGoodBasesSoFar.
+    ///   A uChunkSize of 1 means no chunking for obvious math reasons.
+    virtual void x_EdgeSeqMapGapAdjust( 
+        const CSeqVector & seqvec,
+        TSignedSeqPos & in_out_uStartOfGoodBasesSoFar,
+        const TSignedSeqPos uEndOfGoodBasesSoFar,
+        const TSignedSeqPos iTrimDirection,
+        const TSignedSeqPos uChunkSize );
+
+    /// This holds the output of x_CountAmbigInRange
+    struct NCBI_XOBJUTIL_EXPORT SAmbigCount : public CObject {
+        SAmbigCount(const TSignedSeqPos iTrimDirection) :
+            num_ambig_bases(0),
+            pos_after_last_gap( 
+                (iTrimDirection > 0) 
+                ? numeric_limits<TSignedSeqPos >::max()
+                : numeric_limits<TSignedSeqPos >::min() )
+            { }
+
+        /// the number of ambiguous bases found in the range
+        /// supplied to x_CountAmbigInRange
+        TSignedSeqPos num_ambig_bases;
+        /// Inclusive. This is far past the end if the whole range
+        /// is ambiguous. 
+        TSignedSeqPos pos_after_last_gap;
+    };
+
+    /// This counts the number of ambiguous bases in the range
+    /// [leftmost_pos_to_check, rightmost_pos_to_check].  Note that
+    /// rightmost_pos_to_check is inclusive.
+    ///
+    /// @param out_result
+    ///   This will store the result.  Pass in a struct initialized
+    ///   by the default constructor.
+    /// @param seqvec
+    ///   This is used to get the bases.
+    /// @param iStartPosInclusive
+    ///   This is where we start our count.
+    /// @param iEndPosInclusive
+    ///   This is where we end our count.  Note that it can be &lt; or
+    ///   &gt; iStartPosInclusive, depending on trim direction.
+    /// @param iTrimDirection
+    ///   1 to trim from left to right, -1 to trim from right to left.
+    virtual void x_CountAmbigInRange(
+        SAmbigCount & out_result,
+        const CSeqVector & seqvec,
+        const TSignedSeqPos iStartPosInclusive_arg,
+        const TSignedSeqPos iEndPosInclusive_arg,
+        const TSignedSeqPos iTrimDirection );
+
+    /// This returns the (inclusive) position at the beginning of the
+    /// segment.
+    ///
+    /// @param segment
+    ///   This is the segment we're trying to find the beginning of.
+    /// @param iTrimDirection
+    ///   This is which direction in which we're trimming.  The beginning
+    ///   will be in the opposite direction.
+    /// @return
+    ///   This returns the (inclusive) position at the beginning of the given
+    ///   segment. As always,
+    ///   the definition of "beginning" depends on iTrimDirection.
+    TSignedSeqPos x_SegmentGetBeginningInclusive(
+        const CSeqMap_CI & segment,
+        const TSignedSeqPos iTrimDirection ) 
+    {
+        // symmetrical
+        return x_SegmentGetEndInclusive( segment, -iTrimDirection );
+    }
+
+    /// This returns the (inclusive) position at the end of the
+    /// segment currently at iStartPosInclusive_arg.
+    ///
+    /// @param segment
+    ///   This is the segment we're trying to find the end of.
+    /// @param iTrimDirection
+    ///   This is which direction in which we're trimming.  The end
+    ///   of the segment will be found by looking in that direction.
+    /// @return
+    ///   This returns the (inclusive) position at the end of the given
+    ///   segment. The definition of "end" depends on iTrimDirection.
+    TSignedSeqPos x_SegmentGetEndInclusive(
+        const CSeqMap_CI & segment,
+        const TSignedSeqPos iTrimDirection );
+
+    /// Returns the "next" segment.  The definition of "next"
+    /// depends on iTrimDirection
+    ///
+    /// @param in_out_segment
+    ///   Caller gives the current CSeqMap_CI, which will be
+    ///   returned adjusted in the trim direction.
+    /// @param iTrimDirection
+    ///   The direction in which to increment.  1 means normal incrementing
+    ///   and -1 really means decrementing.
+    /// @return
+    ///   Reference to in_out_segment_it.
+    CSeqMap_CI & x_SeqMapIterDoNext(
+        CSeqMap_CI & in_out_segment_it,
+        const TSignedSeqPos iTrimDirection );
+
+    void x_SliceBioseq( 
+        TSignedSeqPos leftmost_good_base, 
+        TSignedSeqPos rightmost_good_base,
+        CBioseq_Handle & bioseq_handle );
+
+    // For each letter of the alphabet, returns whether or not it's
+    // ambiguous. Index 0 is 'A', index 1 is 'B', etc.
+    typedef bool TAmbigLookupTable[26];
+    TAmbigLookupTable m_arrNucAmbigLookupTable;
+    TAmbigLookupTable m_arrProtAmbigLookupTable;
+};
+
+/// This iterates over the runs of Ns of each sequence
+class NCBI_XOBJUTIL_EXPORT CBioseqGaps_CI : public CObject
+{
+public:
+
+    /// The params that control the behavior of CBioseqGaps_CI
+    struct NCBI_XOBJUTIL_EXPORT Params {
+        /// Default ctor gives params which are usually reasonable.
+        Params(void) : max_gap_len_to_ignore(10),
+            max_num_gaps_per_seq( numeric_limits<TSeqPos>::max() ),
+            max_num_seqs( numeric_limits<TSeqPos>::max() ),
+            mol_filter(CSeq_inst::eMol_not_set),
+            level_filter(CBioseq_CI::eLevel_All)
+        {
+        }
+
+        /// We completely ignore any gaps we find that have this
+        /// number of bases or fewer.
+        TSeqPos                       max_gap_len_to_ignore;
+        /// We only return up to this many gaps for each sequence
+        TSeqPos                       max_num_gaps_per_seq;
+        /// We only return gaps on up to this many sequences.
+        TSeqPos                       max_num_seqs;
+
+        /// CSeq_inst::eMol_na to only look at gaps on nucleotide
+        /// sequences.  CSeq_inst::eMol_aa to only look at gaps
+        /// on amino acid sequences.
+        /// CSeq_inst::eMol_not_set to avoid filtering.
+        CSeq_inst::EMol               mol_filter;
+        /// Works like the level filter in CBioseq_CI
+        CBioseq_CI::EBioseqLevelFlag  level_filter;
+    };
+
+    /// This constructor initializes the iterator.
+    ///
+    /// @param entry_h
+    ///   This will iterate over all descendents of this entry.
+    /// @param params
+    ///   Controls the behavior of the iterator.  If not specified,
+    ///   a reasonable default will be used.
+    CBioseqGaps_CI(
+        const CSeq_entry_Handle & entry_h,
+        const Params & params = Params() );
+
+    /// Move the iterator forward to next gap 
+    /// (or the end, if there are no more to return)
+    CBioseqGaps_CI & operator ++ (void) { x_Next(); return *this; }
+
+    DECLARE_OPERATOR_BOOL(m_bioseq_CI);
+
+    /// This indicates the state of the iterator right now.
+    /// This structure is undefined if the iterator
+    /// has reached the end, though the caller probably
+    /// won't be able to access it anyway since x_GetCurrent
+    /// will throw an exception in that case.
+    struct NCBI_XOBJUTIL_EXPORT SCurrentGapInfo {
+        /// Constructor initializes to state that it
+        /// should be when the iterator first starts.
+        SCurrentGapInfo(void) : num_seqs_seen_so_far(0),
+            start_pos(0),
+            length(0),
+            num_gaps_seen_so_far_on_this_seq(0) { }
+
+        /// The seq-id that this gap is on.
+        CSeq_id_Handle  seq_id;
+        /// This indicates how many sequences we've seen so far,
+        /// including the one we're currently on.
+        /// For example, 3 means we're on the 3rd sequence to
+        /// contain a relevant gap.
+        size_t          num_seqs_seen_so_far; 
+
+        /// the 0-based position at which the current gap starts
+        /// on the current sequence.
+        TSeqPos         start_pos;
+        /// the length of the current gap
+        TSeqPos         length;
+        /// how many gaps we've seen so far on this sequence.
+        /// For example, 2 would mean we're currently on
+        /// the second relevant gap on this sequence.
+        size_t          num_gaps_seen_so_far_on_this_seq;
+    };
+
+    /// Get information about the gap we're currently on.
+    const SCurrentGapInfo & operator*(void) const {
+        return x_GetCurrent();
+    }
+
+    /// Get information about the gap we're currently on.
+    const SCurrentGapInfo * operator ->(void) const {
+        return &x_GetCurrent();
+    }
+
+protected:
+    /// This points to the bioseq we're currently on.
+    /// When this iterator becomes invalid, that means this
+    /// CBioseqGaps_CI is invalid, too.
+    CBioseq_CI  m_bioseq_CI;
+    /// This indicates information about the gap we're currently on.
+    SCurrentGapInfo    m_infoOnCurrentGap;
+    /// This holds the params the caller gave when this
+    /// object was initially created.
+    Params      m_Params;
+
+    /// This gives info on the gap we're currently on.
+    /// Throws if this iterator has finished.
+    virtual const SCurrentGapInfo & x_GetCurrent(void) const;
+
+    /// This moves this iterator to the next relevant gap.
+    /// Throws if this iterator has finished.
+    virtual void x_Next(void);
+
+    /// This advances m_bioseq_CI although it
+    /// has extra logic to terminate m_bioseq_CI
+    /// if we've exceeded the number of bioseqs we can look for.
+    virtual void x_NextBioseq(void);
+
+    /// This indicates what happened when we tried to run
+    /// x_FindNextGapOnBioseq.
+    enum EFindNext {
+        /// No more relevant gaps were found on this bioseq.  The other output
+        /// parameters will be in an undefined state.
+        eFindNext_NotFound,
+        /// Another relevant gap was found, and the output parameters are 
+        /// filled in to represent information about it.
+        eFindNext_Found
+    };
+
+    /// This finds the next gap on the bioseq, starting at given pos.
+    ///
+    /// @param bioseq_h
+    ///   the bioseq on which we're seeking the next relevant gap.
+    /// @param pos_to_start_looking
+    ///   This is the position on bioseq_h to start looking for a
+    ///   relevant gap.
+    /// @param out_pos_of_gap
+    ///   If a gap is found, this holds the 0-based position of the
+    ///   start of that gap.  This is undefined if no gap was found.
+    /// @param out_len_of_gap
+    ///   If a gap is found, this holds the length of the
+    ///   gap.  This is undefined if no gap was found.
+    /// @return
+    ///   This indicates whether or not a relevant gap was found.
+    virtual EFindNext x_FindNextGapOnBioseq(
+        const CBioseq_Handle & bioseq_h, 
+        const TSeqPos pos_to_start_looking,
+        TSeqPos & out_pos_of_gap,
+        TSeqPos & out_len_of_gap ) const;
+};
+
 /* @} */
 
+/// Reverse complement a Bioseq in place.
+/// If delta sequence, will also need to reverse order of segments
+void NCBI_XOBJUTIL_EXPORT ReverseComplement(CSeq_inst& seq, CScope* scope);
+
 
 END_SCOPE(objects)
 END_NCBI_SCOPE
