@@ -1,4 +1,4 @@
-/*  $Id: align_format_util.hpp 488861 2016-01-06 15:45:07Z fongah2 $
+/*  $Id: align_format_util.hpp 509004 2016-08-02 17:32:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -63,9 +63,10 @@ BEGIN_SCOPE(align_format)
 ///class info
 static const char kClassInfo[] = "class=\"info\"";
 
+static const char kDefaultProtocol[] = "https:";
 ///entrez
 // .ncbirc alias: ENTREZ
-static const char kEntrezUrl[] = "<a title=\"Show report for <@acc@>\" <@cssInf@>href=\"//www.ncbi.nlm.nih.gov/<@db@>/<@gi@>?report=genbank&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\" <@target@>>";
+static const char kEntrezUrl[] = "<a title=\"Show report for <@acc@>\" <@cssInf@>href=\"<@protocol@>//www.ncbi.nlm.nih.gov/<@db@>/<@gi@>?report=genbank&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\" <@target@>>";
 
 //.ncbirc alias: ENTREZ_TM
 static const char kEntrezTMUrl[] = "//www.ncbi.nlm.nih.gov/<@db@>/<@gi@>?report=genbank&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>";
@@ -466,6 +467,16 @@ public:
         eDbTypeNotSet
     };
 
+    //Formatting flag for adding spaces
+    enum SpacesFormatingFlag {        
+        eSpacePosToCenter = (1 << 0),       ///place the param in the center of the string
+        eSpacePosAtLineStart = (1 << 1),    ///add spaces at the begining of the string
+        eSpacePosAtLineEnd = (1 << 2),      ///add spaces at the end of the string  
+        eAddEOLAtLineStart = (1 << 3),      ///add EOL at the beginning of the string  
+        eAddEOLAtLineEnd =  (1 << 4)        ///add EOL at the end of the string  
+    };
+
+
     ///Output blast errors
     ///@param error_return: list of errors to report
     ///@param error_post: post to stderr or not
@@ -1092,7 +1103,29 @@ public:
     ///
     ///<@tmplParamName@> is replaced by templParamVal
     static string MapTemplate(string inpString,string tmplParamName,string templParamVal);
-    
+        
+    ///Replace template tags by real data and calculate and add spaces dependent on maxParamLength and spacesFormatFlag
+    ///@param inpString: string containing template data
+    ///@param tmplParamName:string with template tag name
+    ///@param templParamVal: string value that replaces template
+    ///@param maxParamLength: unsigned int maxParamLength
+    ///@param spacesFormatFlag: int formatting flag
+    ///@return:string containing template data replaced by real data
+    ///
+    ///<@tmplParamName@> is replaced by templParamVal
+    static string MapSpaceTemplate(string inpString,string tmplParamName,string templParamVal, unsigned int maxParamLength, int spacesFormatFlag = eSpacePosAtLineEnd);
+
+    ///Calculate the number of spaces and add them to paramVal
+    ///@param string: input parameter value
+    ///@param string: max length for the string that holds parameter
+    ///@param int: additional fomatting after adding spaces
+    ///@param  string: the position of spaces and additional formatting
+    ///@return:string containing paramVal and spaces place appropriately
+    static string AddSpaces(string paramVal, unsigned int maxParamLength, int spacesFormatFlag = eSpacePosToCenter);
+
+
+    static string GetProtocol(void);
+
     ///Create URL for seqid
     ///@param seqUrlInfo: struct SSeqURLInfo containing data for URL construction
     ///@param id: seqid CSeq_id
@@ -1189,6 +1222,18 @@ public:
 
     static map < string, CRef<objects::CSeq_align_set>  >  HspListToHitMap(vector <string> seqIdList, const objects::CSeq_align_set& source);
 
+    ///Scan the the list of blast deflines and find seqID to be use in display    
+    ///@param handle: CBioseq_Handle [in]
+    ///@param aln_id: CSeq_id object for alignment seq [in]
+    ///@param use_this_gi: list<int> list of gis to use [in]
+    ///@param gi: gi to be used for display if exists or 0    
+    ///@param taxid: taxid to be used for display if exists or 0    
+    ///@return: CSeq_id object to be used for display
+    static CRef<objects::CSeq_id> GetDisplayIds(const objects::CBioseq_Handle& handle,
+                                                const objects::CSeq_id& aln_id,
+                                                list<TGi>& use_this_gi,
+                                                TGi& gi,
+                                                int& taxid);
     ///Scan the the list of blast deflines and find seqID to be use in display    
     ///@param handle: CBioseq_Handle [in]
     ///@param aln_id: CSeq_id object for alignment seq [in]
@@ -1229,6 +1274,9 @@ public:
 
     static string GetTitle(const objects::CBioseq_Handle & bh);
 
+    /// Get sequence id with no database source (bare accession)
+    static string GetBareId(const objects::CSeq_id& id);
+
 protected:
 
     ///Wrap a string to specified length.  If break happens to be in
