@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___FASTA__HPP
 #define OBJTOOLS_READERS___FASTA__HPP
 
-/*  $Id: fasta.hpp 448717 2014-10-08 15:44:23Z ivanov $
+/*  $Id: fasta.hpp 506392 2016-07-07 14:28:59Z gotvyans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -64,7 +64,7 @@ class CSeq_loc;
 
 class CSeqIdGenerator;
 
-class IMessageListener;
+class ILineErrorListener;
 
 /// Base class for reading FASTA sequences.
 ///
@@ -117,24 +117,24 @@ public:
     virtual ~CFastaReader(void);
 
     /// CReaderBase overrides
-    virtual CRef<CSerialObject> ReadObject   (ILineReader &lr, IMessageListener *pErrors);
-    virtual CRef<CSeq_entry>    ReadSeqEntry (ILineReader &lr, IMessageListener *pErrors);
+    virtual CRef<CSerialObject> ReadObject   (ILineReader &lr, ILineErrorListener *pErrors);
+    virtual CRef<CSeq_entry>    ReadSeqEntry (ILineReader &lr, ILineErrorListener *pErrors);
 
     /// Indicates (negatively) whether there is any more input.
     bool AtEOF(void) const { return m_LineReader->AtEOF(); }
 
     /// Read a single effective sequence, which may turn out to be a
     /// segmented set.
-    virtual CRef<CSeq_entry> ReadOneSeq(IMessageListener * pMessageListener = 0);
+    virtual CRef<CSeq_entry> ReadOneSeq(ILineErrorListener * pMessageListener = 0);
 
     /// Read multiple sequences (by default, as many as are available.)
-    CRef<CSeq_entry> ReadSet(int max_seqs = kMax_Int, IMessageListener * pMessageListener = 0);
+    CRef<CSeq_entry> ReadSet(int max_seqs = kMax_Int, ILineErrorListener * pMessageListener = 0);
 
     /// Read as many sequences as are available, and interpret them as
     /// an alignment, with hyphens marking relative deletions.
     /// @param reference_row
     ///   0-based; the special value -1 yields a full (pseudo-?)N-way alignment.
-    CRef<CSeq_entry> ReadAlignedSet(int reference_row, IMessageListener * pMessageListener = 0);
+    CRef<CSeq_entry> ReadAlignedSet(int reference_row, ILineErrorListener * pMessageListener = 0);
 
     // also allow changing?
     TFlags GetFlags(void) const { return m_Flags.top(); }
@@ -181,41 +181,12 @@ public:
     /// @returns currently set maximum ID length
     Uint4 GetMaxIDLength(void) const { return m_MaxIDLength; }
 
-    /// indicates which linkage-evidences a given gap-type
-    /// can accept, if any
-    enum ELinkEvid {
-        /// only the "unspecified" linkage-evidence is allowed
-        eLinkEvid_UnspecifiedOnly,
-        /// no linkage-evidence is allowed
-        eLinkEvid_Forbidden,
-        /// any linkage-evidence is allowed, and at least one is required
-        eLinkEvid_Required, 
-    };
-    /// Holds information about a given gap-type string
-    struct SGapTypeInfo {
-        /// The underlying type that the string corresponds to
-        CSeq_gap::EType m_eType;
-        /// Indicates what linkage-evidences are compatible with this
-        ELinkEvid       m_eLinkEvid;
-    };
-    /// Map a gap-type string to its information
-    /// Note that PCase_CStr, which means direct lookup is 
-    /// NOT insensitive to case, etc.
-    typedef CStaticArrayMap<const char *, SGapTypeInfo, PCase_CStr> TGapTypeMap;
-
-    /// From a gap-type string, get the SGapTypeInfo, 
-    /// insensitive to case, etc.  This is NOT quite the same as calling
-    /// find on the TGapTypeMap.
-    ///
-    /// @returns NULL if not found
-    static const SGapTypeInfo * NameToGapTypeInfo(const CTempString & sName);
-    /// This is for if the user needs to get the gap-type string
-    /// to SGapTypeInfo info directly (For example, to iterate through
-    /// all possible types).
-    static const TGapTypeMap & GetNameToGapTypeInfoMap(void);
+    // Make case-sensitive and other kinds of insensitivity, too
+    // (such as "spaces" and "underscores" becoming "hyphens"
+    static string CanonicalizeString(const CTempString & sValue);
 
     void SetMinGaps(TSeqPos gapNmin, TSeqPos gap_Unknown_length);
-    void SetGapLinkageEvidence(CSeq_gap::EType type, CLinkage_evidence::EType evidence);
+    void SetGapLinkageEvidences(CSeq_gap::EType type, const set<int>& evidences);
 
     void IgnoreProblem(ILineError::EProblem problem);
 
@@ -237,27 +208,27 @@ protected:
         TSeqPos m_iLineNum;
     };
 
-    virtual CRef<CSeq_entry> x_ReadSegSet(IMessageListener * pMessageListener);
+    virtual CRef<CSeq_entry> x_ReadSegSet(ILineErrorListener * pMessageListener);
 
-    virtual void   ParseDefLine  (const TStr& s, IMessageListener * pMessageListener);
-    virtual bool   ParseIDs      (const TStr& s, IMessageListener * pMessageListener);
-    virtual size_t ParseRange    (const TStr& s, TSeqPos& start, TSeqPos& end, IMessageListener * pMessageListener);
-    virtual void   ParseTitle    (const SLineTextAndLoc & lineInfo, IMessageListener * pMessageListener);
+    virtual void   ParseDefLine  (const TStr& s, ILineErrorListener * pMessageListener);
+    virtual bool   ParseIDs      (const TStr& s, ILineErrorListener * pMessageListener);
+    virtual size_t ParseRange    (const TStr& s, TSeqPos& start, TSeqPos& end, ILineErrorListener * pMessageListener);
+    virtual void   ParseTitle    (const SLineTextAndLoc & lineInfo, ILineErrorListener * pMessageListener);
     virtual bool   IsValidLocalID(const TStr& s);
     virtual void   GenerateID    (void);
-    virtual void   ParseDataLine (const TStr& s, IMessageListener * pMessageListener);
-    virtual void   CheckDataLine (const TStr& s, IMessageListener * pMessageListener);
-    virtual void   x_CloseGap    (TSeqPos len, bool atStartOfLine, IMessageListener * pMessageListener);
+    virtual void   ParseDataLine (const TStr& s, ILineErrorListener * pMessageListener);
+    virtual void   CheckDataLine (const TStr& s, ILineErrorListener * pMessageListener);
+    virtual void   x_CloseGap    (TSeqPos len, bool atStartOfLine, ILineErrorListener * pMessageListener);
     virtual void   x_OpenMask    (void);
     virtual void   x_CloseMask   (void);
-    virtual bool   ParseGapLine  (const TStr& s, IMessageListener * pMessageListener);
-    virtual void   AssembleSeq   (IMessageListener * pMessageListener);
-    virtual void   AssignMolType (IMessageListener * pMessageListener);
+    virtual bool   ParseGapLine  (const TStr& s, ILineErrorListener * pMessageListener);
+    virtual void   AssembleSeq   (ILineErrorListener * pMessageListener);
+    virtual void   AssignMolType (ILineErrorListener * pMessageListener);
     virtual bool   CreateWarningsForSeqDataInTitle(
         const TStr& sLineText, 
         TSeqPos iLineNum,
-        IMessageListener * pMessageListener);
-    virtual void   PostWarning(IMessageListener * pMessageListener,
+        ILineErrorListener * pMessageListener);
+    virtual void   PostWarning(ILineErrorListener * pMessageListener,
             EDiagSev _eSeverity, size_t _uLineNum, CTempString _MessageStrmOps, 
             CObjReaderParseException::EErrCode _eErrCode, 
             ILineError::EProblem _eProblem, 
@@ -269,13 +240,13 @@ protected:
     typedef map<TSeqPos, TSubMap>       TStartsMap;
     typedef vector<CRef<CSeq_id> >      TIds;
 
-    CRef<CSeq_entry> x_ReadSeqsToAlign(TIds& ids, IMessageListener * pMessageListener);
+    CRef<CSeq_entry> x_ReadSeqsToAlign(TIds& ids, ILineErrorListener * pMessageListener);
     void             x_AddPairwiseAlignments(CSeq_annot& annot, const TIds& ids,
                                              TRowNum reference_row);
     void             x_AddMultiwayAlignment(CSeq_annot& annot, const TIds& ids);
 
     // inline utilities 
-    void CloseGap(bool atStartOfLine=true, IMessageListener * pMessageListener = 0);
+    void CloseGap(bool atStartOfLine=true, ILineErrorListener * pMessageListener = 0);
     void OpenMask(void);
     void CloseMask(void)
         { if (m_MaskRangeStart != kInvalidSeqPos) { x_CloseMask(); } }
@@ -302,15 +273,11 @@ protected:
     TSeqPos GetCurrentPos(EPosType pos_type);
 
     void x_ApplyAllMods( CBioseq & bioseq, TSeqPos iLineNum, 
-        IMessageListener * pMessageListener );
+        ILineErrorListener * pMessageListener );
 
     std::string x_NucOrProt(void) const;
     
-    // Make case-sensitive and other kinds of insensitivity, too
-    // (such as "spaces" and "underscores" becoming "hyphens"
-    static string CanonicalizeString(const TStr & sValue);
-
-private:
+protected:
     struct SGap : public CObject {
         enum EKnownSize {
             eKnownSize_No,
@@ -414,7 +381,7 @@ public:
     void SetCounter(TInt n)           { m_Counter.Set(n); }
     void SetSuffix (const string& s)  { m_Suffix  = s;    }
 
-private:
+protected:
     string         m_Prefix, m_Suffix;
     CAtomicCounter_WithAutoInit m_Counter;
 };
@@ -442,7 +409,7 @@ NCBI_XOBJREAD_EXPORT
 CRef<CSeq_entry> ReadFasta(CNcbiIstream& in, TReadFastaFlags flags = 0,
                            int* counter = 0,
                            vector<CConstRef<CSeq_loc> >* lcv = 0,
-                           IMessageListener * pMessageListener = 0);
+                           ILineErrorListener * pMessageListener = 0);
 
 //////////////////////////////////////////////////////////////////
 //
