@@ -1,4 +1,4 @@
-/*  $Id: blast_seqalign.cpp 194132 2010-06-10 16:07:25Z vasilche $
+/*  $Id: blast_seqalign.cpp 504861 2016-06-20 15:45:40Z boratyng $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -41,8 +41,10 @@
 #include <objects/seqloc/Seq_interval.hpp>
 #include <objects/seqalign/seqalign__.hpp>
 #include <objects/general/Object_id.hpp>
+#include <objects/general/User_object.hpp>
 #include <serial/iterator.hpp>
 #include <objmgr/util/seq_align_util.hpp>
+#include "../core/jumper.h"
 
 #include <algorithm>
 
@@ -393,6 +395,123 @@ s_CorrectUASequence(BlastHSP* hsp)
     return;
 }
 
+
+void MakeSplicedSeg(CSpliced_seg& spliced_seg,
+                    CRef<CSeq_id> product_id,
+                    CRef<CSeq_id> genomic_id,
+                    int product_length,
+                    const BlastHSPChain* chain)
+{
+    spliced_seg.SetProduct_id(*product_id);
+    spliced_seg.SetGenomic_id(*genomic_id);
+    
+    _ASSERT(chain->num_hsps > 0);
+    _ASSERT(chain->hsp_array[0]);
+    ENa_strand product_strand = s_Frame2Strand(chain->hsp_array[0]->query.frame);
+    ENa_strand genomic_strand = s_Frame2Strand(
+                                        chain->hsp_array[0]->subject.frame);
+
+    spliced_seg.SetProduct_type(CSpliced_seg::eProduct_type_transcript);
+    spliced_seg.SetProduct_length(product_length);
+
+    CSpliced_seg::TExons& exons = spliced_seg.SetExons();
+    const Uint1 kGap = 15; // Gap in BLASTNA
+
+    for (int k=0;k < chain->num_hsps;k++) {
+        BlastHSP* hsp = chain->hsp_array[k];
+        _ASSERT(hsp);
+
+        _ASSERT(hsp->gap_info->size > 1 ||
+                hsp->query.end - hsp->query.offset ==
+                hsp->subject.end - hsp->subject.offset);
+
+        CRef<CSpliced_exon> exon(new CSpliced_exon);
+        exon->SetProduct_start().SetNucpos(hsp->query.offset);
+        exon->SetProduct_end().SetNucpos(hsp->query.end - 1);
+        exon->SetGenomic_start(hsp->subject.offset);
+        exon->SetGenomic_end(hsp->subject.end - 1);
+
+        exon->SetProduct_strand(product_strand);
+        exon->SetGenomic_strand(genomic_strand);
+
+        const JumperEditsBlock* hsp_edits = hsp->map_info->edits;
+        int query_pos = hsp->query.offset;
+        int subject_pos = hsp->subject.offset;
+        int num_matches = 0;
+
+        // save splice signal before next exon
+        if (hsp->map_info->left_edge & MAPPER_SPLICE_SIGNAL) {
+            CSpliced_exon::TAcceptor_before_exon::TBases l_bases(2u, ' ');
+            l_bases[0] = BLASTNA_TO_IUPACNA[
+                   (int)((hsp->map_info->left_edge >> 2) & 3)];
+            l_bases[1] = BLASTNA_TO_IUPACNA[
+                   (int)(hsp->map_info->left_edge & 3)];
+            exon->SetAcceptor_before_exon().SetBases(l_bases);
+        }
+
+        // save splice signal after exon
+        if (hsp->map_info->right_edge & MAPPER_SPLICE_SIGNAL) {
+            CSpliced_exon::TDonor_after_exon::TBases r_bases(2u, ' ');
+            r_bases[0] = BLASTNA_TO_IUPACNA[
+                (int)((hsp->map_info->right_edge >> 2) & 3)];
+            r_bases[1] = BLASTNA_TO_IUPACNA[
+                (int)(hsp->map_info->right_edge & 3)];
+            exon->SetDonor_after_exon().SetBases(r_bases);
+        }
+
+        for (int i=0;i < hsp_edits->num_edits;i++) {
+            num_matches = hsp_edits->edits[i].query_pos - query_pos;
+            query_pos += num_matches;
+            subject_pos += num_matches;
+            _ASSERT(num_matches >= 0);
+            if (num_matches > 0) {
+                // record number of matches
+                CRef<CSpliced_exon_chunk> chunk(new CSpliced_exon_chunk);
+                chunk->SetMatch(num_matches);
+                exon->SetParts().push_back(chunk);
+            }
+
+            // record mismatch or gap
+            CRef<CSpliced_exon_chunk> chunk(new CSpliced_exon_chunk);
+            _ASSERT(hsp_edits->edits[i].query_base != kGap ||
+                    hsp_edits->edits[i].subject_base != kGap);
+
+            if (hsp_edits->edits[i].query_base == kGap) {
+                chunk->SetGenomic_ins(1);
+                subject_pos++;
+            }
+            else if (hsp_edits->edits[i].subject_base == kGap) {
+                chunk->SetProduct_ins(1);
+                query_pos++;
+            }
+            else {
+                chunk->SetMismatch(1);
+                query_pos++;
+                subject_pos++;
+            }
+
+            exon->SetParts().push_back(chunk);
+        }
+
+        num_matches = MAX(hsp->query.end - query_pos, 0);
+        _ASSERT(hsp->query.end - query_pos >= -1);
+        // an HSP may end with a mismatch or a gap, if a splice signal was
+        // found and HSP extent was updated (mapping reads to a genome)
+        _ASSERT(num_matches >= 0);
+        if (num_matches > 0) {
+            CRef<CSpliced_exon_chunk> chunk(new CSpliced_exon_chunk);
+            chunk->SetMatch(num_matches);
+            exon->SetParts().push_back(chunk);
+        }
+
+        exons.push_back(exon);
+    }
+
+#if _DEBUG
+    spliced_seg.Validate(true);
+#endif
+}
+
 /// Creates a Seq-align for a single HSP from precalculated vectors of start 
 /// positions, lengths and strands of segments, sequence identifiers and other 
 /// information.
@@ -449,6 +568,10 @@ s_BlastHSP2SeqAlign(EBlastProgramType program, BlastHSP* hsp,
     bool translate1, translate2;
     bool is_disc_align = false;
 
+    if (hsp->score == 0) {
+        return CRef<CSeq_align>();
+    }
+
     GapEditScript* t = hsp->gap_info;
     for (int i=0; i<t->size; i++) {
         if (t->op_type[i] == eGapAlignDecline)
@@ -524,8 +647,11 @@ s_BlastHSP2SeqAlign(EBlastProgramType program, BlastHSP* hsp,
                               strands, query_length, subject_length,
                               translate1, translate2);
 
-        return s_CreateSeqAlign(id1, id2, starts, lengths, strands,
-                                translate1, translate2);
+        CRef<CSeq_align> retval =  s_CreateSeqAlign(id1, id2, starts, lengths,
+                                                    strands, translate1,
+                                                    translate2);
+
+        return retval;
     }
 }
 
@@ -869,12 +995,12 @@ s_OOFBlastHSP2SeqAlign(EBlastProgramType program, BlastHSP* hsp,
 /// @param i Integer value of the score. [in]
 /// @return Resulting CScore object.
 static CRef<CScore> 
-s_MakeScore(const string& ident_string, double d = 0.0, int i = 0)
+s_MakeScore(const string& ident_string, double d, int i, bool is_integer)
 {
     CRef<CScore> retval(new CScore());
     retval->SetId().SetStr(ident_string);
 
-    if (i)
+    if (is_integer)
         retval->SetValue().SetInt(i);
     else
         retval->SetValue().SetReal(d);
@@ -887,13 +1013,14 @@ s_MakeScore(const string& ident_string, double d = 0.0, int i = 0)
 /// [in]
 /// @param gi_list list of GIs associated with this HSP [in]
 static size_t 
-s_CalculateScoreVectorSize(const BlastHSP* hsp, const vector<int>  & gi_list)
+s_CalculateScoreVectorSize(const BlastHSP* hsp, const vector<TGi>  & gi_list)
 {
     _ASSERT(hsp);
-    size_t retval = 0;
+    // query coverage hsp
+    size_t retval = 1;
 
     if (hsp->score) {
-        retval++;
+        retval+=2;
     }
 
     if (hsp->num > 1) {
@@ -909,7 +1036,7 @@ s_CalculateScoreVectorSize(const BlastHSP* hsp, const vector<int>  & gi_list)
         retval++;
     }
 
-    if (hsp->num_ident > 0) {
+    if (hsp->num_ident >= 0) {
         retval++;
     }
 
@@ -917,6 +1044,10 @@ s_CalculateScoreVectorSize(const BlastHSP* hsp, const vector<int>  & gi_list)
         retval++;
     }
     
+    if(hsp->num_positives > 0){
+    	retval++;
+    }
+
     if ( !gi_list.empty() ) {
         retval += gi_list.size();
     }
@@ -930,7 +1061,8 @@ s_CalculateScoreVectorSize(const BlastHSP* hsp, const vector<int>  & gi_list)
 static void
 s_BuildScoreList(const BlastHSP     * hsp,
                  CSeq_align::TScore & scores,
-                 const vector<int>  & gi_list)
+                 const vector<TGi>  & gi_list,
+                 Int4 query_length)
 {
     if (!hsp)
         return;
@@ -939,47 +1071,59 @@ s_BuildScoreList(const BlastHSP     * hsp,
 
     if (hsp->score) {
         static const string kScore("score");
-        scores.push_back(s_MakeScore(kScore, 0.0, hsp->score));
+        scores.push_back(s_MakeScore(kScore, 0.0, hsp->score, true));
+        static const string kBlastScore("blast_score");
+        scores.push_back(s_MakeScore(kBlastScore, 0.0, hsp->score, true));
     }
 
     if (hsp->num > 1) {
         static const string kSumN("sum_n");
-        scores.push_back(s_MakeScore(kSumN, 0.0, hsp->num));
+        scores.push_back(s_MakeScore(kSumN, 0.0, hsp->num, true));
     }
 
     // Set the E-Value
     double evalue = (hsp->evalue < SMALLEST_EVALUE) ? 0.0 : hsp->evalue;
     if (evalue >= 0.0) {
         static string score_type = (hsp->num <= 1) ? "e_value" : "sum_e";
-        scores.push_back(s_MakeScore(score_type, evalue));
+        scores.push_back(s_MakeScore(score_type, evalue, 0, false));
     }
 
     // Calculate the bit score from the raw score
 
     if (hsp->bit_score >= 0.0) {
         static const string kBitScore("bit_score");
-        scores.push_back(s_MakeScore(kBitScore, hsp->bit_score));
+        scores.push_back(s_MakeScore(kBitScore, hsp->bit_score, 0, false));
     }
 
     // Set the identity score
-    if (hsp->num_ident > 0) {
+    if (hsp->num_ident >= 0) {
         static const string kNumIdent("num_ident");
-        scores.push_back(s_MakeScore(kNumIdent, 0.0, hsp->num_ident));
+        scores.push_back(s_MakeScore(kNumIdent, 0.0, hsp->num_ident, true));
     }
 
     if (hsp->comp_adjustment_method > 0) {
         static const string kCompAdj("comp_adjustment_method");
         scores.push_back(s_MakeScore(kCompAdj, 0.0,
-                                     hsp->comp_adjustment_method));
+                                     hsp->comp_adjustment_method, true));
     }
     
     if ( !gi_list.empty() ) {
-        static const string kUseThisGi("use_this_gi");
-        ITERATE(vector<int>, gi, gi_list) {
-            scores.push_back(s_MakeScore(kUseThisGi, 0.0, *gi));
+        ITERATE(vector<TGi>, gi, gi_list) { 
+	    const string kGI = string("gi:") + NStr::NumericToString(*gi);
+            scores.push_back(s_MakeScore(kGI, 0.0, 0, true));
         }
     }
     
+    if (hsp->num_positives > 0) {
+            static const string kNumPositives("num_positives");
+            scores.push_back(s_MakeScore(kNumPositives, 0.0, hsp->num_positives, true));
+    }
+
+    if(query_length > 0) {
+            static const string kQueryCovHsp("hsp_percent_coverage");
+            double hsp_coverage = Blast_HSPGetQueryCoverage( hsp, query_length);
+            scores.push_back(s_MakeScore(kQueryCovHsp, hsp_coverage, 0, false));
+    }
     return;
 }
 
@@ -992,11 +1136,12 @@ s_BuildScoreList(const BlastHSP     * hsp,
 static void
 s_AddScoresToSeqAlign(CRef<CSeq_align>  & seqalign,
                       const BlastHSP    * hsp,
-                      const vector<int> & gi_list)
+                      const vector<TGi> & gi_list,
+                      Int4 query_length)
 {
     // Add the scores for this HSP
     CSeq_align::TScore& score_list = seqalign->SetScore();
-    s_BuildScoreList(hsp, score_list, gi_list);
+    s_BuildScoreList(hsp, score_list, gi_list, query_length);
 }
 
 
@@ -1013,7 +1158,7 @@ CRef<CDense_diag>
 x_UngappedHSPToDenseDiag(BlastHSP* hsp, CRef<CSeq_id> query_id, 
                          CRef<CSeq_id> subject_id,
                          Int4 query_length, Int4 subject_length,
-                         const vector<int> & gi_list)
+                         const vector<TGi> & gi_list)
 {
     CRef<CDense_diag> retval(new CDense_diag());
     
@@ -1045,7 +1190,7 @@ x_UngappedHSPToDenseDiag(BlastHSP* hsp, CRef<CSeq_id> query_id,
     }
 
     CSeq_align::TScore& score_list = retval->SetScores();
-    s_BuildScoreList(hsp, score_list, gi_list);
+    s_BuildScoreList(hsp, score_list, gi_list, query_length);
 
     return retval;
 }
@@ -1063,7 +1208,7 @@ CRef<CStd_seg>
 x_UngappedHSPToStdSeg(BlastHSP* hsp, CRef<CSeq_id> query_id, 
                       CRef<CSeq_id> subject_id,
                       Int4 query_length, Int4 subject_length,
-                      const vector<int> & gi_list)
+                      const vector<TGi> & gi_list)
 {
     CRef<CStd_seg> retval(new CStd_seg());
 
@@ -1119,7 +1264,7 @@ x_UngappedHSPToStdSeg(BlastHSP* hsp, CRef<CSeq_id> query_id,
     retval->SetLoc().push_back(subject_loc);
 
     CSeq_align::TScore& score_list = retval->SetScores();
-    s_BuildScoreList(hsp, score_list, gi_list);
+    s_BuildScoreList(hsp, score_list, gi_list, query_length);
 
     return retval;
 }
@@ -1139,7 +1284,7 @@ BLASTUngappedHspListToSeqAlign(EBlastProgramType program,
                                CRef<CSeq_id> subject_id, 
                                Int4 query_length,
                                Int4 subject_length,
-                               const vector<int> & gi_list,
+                               const vector<TGi> & gi_list,
                                vector<CRef<CSeq_align > > & sa_vector)
 {
     CRef<CSeq_align> seqalign(new CSeq_align()); 
@@ -1198,7 +1343,7 @@ void
 BLASTHspListToSeqAlign(EBlastProgramType program, BlastHSPList* hsp_list, 
                        CRef<CSeq_id> query_id, CRef<CSeq_id> subject_id, 
                        Int4 query_length, Int4 subject_length, bool is_ooframe,
-                       const vector<int> & gi_list,
+                       const vector<TGi> & gi_list,
                        vector<CRef<CSeq_align > > & sa_vector)
 {
     // Process the list of HSPs corresponding to one subject sequence and
@@ -1222,15 +1367,22 @@ BLASTHspListToSeqAlign(EBlastProgramType program, BlastHSPList* hsp_list,
                 s_BlastHSP2SeqAlign(program, hsp, query_id, subject_id, 
                                     query_length, subject_length);
         }
+
+        if (seqalign.Empty()) continue;
         
-        s_AddScoresToSeqAlign(seqalign, hsp, gi_list);
+        if(eBlastTypePsiBlast == program)
+        {
+        	if(hsp->num_ident == 0)
+        		hsp->num_ident = -1;
+        }
+        s_AddScoresToSeqAlign(seqalign, hsp, gi_list, query_length);
         sa_vector.push_back(seqalign);
     }
     
     return;
 }
 
-static CRef<CSeq_align_set> s_CreateEmptySeq_align_set()
+CRef<CSeq_align_set> CreateEmptySeq_align_set(void)
 {
     CRef<CSeq_align_set> retval(new CSeq_align_set);
     retval->Set().clear();
@@ -1277,7 +1429,7 @@ BlastHitList2SeqAlign_OMF(const BlastHitList     * hit_list,
                           bool                     is_ooframe,
                           TSeqLocInfoVector      & subj_masks)
 {
-    CRef<CSeq_align_set> seq_aligns = s_CreateEmptySeq_align_set();
+    CRef<CSeq_align_set> seq_aligns = CreateEmptySeq_align_set();
     
     if (!hit_list) {
         return seq_aligns;
@@ -1319,7 +1471,7 @@ BlastHitList2SeqAlign_OMF(const BlastHitList     * hit_list,
         }
 
         // Get GIs for entrez query restriction.
-        vector<int> gi_list;
+        vector<TGi> gi_list;
         GetFilteredRedundantGis(*seqinfo_src, hsp_list->oid, gi_list);
         
         // stores a CSeq_align for each matching sequence
@@ -1344,11 +1496,24 @@ BlastHitList2SeqAlign_OMF(const BlastHitList     * hit_list,
                                                gi_list,
                                                hit_align);
         }
-        
-        NON_CONST_ITERATE(vector<CRef<CSeq_align > >, iter, hit_align) {
-           RemapToQueryLoc(*iter, query_loc);
-           seq_aligns->Set().push_back(*iter);
+
+	if (seqinfo_src->CanReturnPartialSequence() == true)
+	{
+        	CConstRef<CSeq_loc> subj_loc = seqinfo_src->GetSeqLoc(kOid);
+        	NON_CONST_ITERATE(vector<CRef<CSeq_align > >, iter, hit_align) {
+             	   RemapToQueryLoc(*iter, query_loc);
+             	   if ( !is_ooframe )
+                   	s_RemapToSubjectLoc(*iter, *subj_loc); 
+             	   seq_aligns->Set().push_back(*iter);
+		}
         }
+	else
+	{
+		NON_CONST_ITERATE(vector<CRef<CSeq_align > >, iter, hit_align) {
+           	    RemapToQueryLoc(*iter, query_loc);
+           	    seq_aligns->Set().push_back(*iter);
+		}
+	}
     }
     return seq_aligns;
 }
@@ -1434,7 +1599,8 @@ static void s_AdjustNegativeSubjFrameInBlastn(ENa_strand subj_strand,
                                      BlastHSPList* hsp_list)
 {
     _ASSERT(hsp_list);
-    if (subj_strand != eNa_strand_minus || program != eBlastTypeBlastn)
+    if (subj_strand != eNa_strand_minus ||
+        (program != eBlastTypeBlastn && program != eBlastTypeMapping))
         return;
 
     for (int index = 0; index < hsp_list->hspcnt; index++) { 
@@ -1513,7 +1679,7 @@ s_BLAST_OneSubjectResults2CSeqAlign(const BlastHSPResults* results,
             TSeqPos query_length = query_data.GetSeqLength(qindex); 
             s_AdjustNegativeSubjFrameInBlastn(kSubjStrand, prog, hsp_list);
             
-            vector<int> gi_list;
+            vector<TGi> gi_list;
             GetFilteredRedundantGis(seqinfo_src, hsp_list->oid, gi_list);
 
             // Union subject sequence ranges
@@ -1563,7 +1729,7 @@ s_BLAST_OneSubjectResults2CSeqAlign(const BlastHSPResults* results,
                 seq_aligns->Set().push_back(*iter);
             }
         } else {
-            seq_aligns = s_CreateEmptySeq_align_set();
+            seq_aligns = CreateEmptySeq_align_set();
         }
         retval.push_back(seq_aligns);
     }
@@ -1611,41 +1777,39 @@ s_BlastResults2SeqAlignSequenceCmp_OMF(const BlastHSPResults* results,
                                        vector<TSeqLocInfoVector>& subj_masks)
 {
     TSeqAlignVector retval;
-    retval.reserve(query_data.GetNumQueries() * seqinfo_src->Size());
+    size_t seqinfo_size = seqinfo_src->Size();
+    retval.reserve(query_data.GetNumQueries() * seqinfo_size);
 
-    _ASSERT(results->num_queries == (int)query_data.GetNumQueries());
+    int num_of_queries = results->num_queries;
+    _ASSERT(num_of_queries == (int)query_data.GetNumQueries());
 
     // Compute the subject masks
     subj_masks.clear();
-    subj_masks.resize(results->num_queries);
+    subj_masks.resize(num_of_queries *seqinfo_size);
 
-    for (Uint4 index = 0; index < seqinfo_src->Size(); index++) {
+    for (Uint4 index = 0; index < seqinfo_size; index++) {
+    	vector<TSeqLocInfoVector> tmp_subj_masks(num_of_queries);
         TSeqAlignVector seqalign =
             s_BLAST_OneSubjectResults2CSeqAlign(results, query_data, 
                                                 *seqinfo_src, prog, index, 
                                                 is_gapped, is_ooframe, 
-                                                subj_masks);
+                                                tmp_subj_masks);
 
         /* Merge the new vector with the current. Assume that both vectors
            contain CSeq_align_sets for all queries, i.e. have the same 
            size. */
         _ASSERT(seqalign.size() == query_data.GetNumQueries());
 
-        if (retval.size() == 0) {
-            // First time around, just fill the empty vector with the 
-            // seqaligns from the first subject.
-            retval.swap(seqalign);
-        } else {
-
-            for (TSeqAlignVector::size_type i = 0; i < seqalign.size(); ++i) {
+        for (TSeqAlignVector::size_type i = 0; i < seqalign.size(); ++i) {
                 retval.push_back(seqalign[i]);
-            }
-
+                //seqalign size = num of queries
+                subj_masks[ seqinfo_size * i + index] = tmp_subj_masks[i];
         }
     }
-    _ASSERT(retval.size() == query_data.GetNumQueries() * seqinfo_src->Size());
+    _ASSERT(retval.size() == query_data.GetNumQueries() * seqinfo_size);
+
     return s_TransposeSeqAlignVector(retval, query_data.GetNumQueries(),
-                                     seqinfo_src->Size());
+                                     seqinfo_size);
 }
 
 static TSeqAlignVector
@@ -1737,6 +1901,120 @@ LocalBlastResults2SeqAlign(BlastHSPResults   * hsp_results,
     return retval;
 }
 
+/// Creates a Std-seg object from HSP information and sequence identifiers
+/// for a non-translated ungapped search.
+/// @param hsp An HSP structure [in]
+/// @param query_id Query sequence identifier [in]
+/// @param subject_id Subject sequence identifier [in]
+/// @param query_length Length of the query [in]
+/// @param subject_length Length of the subject [in]
+/// @param gi_list List of GIs for the subject sequence.
+/// @return Resulting Std-seg object.
+CRef<CStd_seg>
+x_NonTranslatedHSPToStdSeg(BlastHSP* hsp, CRef<CSeq_id> query_id,
+                      	   CRef<CSeq_id> subject_id,
+                      	   Int4 query_length, Int4 subject_length,
+                      	   const vector<TGi> & gi_list)
+{
+    CRef<CStd_seg> retval(new CStd_seg());
+
+    retval->SetDim(kBlastAlignmentDim);
+    retval->SetLoc().reserve(kBlastAlignmentDim);
+
+    CRef<CSeq_loc> query_loc(new CSeq_loc());
+    CRef<CSeq_loc> subject_loc(new CSeq_loc());
+    query_loc->SetInt().SetId(*query_id);
+    subject_loc->SetInt().SetId(*subject_id);
+
+    // Set the sequence ids
+    CStd_seg::TIds& ids = retval->SetIds();
+    ids.reserve(kBlastAlignmentDim);
+    ids.push_back(query_id);
+    ids.push_back(subject_id);
+
+    query_loc->SetInt().SetStrand(s_Frame2Strand(hsp->query.frame));
+    subject_loc->SetInt().SetStrand(s_Frame2Strand(hsp->subject.frame));
+
+    if (hsp->query.frame >= 0) {
+           query_loc->SetInt().SetFrom(hsp->query.offset);
+           query_loc->SetInt().SetTo(hsp->query.end -1);
+   } else {
+           query_loc->SetInt().SetFrom(query_length - hsp->query.end);
+           query_loc->SetInt().SetTo(query_length - hsp->query.offset -1);
+   }
+
+   if (hsp->subject.frame >= 0) {
+       subject_loc->SetInt().SetFrom(hsp->subject.offset);
+       subject_loc->SetInt().SetTo(hsp->subject.end-1);
+   } else {
+	   subject_loc->SetInt().SetFrom(subject_length - hsp->subject.end);
+	   subject_loc->SetInt().SetTo(subject_length - hsp->subject.offset -1);
+   }
+
+    retval->SetLoc().push_back(query_loc);
+    retval->SetLoc().push_back(subject_loc);
+
+    CSeq_align::TScore& score_list = retval->SetScores();
+    s_BuildScoreList(hsp, score_list, gi_list, query_length);
+
+    return retval;
+}
+
+
+void
+BLASTPrelminSearchHitListToStdSeg(EBlastProgramType 	   program,
+                     	 	 	  BlastHitList*			   hit_list,
+                     	 	 	  const CSeq_loc & 		   query_loc,
+                     	 	 	  TSeqPos				   query_length,
+                     	 	 	  const IBlastSeqInfoSrc * subject_seqinfo,
+                     	 	 	  list<CRef<CStd_seg > > & seg_list)
+{
+	seg_list.clear();
+
+	CRef<CSeq_id> query_id(new CSeq_id);
+	SerialAssign(*query_id, CSeq_loc_CI(query_loc).GetSeq_id());
+	_ASSERT(query_id);
+
+	CRef<CStd_seg>
+	(*fun_ptr) (BlastHSP* , CRef<CSeq_id> , CRef<CSeq_id> , Int4 , Int4 ,
+	                      	   const vector<TGi> & ) = NULL;
+
+	if((TRANSLATED_QUERY_MASK | TRANSLATED_SUBJECT_MASK) & program )
+		fun_ptr = x_UngappedHSPToStdSeg;
+	else
+		fun_ptr = x_NonTranslatedHSPToStdSeg;
+
+    for (int i = 0; i < hit_list->hsplist_count; i++)
+    {
+    	BlastHSPList* hsp_list = hit_list->hsplist_array[i];
+	    if (!hsp_list)
+	    	continue;
+
+	    BlastHSP ** hsp_array = hsp_list->hsp_array;
+
+	if(hsp_list->hspcnt > 0)
+	{
+            const Uint4 oid = hsp_list->oid;
+       	    TSeqPos subject_length = 0;
+       	    CRef<CSeq_id> subject_id;
+       	    vector<TGi> gi_list;
+       	    GetFilteredRedundantGis(*subject_seqinfo, oid, gi_list);
+       	    GetSequenceLengthAndId(subject_seqinfo, oid, subject_id, &subject_length);
+
+            for (int j = 0; j < hsp_list->hspcnt; j++)
+            {
+        	BlastHSP* hsp = hsp_array[j];
+
+        	if(!hsp)
+        	    continue;
+
+               	seg_list.push_back((*fun_ptr) (hsp, query_id, subject_id, 
+                                               query_length, subject_length, gi_list));
+	    }
+        }
+    }
+}
+
 END_SCOPE(blast)
 END_NCBI_SCOPE
 
