@@ -1,7 +1,3 @@
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] =
-    "$Id: blast_setup_cxx.cpp 199077 2010-07-30 17:40:03Z maning $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
  *
@@ -36,6 +32,7 @@ static char const rcsid[] =
 /// Auxiliary setup functions for Blast objects interface.
 
 #include <ncbi_pch.hpp>
+#include <util/util_misc.hpp>
 #include <corelib/ncbiapp.hpp>
 #include <corelib/metareg.hpp>
 #include <algo/blast/api/blast_options.hpp>
@@ -135,7 +132,8 @@ s_AdjustFirstContext(BlastQueryInfo* query_info,
     _ASSERT(query_info);
 
 #if _DEBUG      /* to eliminate compiler warning in release mode */
-    bool is_na = (prog == eBlastTypeBlastn) ? true : false;
+    bool is_na = (prog == eBlastTypeBlastn || prog == eBlastTypeMapping)
+        ? true : false;
 #endif
 	bool translate = Blast_QueryIsTranslated(prog) ? true : false;
 
@@ -164,7 +162,9 @@ SetupQueryInfo_OMF(const IBlastQuerySource& queries,
     }
 
     const unsigned int kNumContexts = GetNumberOfContexts(prog);
-    bool is_na = (prog == eBlastTypeBlastn) ? true : false;
+    bool is_na = (prog == eBlastTypeBlastn || prog == eBlastTypeMapping)
+        ? true : false;
+
 	bool translate = Blast_QueryIsTranslated(prog) ? true : false;
 
     if (is_na || translate) {
@@ -176,6 +176,7 @@ SetupQueryInfo_OMF(const IBlastQuerySource& queries,
     unsigned int ctx_index = 0; // index into BlastQueryInfo::contexts array
     // Longest query length, to be saved in the query info structure
     Uint4 max_length = 0;
+    Uint4 min_length = INT4_MAX;
 
     for(TSeqPos j = 0; j < queries.Size(); j++) {
         TSeqPos length = 0;
@@ -193,6 +194,7 @@ SetupQueryInfo_OMF(const IBlastQuerySource& queries,
                 unsigned int prot_length = 
                     BLAST_GetTranslatedProteinLength(length, i);
                 max_length = MAX(max_length, prot_length);
+                min_length = MIN(min_length, prot_length);
                 
                 Uint4 ctx_len(0);
                 
@@ -200,11 +202,15 @@ SetupQueryInfo_OMF(const IBlastQuerySource& queries,
                 case eNa_strand_plus:
                     ctx_len = (i<3) ? prot_length : 0;
                     s_QueryInfo_SetContext(query_info, ctx_index + i, ctx_len);
+                    // the missing frame is present in query_info as
+                    // zero-lenghth context
+                    min_length = 0;
                     break;
 
                 case eNa_strand_minus:
                     ctx_len = (i<3) ? 0 : prot_length;
                     s_QueryInfo_SetContext(query_info, ctx_index + i, ctx_len);
+                    min_length = 0;
                     break;
 
                 case eNa_strand_both:
@@ -219,17 +225,22 @@ SetupQueryInfo_OMF(const IBlastQuerySource& queries,
             }
         } else {
             max_length = MAX(max_length, length);
+            min_length = MIN(min_length, length);
             
             if (is_na) {
                 switch (strand) {
                 case eNa_strand_plus:
                     s_QueryInfo_SetContext(query_info, ctx_index, length);
                     s_QueryInfo_SetContext(query_info, ctx_index+1, 0);
+                    // the missing strand is present in query_info as
+                    // zero-lenghth context
+                    min_length = 0;
                     break;
 
                 case eNa_strand_minus:
                     s_QueryInfo_SetContext(query_info, ctx_index, 0);
                     s_QueryInfo_SetContext(query_info, ctx_index+1, length);
+                    min_length = 0;
                     break;
 
                 case eNa_strand_both:
@@ -245,9 +256,17 @@ SetupQueryInfo_OMF(const IBlastQuerySource& queries,
                 s_QueryInfo_SetContext(query_info, ctx_index, length);
             }
         }
+
+        // mark queries that have pairs (for mapping)
+        if (Blast_ProgramIsMapping(prog) && queries.IsFirstOfAPair(j)) {
+            _ASSERT(!translate);
+            query_info->contexts[ctx_index].has_pair = true;
+            query_info->contexts[ctx_index + 1].has_pair = true;
+        }
         ctx_index += kNumContexts;
     }
     query_info->max_length = max_length;
+    query_info->min_length = min_length;
     *qinfo = query_info.Release();
 }
 
@@ -400,8 +419,8 @@ s_RestrictSeqLocs_Multiframe(CBlastQueryFilteredFrames & frame_to_bsl,
                     
                 BlastSeqLoc_RestrictToInterval(bsl,
                                                qseqloc->GetStart(eExtreme_Positional),
-                                               qseqloc->GetStop (eExtreme_Positional));
-                    
+					        qseqloc->GetStop (eExtreme_Positional));
+
                 break;
             }
         }
@@ -484,7 +503,9 @@ SetupQueries_OMF(IBlastQuerySource& queries,
                    "Query sequence buffer");
     }
 
-    bool is_na = (prog == eBlastTypeBlastn) ? true : false;
+    bool is_na = (prog == eBlastTypeBlastn || prog == eBlastTypeMapping)
+        ? true : false;
+
 	bool translate = Blast_QueryIsTranslated(prog) ? true : false;
 
     unsigned int ctx_index = 0;      // index into context_offsets array
@@ -631,9 +652,25 @@ SetupQueries_OMF(IBlastQuerySource& queries,
 }
 
 static void
-s_SeqLoc2MaskedSubjRanges(const CSeq_loc* slp, CSeqDB::TSequenceRanges& output)
+s_SeqLoc2MaskedSubjRanges(const CSeq_loc* slp, 
+                          const CSeq_loc* range,
+                          Int4 total_length,
+                          CSeqDB::TSequenceRanges& output)
 {
     output.clear();
+    
+    TSeqPos offset, length; 
+
+    _ASSERT(range->IsInt() || range->IsWhole());
+  
+    if (range->IsInt()) {
+         offset = range->GetInt().GetFrom();
+         length = range->GetInt().GetTo() - offset + 1;
+    } else {
+         offset = 0;
+         length = total_length;
+    }
+
     if (!slp || 
         slp->Which() == CSeq_loc::e_not_set || 
         slp->IsEmpty() || 
@@ -645,31 +682,40 @@ s_SeqLoc2MaskedSubjRanges(const CSeq_loc* slp, CSeqDB::TSequenceRanges& output)
 
     if (slp->IsInt()) {
         output.reserve(1);
-		CSeqDB::TOffsetPair p;
-		p.first=slp->GetInt().GetFrom();
-		p.second=slp->GetInt().GetTo();
-        output.push_back(p);
+        CSeqDB::TOffsetPair p;
+        p.first = (slp->GetInt().GetFrom() > offset)? slp->GetInt().GetFrom() - offset : 0;
+        p.second = MIN(slp->GetInt().GetTo() - offset, length-1);
+
+        if (slp->GetInt().GetTo() >= offset && p.first < length) {
+            output.push_back(p);
+        }
     } else if (slp->IsPacked_int()) {
         output.reserve(slp->GetPacked_int().Get().size());
         ITERATE(CPacked_seqint::Tdata, itr, slp->GetPacked_int().Get()) {
-    		CSeqDB::TOffsetPair p;
-		    p.first=(*itr)->GetFrom();
-		    p.second=(*itr)->GetTo();
-            output.push_back(p);
+    	    CSeqDB::TOffsetPair p;
+            p.first = ((*itr)->GetFrom() > offset)? (*itr)->GetFrom() - offset : 0;
+            p.second = MIN((*itr)->GetTo() - offset, length-1);
+
+            if ((*itr)->GetTo() >= offset && p.first < length) {
+                output.push_back(p);
+            }
         }
     } else if (slp->IsMix()) {
         output.reserve(slp->GetMix().Get().size());
         ITERATE(CSeq_loc_mix::Tdata, itr, slp->GetMix().Get()) {
+    	    CSeqDB::TOffsetPair p;
             if ((*itr)->IsInt()) {
-				CSeqDB::TOffsetPair p;
-				p.first=(*itr)->GetInt().GetFrom();
-				p.second=(*itr)->GetInt().GetTo();
-                output.push_back(p);
+                p.first = ((*itr)->GetInt().GetFrom() > offset)? (*itr)->GetInt().GetFrom() - offset : 0;
+                p.second = MIN((*itr)->GetInt().GetTo() - offset, length-1);
+                if ((*itr)->GetInt().GetTo() >= offset && p.first < length) {
+                    output.push_back(p);
+                }
             } else if ((*itr)->IsPnt()) {
-				CSeqDB::TOffsetPair p;
-				p.first=(*itr)->GetPnt().GetPoint();
-				p.second=(*itr)->GetPnt().GetPoint();
-                output.push_back(p);
+                p.first = ((*itr)->GetPnt().GetPoint() > offset)? (*itr)->GetPnt().GetPoint() - offset : 0;
+                p.second = MIN((*itr)->GetPnt().GetPoint() - offset, length-1);
+                if ((*itr)->GetPnt().GetPoint() >= offset && p.first < length) {
+                    output.push_back(p);
+                }
             }
         }
     } else {
@@ -710,13 +756,36 @@ SetupSubjects_OMF(IBlastQuerySource& subjects,
     for (TSeqPos i = 0; i < subjects.Size(); i++) {
         BLAST_SequenceBlk* subj = NULL;
 
-        SBlastSequence sequence =
-            subjects.GetBlastSequence(i, encoding, 
-                                      eNa_strand_plus, sentinels);
-
-        if (BlastSeqBlkNew(&subj) < 0) {
-            NCBI_THROW(CBlastSystemException, eOutOfMemory, 
-                       "Subject sequence block");
+        SBlastSequence sequence;
+        try {
+        	if (BlastSeqBlkNew(&subj) < 0) {
+        		NCBI_THROW(CBlastSystemException, eOutOfMemory, "Subject sequence block");
+             }
+        	sequence =
+        			subjects.GetBlastSequence(i, encoding, eNa_strand_plus, sentinels);
+        }
+        catch(CBlastException & e ) {
+        	// Skip bad subject sequence
+        	if(e.GetErrCode() == CBlastException::eInvalidArgument) {
+        		seqblk_vec->push_back(subj);
+        		string warning = kEmptyStr;
+        		const CSeq_id *  id = subjects.GetSeqId(i);
+        		string title = subjects.GetTitle(i);
+        		if(id != NULL) {
+        			warning = id->GetSeqIdString() + " ";
+        		}
+        		warning += subjects.GetTitle(i);
+        		if(warning != kEmptyStr){
+        			warning += ": ";
+        		}
+        		warning += "Subject sequence contians no data";
+        		ERR_POST(Warning << warning);
+        		continue;
+        	}
+        	else {
+        		subj = BlastSequenceBlkFree(subj);
+        		NCBI_RETHROW_SAME(e, e.GetMsg());
+        	}
         }
 
         if (Blast_SubjectIsTranslated(prog)) {
@@ -733,16 +802,22 @@ SetupSubjects_OMF(IBlastQuerySource& subjects,
 
         /* Set the lower case mask, if it exists */
         if (subjects.GetMask(i).NotEmpty()) {
+            CConstRef<CSeq_loc> range = subjects.GetSeqLoc(i);
             const CSeq_loc* masks = subjects.GetMask(i);
+            Int4 length = subjects.GetLength(i);
             CSeqDB::TSequenceRanges masked_ranges;
             _ASSERT(masks);
-            s_SeqLoc2MaskedSubjRanges(masks, masked_ranges);
-            _ASSERT( !masked_ranges.empty() );
-            /// @todo: FIXME: this is inefficient, ideally, the masks shouldn't
-            /// be copied for performance reasons...
-            /// TODO bl2seq only use soft masking?
-            BlastSeqBlkSetSeqRanges(subj, (SSeqRange*) masked_ranges.get_data(),
-                                    masked_ranges.size() + 1, true, DB_MASK_SOFT);
+            s_SeqLoc2MaskedSubjRanges(masks, &*range, length,  masked_ranges);
+            if ( !masked_ranges.empty() ) {
+                /// @todo: FIXME: this is inefficient, ideally, the masks shouldn't
+                /// be copied for performance reasons...
+                /// TODO bl2seq only use soft masking?
+            	subj->length = length;
+                BlastSeqBlkSetSeqRanges(subj, (SSeqRange*) masked_ranges.get_data(),
+                                    masked_ranges.size() + 1, true, eSoftSubjMasking);
+            } else {
+                subj->num_seq_ranges = 0;
+            }
         } else {
             subj->num_seq_ranges = 0;
         }
@@ -809,8 +884,7 @@ GetSequenceProtein(IBlastSeqVector& sv, string* warnings = 0)
     *buf_var++ = GetSentinelByte(eBlastEncodingProtein);
     for (i = 0; i < sv.size(); i++) {
         // Change unsupported residues to X
-        if (sv[i] == AMINOACID_TO_NCBISTDAA[(int)'U'] ||
-            sv[i] == AMINOACID_TO_NCBISTDAA[(int)'O']) {
+        if (sv[i] == AMINOACID_TO_NCBISTDAA[(int)'O']) {
             replaced_residues.push_back(i);
             *buf_var++ = AMINOACID_TO_NCBISTDAA[(int)'X'];
         } else if (!s_IsValidResidue(sv[i])) {
@@ -828,7 +902,7 @@ GetSequenceProtein(IBlastSeqVector& sv, string* warnings = 0)
         }
         if (invalid_residues.size() > kMaxResiduesToWarnAbout) {
             error += ",... (only first ";
-            error += NStr::IntToString(kMaxResiduesToWarnAbout) + " shown)";
+            error += NStr::SizetToString(kMaxResiduesToWarnAbout) + " shown)";
         }
         NCBI_THROW(CBlastException, eInvalidCharacter, error);
     }
@@ -844,7 +918,7 @@ GetSequenceProtein(IBlastSeqVector& sv, string* warnings = 0)
         }
         if (replaced_residues.size() > kMaxResiduesToWarnAbout) {
             *warnings += ",... (only first ";
-            *warnings += NStr::IntToString(kMaxResiduesToWarnAbout);
+            *warnings += NStr::SizetToString(kMaxResiduesToWarnAbout);
             *warnings += " shown)";
         }
     }
@@ -1011,6 +1085,7 @@ GetQueryEncoding(EBlastProgramType program)
     switch (program) {
     case eBlastTypeBlastn:
     case eBlastTypePhiBlastn: 
+    case eBlastTypeMapping:
         retval = eBlastEncodingNucleotide; 
         break;
 
@@ -1043,6 +1118,7 @@ GetSubjectEncoding(EBlastProgramType program)
 
     switch (program) {
     case eBlastTypeBlastn: 
+    case eBlastTypeMapping:
         retval = eBlastEncodingNucleotide; 
         break;
 
@@ -1217,103 +1293,87 @@ char* BlastFindMatrixPath(const char* matrix_name, Boolean is_prot)
     if (!matrix_name)
         return NULL;
 
-    try {
+    try{
+
        string mtx(matrix_name);
        mtx = NStr::ToUpper(mtx);
 
-       // Look for matrix file in local directory
-       string full_path = mtx;       // full path to matrix file
-       if (CFile(full_path).Exists()) {
-           string cwd = CDir::GetCwd();
-           cwd += CFile::GetPathSeparator();
-           return strdup(cwd.c_str());
+       // Try all the default directories
+       string full_path = g_FindDataFile(mtx);
+       if(!full_path.empty()){
+           return s_GetCStringOfMatrixPath(full_path, mtx);
        }
-    
-       string path("");
 
-       // Obtain the matrix path from the ncbi configuration file
-       try {
-           CMetaRegistry::SEntry sentry;
-           sentry = CMetaRegistry::Load("ncbi", CMetaRegistry::eName_RcOrIni);
-           path = sentry.registry ? sentry.registry->Get("NCBI", "Data") : "";
-       } catch (const CRegistryException&) { /* ignore */ }
-    
-       full_path = CFile::MakePath(path, mtx);
-       if (CFile(full_path).Exists()) {
-           return s_GetCStringOfMatrixPath(full_path, mtx);
+       // Try all the default directories with original string case -RMH-
+       full_path = g_FindDataFile(matrix_name);
+       if(!full_path.empty()){
+           return s_GetCStringOfMatrixPath(full_path, matrix_name);
        }
-   
-       // Try appending "aa" or "nt" 
-       full_path = path;
-       full_path += CFile::GetPathSeparator();
-       full_path += is_prot ? "aa" : "nt";
-       full_path += CFile::GetPathSeparator();
-       full_path += mtx;
-       if (CFile(full_path).Exists()) {
-           return s_GetCStringOfMatrixPath(full_path, mtx);
-       }
-   
-       // Try using local "data" directory
-       full_path = "data";
-       full_path += CFile::GetPathSeparator();
-       full_path += mtx;
-       if (CFile(full_path).Exists()) {
-           return s_GetCStringOfMatrixPath(full_path, mtx);
-       }
-   
+
+       // Try env BLASTMAT directory
        CNcbiApplication* app = CNcbiApplication::Instance();
-       if (!app)
-       {
+       if (!app) {
            return NULL;
        }
-   
        const string& blastmat_env = app->GetEnvironment().Get("BLASTMAT");
        if (CDir(blastmat_env).Exists()) {
            full_path = blastmat_env;
            full_path += CFile::GetPathSeparator();
+           full_path += mtx;
+           if (CFile(full_path).Exists()) {
+               return s_GetCStringOfMatrixPath(full_path, mtx);
+           }
+           // Try env BLASTMAT directory with original matrix string case -RMH-
+           full_path = blastmat_env;
+           full_path += CFile::GetPathSeparator();
+           full_path += matrix_name;
+           if (CFile(full_path).Exists()) {
+               return s_GetCStringOfMatrixPath(full_path, matrix_name);
+           }
+
+           // Try original path/nt/matrix or path/aa/matrix alternatives -RMH-
+           full_path = blastmat_env; 
+           full_path += CFile::GetPathSeparator();
            full_path += is_prot ? "aa" : "nt";
            full_path += CFile::GetPathSeparator();
            full_path += mtx;
            if (CFile(full_path).Exists()) {
                return s_GetCStringOfMatrixPath(full_path, mtx);
            }
-       }
-   
-#ifdef OS_UNIX
-       full_path = BLASTMAT_DIR;
-       full_path += CFile::GetPathSeparator();
-       full_path += is_prot ? "aa" : "nt";
-       full_path += CFile::GetPathSeparator();
-       full_path += mtx;
-       if (CFile(full_path).Exists()) {
-           return s_GetCStringOfMatrixPath(full_path, mtx);
-       }
-#endif
 
-       // Try again without the "aa" or "nt"
-       if (CDir(blastmat_env).Exists()) {
+           // Allow original case to be checked. -RMH-
            full_path = blastmat_env;
            full_path += CFile::GetPathSeparator();
-           full_path += mtx;
+           full_path += is_prot ? "aa" : "nt";
+           full_path += CFile::GetPathSeparator();
+           full_path += matrix_name;
            if (CFile(full_path).Exists()) {
-               return s_GetCStringOfMatrixPath(full_path, mtx);
+               return s_GetCStringOfMatrixPath(full_path, matrix_name);
            }
+
        }
 
-#ifdef OS_UNIX
-       full_path = BLASTMAT_DIR;
+       // Try local "data" directory
+       full_path = "data";
        full_path += CFile::GetPathSeparator();
        full_path += mtx;
        if (CFile(full_path).Exists()) {
            return s_GetCStringOfMatrixPath(full_path, mtx);
        }
-#endif
+
+       // Try local "data" directory with original matrix string case -RMH-
+       full_path = "data";
+       full_path += CFile::GetPathSeparator();
+       full_path += matrix_name;
+       if (CFile(full_path).Exists()) {
+           return s_GetCStringOfMatrixPath(full_path, mtx);
+       }
+
     } catch (...)  { } // Ignore all exceptions and return NULL.
 
     return NULL;
 }
 
-
 /// Checks if a BLAST database exists at a given file path: looks for 
 /// an alias file first, then for an index file
 static bool BlastDbFileExists(string& path, bool is_prot)
@@ -1475,6 +1535,23 @@ CBlastQueryFilteredFrames(EBlastProgramType           program,
         
         AddSeqLoc(intv, frame);
         frames.insert(frame);
+        if (Blast_QueryIsTranslated(program))
+        { 
+            if(frame == ncbi::CSeqLocInfo::eFramePlus1)
+            {
+        	AddSeqLoc(intv, ncbi::CSeqLocInfo::eFramePlus2);
+        	frames.insert(ncbi::CSeqLocInfo::eFramePlus2);
+        	AddSeqLoc(intv, ncbi::CSeqLocInfo::eFramePlus3);
+        	frames.insert(ncbi::CSeqLocInfo::eFramePlus3);
+            }  
+            else if (frame == ncbi::CSeqLocInfo::eFrameMinus1)
+            {
+        	AddSeqLoc(intv, ncbi::CSeqLocInfo::eFrameMinus2);
+        	frames.insert(ncbi::CSeqLocInfo::eFrameMinus2);
+        	AddSeqLoc(intv, ncbi::CSeqLocInfo::eFrameMinus3);
+        	frames.insert(ncbi::CSeqLocInfo::eFrameMinus3);
+            }
+        }
     }
 }
 
@@ -1519,8 +1596,8 @@ void CBlastQueryFilteredFrames::UseProteinCoords(TSeqPos dna_length)
                 int to(0), from(0);
                 
                 if (frame < 0) {
-                    from = (dna_length + frame - itr->ssr->right) / CODON_LENGTH;
-                    to = (dna_length + frame - itr->ssr->left) / CODON_LENGTH;
+                    from = ((int) dna_length + frame - itr->ssr->right) / CODON_LENGTH;
+                    to = ((int) dna_length + frame - itr->ssr->left) / CODON_LENGTH;
                 } else {
                     from = (itr->ssr->left - frame + 1) / CODON_LENGTH;
                     to = (itr->ssr->right - frame + 1) / CODON_LENGTH;
@@ -1579,6 +1656,7 @@ void CBlastQueryFilteredFrames::x_VerifyFrame(int frame)
         break;
         
     case eBlastTypeBlastn:
+    case eBlastTypeMapping:
         if ((frame != CSeqLocInfo::eFramePlus1) &&
             (frame != CSeqLocInfo::eFrameMinus1)) {
             okay = false;
@@ -1627,6 +1705,7 @@ bool CBlastQueryFilteredFrames::QueryHasMultipleFrames() const
     case eBlastTypeBlastx:
     case eBlastTypeTblastx:
     case eBlastTypeRpsTblastn:
+    case eBlastTypeMapping:
         return true;
         
     default:
@@ -1641,7 +1720,9 @@ void CBlastQueryFilteredFrames::AddSeqLoc(const objects::CSeq_interval & intv,
                                           int frame)
 {
     _ASSERT( m_Frames.empty() );
-    if ((frame == 0) && (m_Program == eBlastTypeBlastn)) {
+    if ((frame == 0) && (m_Program == eBlastTypeBlastn
+                         || m_Program == eBlastTypeMapping)) {
+
         x_VerifyFrame(CSeqLocInfo::eFramePlus1);
         x_VerifyFrame(CSeqLocInfo::eFrameMinus1);
         static const CSeqLocInfo::ETranslationFrame kFrames[] = {
