@@ -29,10 +29,6 @@
  *
  * @author Alejandro Schaffer, E. Michael Gertz
  */
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] =
-    "$Id: redo_alignment.c 496008 2016-03-23 11:29:15Z ivanov $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <stdlib.h>
 #include <assert.h>
@@ -139,7 +135,6 @@ BlastCompo_AlignmentsFree(BlastCompo_Alignment ** palign,
         /* Save the value of align->next, because align is to be deleted. */
         BlastCompo_Alignment * align_next = align->next;
 
-        align_next = align->next;
         if (free_context != NULL && align->context != NULL) {
             free_context(align->context);
         }
@@ -626,6 +621,17 @@ s_SubjectCompareWindows(const void * vp1, const void *vp2)
     return result;
 }
 
+static
+int s_GetTranslatedLength(int length,int frame, int is_pos_based) {
+	if(is_pos_based) {
+		int f = GET_SEQ_FRAME(frame);
+		int nucl_length = GET_NUCL_LENGTH(length);
+		return GET_TRANSLATED_LENGTH(nucl_length, f);
+	}
+
+	return	((length - ABS(frame) + 1)/3);
+}
+
 /**
  * Read a list of alignments from a translated search and create a
  * new array of pointers to s_WindowInfo so that each alignment is
@@ -639,7 +645,7 @@ s_WindowsFromTranslatedAligns(BlastCompo_Alignment * alignments,
                               BlastCompo_QueryInfo * query_info,
                               int hspcnt, int border, int sequence_length,
                               s_WindowInfo ***pwindows, int * nWindows,
-                              int subject_is_translated)
+                              int subject_is_translated, int is_pos_based)
 {
     int k;                            /* iteration index */
     s_WindowInfo ** windows;      /* the output list of windows */
@@ -669,7 +675,7 @@ s_WindowsFromTranslatedAligns(BlastCompo_Alignment * alignments,
         frame = align->frame;
         query_index = align->queryIndex;
         query_length = query_info[query_index].seq.length;
-        translated_length = (sequence_length - ABS(frame) + 1)/3;
+        translated_length = s_GetTranslatedLength(sequence_length, frame, is_pos_based);
 
         align_copy = s_AlignmentCopy(align);
         if (align_copy == NULL)
@@ -812,8 +818,10 @@ s_WindowsFromProteinAligns(BlastCompo_Alignment * alignments,
     }
     /* shrink to fit */
     {
-        s_WindowInfo ** new_windows =
-            realloc(windows, window_index * sizeof(s_WindowInfo*));
+        s_WindowInfo ** new_windows = NULL;
+	if (window_index > 0)
+            new_windows = realloc(windows, window_index * sizeof(s_WindowInfo*));
+
         if (new_windows == NULL) {
             goto error_return;
         } else {
@@ -863,19 +871,19 @@ s_WindowsFromAligns(BlastCompo_Alignment * alignments,
                 BlastCompo_QueryInfo * query_info, int hspcnt,
                 int numQueries, int border, int sequence_length,
                 s_WindowInfo ***pwindows, int * nWindows,
-                int query_is_translated, int subject_is_translated)
+                int query_is_translated, int subject_is_translated, int is_pos_based)
 {
     if (subject_is_translated || query_is_translated) {
         return s_WindowsFromTranslatedAligns(alignments, query_info,
                                              hspcnt, border,
                                              sequence_length,
                                              pwindows, nWindows,
-                                             subject_is_translated);
+                                             subject_is_translated, is_pos_based);
     } else {
         return s_WindowsFromProteinAligns(alignments, query_info,
                                           numQueries, sequence_length,
                                           pwindows, nWindows);
-    }
+   }
 }
 
 
@@ -909,7 +917,6 @@ s_GetComposition(Blast_AminoAcidComposition * composition,
 
     data = seq->data;
     length = range->end - range->begin;
-
     if (query_is_translated || subject_is_translated) {
         int start;
         int end;
@@ -1125,7 +1132,7 @@ Blast_RedoOneMatch(BlastCompo_Alignment ** alignments,
     status =
             s_WindowsFromAligns(incoming_aligns, query_info, hspcnt, numQueries,
                     kWindowBorder, matchingSeq->length, &windows,
-                    &nWindows, query_is_translated, subject_is_translated);
+                    &nWindows, query_is_translated, subject_is_translated, params->positionBased);
     if (status != 0) {
         goto function_level_cleanup;
     }
@@ -1246,8 +1253,6 @@ Blast_RedoOneMatch(BlastCompo_Alignment ** alignments,
                     num_adjustments++;
                 }
 
-                // By this point there's a discrepancy in
-                //  gapping_params->context->gap_align->sbp->matrix->data
                 if ( !adjust_search_failed ) {
                     newAlign = callbacks->redo_one_alignment(
                             in_align,
@@ -1346,7 +1351,7 @@ Blast_RedoOneMatchSmithWaterman(BlastCompo_Alignment ** alignments,
     status =
         s_WindowsFromAligns(incoming_aligns, query_info, hspcnt, numQueries,
                             kWindowBorder, matchingSeq->length, &windows,
-                            &nWindows, query_is_translated, subject_is_translated);
+                            &nWindows, query_is_translated, subject_is_translated, params->positionBased);
     if (status != 0)
         goto function_level_cleanup;
     /* We are performing a Smith-Waterman alignment */
