@@ -1,4 +1,4 @@
-/* $Id: ncbi_connutil.c 488578 2016-01-01 00:35:35Z lavr $
+/* $Id: ncbi_connutil.c 507385 2016-07-18 22:38:48Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,6 +43,8 @@
 #define NCBI_USE_ERRCODE_X   Connect_Util
 
 
+#define CONN_NET_INFO_MAGIC  0x600D600D
+
 #define SizeOf(arr)  (sizeof(arr) / sizeof((arr)[0]))
 
 
@@ -283,6 +285,13 @@ static EFWMode x_ParseFirewall(const char* str, int/*bool*/ generic)
 }
 
 
+static int/*bool*/ s_InfoIsValid(const SConnNetInfo* info)
+{
+    assert(info->magic == CONN_NET_INFO_MAGIC);
+    return info->magic == CONN_NET_INFO_MAGIC ? 1/*true*/ : 0/*false*/;
+}
+
+
 /****************************************************************************
  * ConnNetInfo API
  */
@@ -304,11 +313,14 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
 
     len = service ? strlen(service) : 0;
 
-    /* NB: *NOT* cleared up with all 0s */
+    /* NB: created *NOT* cleared up with all 0s */
     if (!(info = (SConnNetInfo*) malloc(sizeof(*info) + len)))
         return 0/*failure*/;
     info->reserved = 0/*MBZ*/;
 
+    /* store the service name, which this structure has been created for */
+    memcpy((char*) info->svc, service ? service : "", ++len);
+
     /* client host: default */
     info->client_host[0] = '\0';
 
@@ -437,8 +449,8 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     /* credentials */
     info->credentials = 0;
 
-    /* store the service name, which this structure has been created for */
-    strcpy((char*) info->svc, service ? service : "");
+    /* magic */
+    info->magic = CONN_NET_INFO_MAGIC;
 
     /* done */
     return info;
@@ -457,8 +469,9 @@ extern int/*bool*/ ConnNetInfo_ParseURL(SConnNetInfo* info, const char* url)
     size_t len;
     char* p;
 
-    if (!url)
+    if (!s_InfoIsValid(info)  ||  !url)
         return 0/*failure*/;
+
     if (!*url)
         return 1/*success*/;
 
@@ -638,6 +651,9 @@ extern int/*bool*/ ConnNetInfo_ParseURL(SConnNetInfo* info, const char* url)
 extern int/*bool*/ ConnNetInfo_SetUserHeader(SConnNetInfo* info,
                                              const char*   user_header)
 {
+    if (!s_InfoIsValid(info))
+        return 0/*failure*/;
+
     if (info->http_user_header)
         free((void*) info->http_user_header);
     if (!user_header  ||  !*user_header)
@@ -653,6 +669,9 @@ extern int/*bool*/ ConnNetInfo_AppendUserHeader(SConnNetInfo* info,
 {
     char* new_header;
 
+    if (!s_InfoIsValid(info))
+        return 0/*failure*/;
+
     if (!info->http_user_header  ||  !*info->http_user_header)
         return ConnNetInfo_SetUserHeader(info, user_header);
 
@@ -712,6 +731,9 @@ static int/*bool*/ s_ModifyUserHeader(SConnNetInfo*      info,
     size_t hdrlen;
     char*  hdr;
 
+    if (!s_InfoIsValid(info))
+        return 0/*failure*/;
+
     if (!user_header || !(newhdrlen = strlen(user_header)))
         return 1/*success*/;
 
@@ -899,7 +921,10 @@ extern int/*bool*/ ConnNetInfo_AppendArg(SConnNetInfo* info,
 {
     size_t len, used;
 
-    if (!arg || !*arg)
+    if (!s_InfoIsValid(info))
+        return 0/*failure*/;
+
+    if (!arg  ||  !*arg)
         return 1/*success*/;
 
     used = strlen(info->args);
@@ -913,7 +938,7 @@ extern int/*bool*/ ConnNetInfo_AppendArg(SConnNetInfo* info,
     if (used)
         info->args[used++] = '&';
     strcpy(info->args + used, arg);
-    if (val && *val) {
+    if (val  &&  *val) {
         used += len;
         info->args[used++] = '=';
         strcpy(info->args + used, val);
@@ -928,7 +953,10 @@ extern int/*bool*/ ConnNetInfo_PrependArg(SConnNetInfo* info,
 {
     size_t len, off, used;
 
-    if (!arg || !*arg)
+    if (!s_InfoIsValid(info))
+        return 0/*failure*/;
+
+    if (!arg  ||  !*arg)
         return 1/*success*/;
 
     used = strlen(info->args);
@@ -959,8 +987,9 @@ extern int/*bool*/ ConnNetInfo_DeleteArg(SConnNetInfo* info,
     size_t arglen;
     char*  a;
 
-    if (!arg || !(argnamelen = strcspn(arg, "=&")))
+    if (!s_InfoIsValid(info)  ||  !arg  ||  !(argnamelen = strcspn(arg, "=&")))
         return 0/*false*/;
+    
     deleted = 0/*false*/;
     for (a = info->args; *a; a += arglen) {
         if (*a == '&')
@@ -989,8 +1018,9 @@ extern int/*bool*/ ConnNetInfo_DeleteArg(SConnNetInfo* info,
 extern void ConnNetInfo_DeleteAllArgs(SConnNetInfo* info,
                                       const char*   args)
 {
-    if (!args)
+    if (!s_InfoIsValid(info)  ||  !args)
         return;
+
     while (*args) {
         const char* a = strchr(args, '&');
         if (!a)
@@ -1007,8 +1037,12 @@ extern int/*bool*/ ConnNetInfo_PreOverrideArg(SConnNetInfo* info,
                                               const char*   arg,
                                               const char*   val)
 {
-    if (!arg || !*arg)
+    if (!s_InfoIsValid(info))
+        return 0/*failure*/;
+
+    if (!arg  ||  !*arg)
         return 1/*success*/;
+
     ConnNetInfo_DeleteAllArgs(info, arg);
     return ConnNetInfo_PrependArg(info, arg, val);
 }
@@ -1018,8 +1052,12 @@ extern int/*bool*/ ConnNetInfo_PostOverrideArg(SConnNetInfo* info,
                                                const char*   arg,
                                                const char*   val)
 {
-    if (!arg || !*arg)
+    if (!s_InfoIsValid(info))
+        return 0/*failure*/;
+
+    if (!arg  ||  !*arg)
         return 1/*success*/;
+
     ConnNetInfo_DeleteAllArgs(info, arg);
     return ConnNetInfo_AppendArg(info, arg, val);
 }
@@ -1060,8 +1098,7 @@ static const char* x_ClientAddress(const char* client_host,
     sprintf(s, "%s(%s)", client_host, addr);
     if (client_host != c)
         free((void*) client_host);
-    client_host = s;
-    for (;  *s;  ++s) {
+    for (client_host = s;  *s;  ++s) {
         if (*s == ' ')
             *s  = '+';
     }
@@ -1078,7 +1115,7 @@ extern int/*bool*/ ConnNetInfo_SetupStandardArgs(SConnNetInfo* info,
     int/*bool*/ local_host;
     const char* s;
 
-    if (!info)
+    if (!info  ||  !s_InfoIsValid(info))
         return 0/*failed*/;
 
     s = CORE_GetAppName();
@@ -1122,20 +1159,38 @@ extern SConnNetInfo* ConnNetInfo_Clone(const SConnNetInfo* info)
 {
     SConnNetInfo* x_info;
 
-    if (!info)
+    if (!info  ||  !s_InfoIsValid(info))
         return 0;
 
     if (!(x_info = (SConnNetInfo*) malloc(sizeof(*info) + strlen(info->svc))))
         return 0;
 
-    memcpy(x_info, info, sizeof(*x_info));
-    x_info->http_user_header = 0;
-    x_info->http_referer = 0;
+    strcpy(x_info->client_host,     info->client_host);
+    x_info->scheme                = info->scheme;
+    x_info->req_method            = info->req_method;
+    x_info->version               = info->version;
+    x_info->firewall              = info->firewall;
+    x_info->stateless             = info->stateless;
+    x_info->lb_disable            = info->lb_disable;
+    x_info->debug_printout        = info->debug_printout;
+    x_info->http_push_auth        = info->http_push_auth;
+    x_info->http_proxy_leak       = info->http_proxy_leak;
+    x_info->reserved              = info->reserved;
+    strcpy(x_info->user,            info->user);
+    strcpy(x_info->pass,            info->pass);
+    strcpy(x_info->host,            info->host);
+    x_info->port                  = info->port;
+    strcpy(x_info->path,            info->path);
+    strcpy(x_info->args,            info->args);
+    strcpy(x_info->http_proxy_host, info->http_proxy_host);
+    x_info->http_proxy_port       = info->http_proxy_port;
+    strcpy(x_info->http_proxy_user, info->http_proxy_user);
+    strcpy(x_info->http_proxy_pass, info->http_proxy_pass);
+    x_info->max_try               = info->max_try;
+    x_info->http_user_header      = 0;
+    x_info->http_referer          = 0;
+    x_info->credentials           = info->credentials;
 
-    if (info->timeout) {
-        x_info->tmo     = *info->timeout;
-        x_info->timeout = &x_info->tmo;
-    }
     if (info->http_user_header  &&  *info->http_user_header
         &&  !(x_info->http_user_header = strdup(info->http_user_header))) {
         ConnNetInfo_Destroy(x_info);
@@ -1146,11 +1201,24 @@ extern SConnNetInfo* ConnNetInfo_Clone(const SConnNetInfo* info)
         ConnNetInfo_Destroy(x_info);
         return 0;
     }
-    strcpy((char*) x_info->svc, info->svc);
+
+    x_info->tmo                   = info->timeout ? *info->timeout : info->tmo;
+    x_info->timeout               = info->timeout ? &x_info->tmo   : 0;
+    strcpy((char*) x_info->svc,     info->svc);
+
+    x_info->magic                 = CONN_NET_INFO_MAGIC;
     return x_info;
 }
 
 
+static const char* x_BadMagic(unsigned int magic, char buf[])
+{
+    sprintf(buf, "0x%08lX (INVALID != 0x%08lX)",
+            (unsigned long) magic, (unsigned long) CONN_NET_INFO_MAGIC);
+    return buf;
+}
+
+
 static const char* x_Port(unsigned short port, char buf[])
 {
     assert(port);
@@ -1273,6 +1341,7 @@ static void s_SaveUserHeader(char* s, const char* name,
         memcpy(s, "NULL\n", 6);
 }
 
+
 extern void ConnNetInfo_Log(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
 {
     char   buf[80];
@@ -1282,7 +1351,7 @@ extern void ConnNetInfo_Log(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
 
     if (!info) {
         LOG_Write(lg, NCBI_C_ERRCODE_X, 10, sev, 0, 0, 0, 0,
-                  "ConnNetInfo_Log: NULL info", 0, 0);
+                  "ConnNetInfo_Log: NULL", 0, 0);
         return;
     }
 
@@ -1302,6 +1371,8 @@ extern void ConnNetInfo_Log(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
 
     strcpy(s, "ConnNetInfo_Log\n"
            "#################### [BEGIN] SConnNetInfo:\n");
+    if (!s_InfoIsValid(info))
+        s_SaveKeyval(s, "magic",           x_BadMagic(info->magic, buf));
     if (*info->svc)
         s_SaveString(s, "service",         info->svc);
     else
@@ -1394,7 +1465,7 @@ extern char* ConnNetInfo_URL(const SConnNetInfo* info)
     char*       url;
     char        buf[40];
 
-    if (!info)
+    if (!info  ||  !s_InfoIsValid(info))
         return 0/*failed*/;
 
     req_method = info->req_method & ~eReqMethod_v1;
@@ -1440,7 +1511,7 @@ extern char* ConnNetInfo_URL(const SConnNetInfo* info)
 extern int/*bool*/ ConnNetInfo_SetTimeout(SConnNetInfo*   info,
                                           const STimeout* timeout)
 {
-    if (!info  ||  timeout == kDefaultTimeout)
+    if (!s_InfoIsValid(info)  ||  timeout == kDefaultTimeout)
         return 0/*failed*/;
     if (timeout) {
         info->tmo     = *timeout;
@@ -1450,7 +1521,6 @@ extern int/*bool*/ ConnNetInfo_SetTimeout(SConnNetInfo*   info,
     return 1/*succeeded*/;
 }
 
-
 extern void ConnNetInfo_Destroy(SConnNetInfo* info)
 {
     if (info) {
@@ -1551,8 +1621,8 @@ extern EIO_Status URL_ConnectEx
     /* select request method and its verbal representation */
     if (x_req_meth == eReqMethod_Any)
         x_req_meth  = content_length ? eReqMethod_Post : eReqMethod_Get;
-    else if (content_length  &&  (x_req_meth == eReqMethod_Get   ||
-                                  x_req_meth == eReqMethod_Head)) {
+    else if (content_length  &&  (x_req_meth == eReqMethod_Head  ||
+                                  x_req_meth == eReqMethod_Get)) {
         CORE_LOGF_X(3, eLOG_Warning,
                     ("[URL_Connect; http%s://%s:%hu%s%s] "
                      " Content-Length (%lu) is ignored with request method %s",
