@@ -1,4 +1,4 @@
-/*  $Id: dbapi_impl_context.cpp 399230 2013-05-13 13:59:10Z ucko $
+/*  $Id: dbapi_impl_context.cpp 510689 2016-08-15 15:29:39Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -63,10 +63,11 @@ namespace impl
 //
 
 CDriverContext::CDriverContext(void) :
+    m_PoolSem(0, 1),
     m_LoginTimeout(0),
     m_Timeout(0),
     m_CancelTimeout(0),
-    m_MaxTextImageSize(0),
+    m_MaxBlobSize(0),
     m_ClientEncoding(eEncoding_ISO8859_1)
 {
     PushCntxMsgHandler    ( &CDB_UserHandler::GetDefault(), eTakeOwnership );
@@ -75,13 +76,15 @@ CDriverContext::CDriverContext(void) :
 
 CDriverContext::~CDriverContext(void)
 {
-    return;
+    try {
+        DeleteAllConn();
+    } STD_CATCH_ALL("CDriverContext::DeleteAllConn");
 }
 
 void
 CDriverContext::SetApplicationName(const string& app_name)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     m_AppName = app_name;
 }
@@ -89,7 +92,7 @@ CDriverContext::SetApplicationName(const string& app_name)
 string
 CDriverContext::GetApplicationName(void) const
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     return m_AppName;
 }
@@ -97,7 +100,7 @@ CDriverContext::GetApplicationName(void) const
 void
 CDriverContext::SetHostName(const string& host_name)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     m_HostName = host_name;
 }
@@ -105,21 +108,21 @@ CDriverContext::SetHostName(const string& host_name)
 string
 CDriverContext::GetHostName(void) const
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     return m_HostName;
 }
 
 unsigned int CDriverContext::GetLoginTimeout(void) const 
 { 
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     return m_LoginTimeout; 
 }
 
 bool CDriverContext::SetLoginTimeout (unsigned int nof_secs)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     m_LoginTimeout = nof_secs;
 
@@ -128,7 +131,7 @@ bool CDriverContext::SetLoginTimeout (unsigned int nof_secs)
 
 unsigned int CDriverContext::GetTimeout(void) const 
 { 
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     return m_Timeout; 
 }
@@ -136,7 +139,7 @@ unsigned int CDriverContext::GetTimeout(void) const
 bool CDriverContext::SetTimeout(unsigned int nof_secs)
 {
     bool success = true;
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     try {
         m_Timeout = nof_secs;
@@ -153,7 +156,7 @@ bool CDriverContext::SetTimeout(unsigned int nof_secs)
 
 unsigned int CDriverContext::GetCancelTimeout(void) const 
 { 
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     return m_CancelTimeout;
 }
@@ -161,7 +164,7 @@ unsigned int CDriverContext::GetCancelTimeout(void) const
 bool CDriverContext::SetCancelTimeout(unsigned int nof_secs)
 {
     bool success = true;
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     try {
         m_CancelTimeout = nof_secs;
@@ -172,13 +175,13 @@ bool CDriverContext::SetCancelTimeout(unsigned int nof_secs)
     return success;
 }
 
-bool CDriverContext::SetMaxTextImageSize(size_t nof_bytes)
+bool CDriverContext::SetMaxBlobSize(size_t nof_bytes)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
-    m_MaxTextImageSize = nof_bytes;
+    m_MaxBlobSize = nof_bytes;
 
-    UpdateConnMaxTextImageSize();
+    UpdateConnMaxBlobSize();
 
     return true;
 }
@@ -186,26 +189,26 @@ bool CDriverContext::SetMaxTextImageSize(size_t nof_bytes)
 void CDriverContext::PushCntxMsgHandler(CDB_UserHandler* h,
                                          EOwnership ownership)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
     m_CntxHandlers.Push(h, ownership);
 }
 
 void CDriverContext::PopCntxMsgHandler(CDB_UserHandler* h)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
     m_CntxHandlers.Pop(h);
 }
 
 void CDriverContext::PushDefConnMsgHandler(CDB_UserHandler* h,
                                             EOwnership ownership)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
     m_ConnHandlers.Push(h, ownership);
 }
 
 void CDriverContext::PopDefConnMsgHandler(CDB_UserHandler* h)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
     m_ConnHandlers.Pop(h);
 
     // Remove this handler from all connections
@@ -232,6 +235,10 @@ void CDriverContext::ResetEnvSybase(void)
     // If user forces his own Sybase client path using $RESET_SYBASE
     // and $SYBASE -- use that unconditionally.
     try {
+        if (env.Get("LC_ALL") == "POSIX") {
+            // Canonicalize, since locales.dat might list C but not POSIX.
+            env.Set("LC_ALL", "C");
+        }
         if (!env.Get("SYBASE").empty()) {
             string reset = env.Get("RESET_SYBASE");
             if ( !reset.empty() && NStr::StringToBool(reset)) {
@@ -263,7 +270,7 @@ void CDriverContext::ResetEnvSybase(void)
 
 void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(m_PoolMutex);
 
     TConnPool::iterator it = find(m_InUse.begin(), m_InUse.end(), conn);
 
@@ -271,17 +278,39 @@ void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
         m_InUse.erase(it);
     }
 
+#ifdef NCBI_THREADS
+    bool for_sem = false;
+    if ( !m_PoolSemSubject.empty()  &&  conn->PoolName() == m_PoolSemSubject) {
+        for_sem = true;
+        m_PoolSemSubject.erase();
+        m_PoolSemConn = NULL;
+        m_PoolSem.TryWait(); // probably redundant, but ensures Post will work
+        m_PoolSem.Post();
+    }
+#endif
+
     if (conn_reusable  &&  conn->IsOpeningFinished()  &&  conn->IsValid()) {
+        if (conn->m_PoolIdleTimeParam.GetSign() != eNegative) {
+            CTime now(CTime::eCurrent);
+            conn->m_CleanupTime = now + conn->m_PoolIdleTimeParam;
+        }
         m_NotInUse.push_back(conn);
+#ifdef NCBI_THREADS
+        if (for_sem) {
+            m_PoolSemConn = conn;
+        }
+#endif
     } else {
         delete conn;
     }
+
+    CloseOldIdleConns(1);
 }
 
 void CDriverContext::CloseUnusedConnections(const string&   srv_name,
                                              const string&   pool_name)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(m_PoolMutex);
 
     TConnPool::value_type con;
 
@@ -289,7 +318,9 @@ void CDriverContext::CloseUnusedConnections(const string&   srv_name,
     NON_CONST_ITERATE(TConnPool, it, m_NotInUse) {
         con = *it;
 
-        if((!srv_name.empty()) && srv_name.compare(con->ServerName())) continue;
+        if ( !srv_name.empty()  && srv_name != con->ServerName()
+            &&  srv_name != con->GetRequestedServer() )
+            continue;
         if((!pool_name.empty()) && pool_name.compare(con->PoolName())) continue;
 
         it = m_NotInUse.erase(it);
@@ -301,7 +332,7 @@ void CDriverContext::CloseUnusedConnections(const string&   srv_name,
 unsigned int CDriverContext::NofConnections(const TSvrRef& svr_ref,
                                             const string& pool_name) const
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(m_PoolMutex);
 
     if ((!svr_ref  ||  !svr_ref->IsValid())  &&  pool_name.empty()) {
         return static_cast<unsigned int>(m_InUse.size() + m_NotInUse.size());
@@ -323,7 +354,8 @@ unsigned int CDriverContext::NofConnections(const TSvrRef& svr_ref,
         ITERATE(TConnPool, it, (*pools[i])) {
             TConnPool::value_type con = *it;
             if(!server.empty()) {
-                if (server.compare(con->ServerName()))
+                if (server != con->ServerName()
+                    &&  server != con->GetRequestedServer())
                     continue;
             }
             else if (host != 0) {
@@ -347,16 +379,35 @@ unsigned int CDriverContext::NofConnections(const string& srv_name,
 
 CDB_Connection* CDriverContext::MakeCDBConnection(CConnection* connection)
 {
+    connection->m_CleanupTime.Clear();
+    CMutexGuard mg(m_PoolMutex);
     m_InUse.push_back(connection);
 
     return new CDB_Connection(connection);
 }
 
+class CDBConnParams_Unpooled : public CDBConnParamsDelegate
+{
+public:
+    CDBConnParams_Unpooled(const CDBConnParams& other)
+        : CDBConnParamsDelegate(other)
+        { }
+
+    string GetParam(const string& key) const
+        {
+            if (key == "is_pooled") {
+                return "false";
+            } else {
+                return CDBConnParamsDelegate::GetParam(key);
+            }
+        }
+};
+    
 CDB_Connection*
 CDriverContext::MakePooledConnection(const CDBConnParams& params)
 {
     if (params.GetParam("is_pooled") == "true") {
-        CMutexGuard mg(m_CtxMtx);
+        CMutexGuard mg(m_PoolMutex);
 
         string pool_name(params.GetParam("pool_name"));
         if (!m_NotInUse.empty()) {
@@ -387,8 +438,8 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
                 }
             }
             else {
-
-                if ( params.GetServerName().empty() ) {
+                string server_name = params.GetServerName();
+                if ( server_name.empty() ) {
                     return NULL;
                 }
 
@@ -396,7 +447,8 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
                 ERASE_ITERATE(TConnPool, it, m_NotInUse) {
                     CConnection* t_con(*it);
 
-                    if (params.GetServerName() == t_con->ServerName()) {
+                    if (server_name == t_con->ServerName()
+                        ||  server_name == t_con->GetRequestedServer()) {
                         it = m_NotInUse.erase(it);
                         if (t_con->Refresh()) {
                             /* Future development ...
@@ -429,8 +481,39 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
                     if (pool_name == t_con->PoolName())
                         ++total_cnt;
                 }
-                if (total_cnt >= pool_max)
-                    return NULL;
+                if (total_cnt >= pool_max) {
+#ifdef NCBI_THREADS
+                    string timeout_str(params.GetParam("pool_wait_time"));
+                    double timeout_val = 0.0;
+                    if ( !timeout_str.empty()  &&  timeout_str != "default") {
+                        timeout_val = NStr::StringToDouble(timeout_str);
+                    }
+                    CTimeout timeout(timeout_val);
+                    m_PoolSemSubject = pool_name;
+                    mg.Release();
+                    if (m_PoolSem.TryWait(timeout)) {
+                        mg.Guard(m_PoolMutex);
+                        CConnection* t_con = NULL;
+                        NON_CONST_REVERSE_ITERATE(TConnPool, it, m_NotInUse) {
+                            if (*it == m_PoolSemConn) {
+                                t_con = m_PoolSemConn;
+                                m_PoolSemConn = NULL;
+                                m_NotInUse.erase((++it).base());
+                            }
+                        }
+                        if (t_con != NULL) {
+                            return MakeCDBConnection(t_con);
+                        }
+                    } else
+#endif
+                    if (params.GetParam("pool_allow_temp_overflow")
+                        == "true") {
+                        return MakePooledConnection
+                            (CDBConnParams_Unpooled(params));
+                    } else {
+                        return NULL;
+                    }
+                }
             }
         }
     }
@@ -468,6 +551,7 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
 void
 CDriverContext::CloseAllConn(void)
 {
+    CMutexGuard mg(m_PoolMutex);
     // close all connections first
     ITERATE(TConnPool, it, m_NotInUse) {
         delete *it;
@@ -482,6 +566,7 @@ CDriverContext::CloseAllConn(void)
 void
 CDriverContext::DeleteAllConn(void)
 {
+    CMutexGuard mg(m_PoolMutex);
     // close all connections first
     ITERATE(TConnPool, it, m_NotInUse) {
         delete *it;
@@ -677,6 +762,9 @@ SDBConfParams::Clear(void)
     is_pooled.clear();
     pool_name.clear();
     pool_maxsize.clear();
+    pool_idle_time.clear();
+    pool_wait_time.clear();
+    pool_allow_temp_overflow.clear();
     args.clear();
 }
 
@@ -705,7 +793,7 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
     }
     if (reg.HasEntry(section_name, "port", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fPortSet;
-        params->port = reg.GetInt(section_name, "port", 0);
+        params->port = reg.Get(section_name, "port");
     }
     if (reg.HasEntry(section_name, "database", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fDatabaseSet;
@@ -715,9 +803,29 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
         params->flags += SDBConfParams::fUsernameSet;
         params->username = reg.Get(section_name, "username");
     }
+    if (reg.HasEntry(section_name, "password_key", IRegistry::fCountCleared)) {
+        params->flags += SDBConfParams::fPasswordKeySet;
+        params->password_key_id = reg.Get(section_name, "password_key");
+    }
     if (reg.HasEntry(section_name, "password", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fPasswordSet;
         params->password = reg.Get(section_name, "password");
+        if (CNcbiEncrypt::IsEncrypted(params->password)) {
+            try {
+                params->password = CNcbiEncrypt::Decrypt(params->password);
+            } NCBI_CATCH("Password decryption for " + service_name);
+        } else if (params->password_key_id.empty()) {
+            ERR_POST(Warning
+                     << "Using unencrypted password for " + service_name);
+        }
+    }
+    if (reg.HasEntry(section_name, "password_file",
+                     IRegistry::fCountCleared)) {
+        // password and password_file are mutually exclusive, but only SDBAPI
+        // actually honors the latter, so it will take care of throwing
+        // exceptions as necessary.
+        params->flags += SDBConfParams::fPasswordFileSet;
+        params->password_file = reg.Get(section_name, "password_file");
     }
     if (reg.HasEntry(section_name, "login_timeout", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fLoginTimeoutSet;
@@ -750,6 +858,22 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
         params->flags += SDBConfParams::fPoolMaxSizeSet;
         params->pool_maxsize = reg.Get(section_name, "conn_pool_maxsize");
     }
+    if (reg.HasEntry(section_name, "conn_pool_idle_time",
+                     IRegistry::fCountCleared)) {
+        params->flags += SDBConfParams::fPoolIdleTimeSet;
+        params->pool_idle_time = reg.Get(section_name, "conn_pool_idle_time");
+    }
+    if (reg.HasEntry(section_name, "conn_pool_wait_time",
+                     IRegistry::fCountCleared)) {
+        params->flags += SDBConfParams::fPoolWaitTimeSet;
+        params->pool_wait_time = reg.Get(section_name, "conn_pool_wait_time");
+    }
+    if (reg.HasEntry(section_name, "conn_pool_allow_temp_overflow",
+                     IRegistry::fCountCleared)) {
+        params->flags += SDBConfParams::fPoolAllowTempSet;
+        params->pool_allow_temp_overflow
+            = reg.Get(section_name, "conn_pool_allow_temp_overflow");
+    }
     if (reg.HasEntry(section_name, "args", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fArgsSet;
         params->args = reg.Get(section_name, "args");
@@ -759,7 +883,7 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
 bool
 CDriverContext::SatisfyPoolMinimum(const CDBConnParams& params)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(m_PoolMutex);
 
     string pool_min_str = params.GetParam("pool_minsize");
     if (pool_min_str.empty()  ||  pool_min_str == "default")
@@ -802,7 +926,7 @@ CDriverContext::SatisfyPoolMinimum(const CDBConnParams& params)
 CDB_Connection* 
 CDriverContext::MakeConnection(const CDBConnParams& params)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     CMakeConnActualParams act_params(params);
     SDBConfParams conf_params;
@@ -813,7 +937,7 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
 
     int was_timeout = GetTimeout();
     int was_login_timeout = GetLoginTimeout();
-    CDB_Connection* t_con = NULL;
+    unique_ptr<CDB_Connection> t_con;
     try {
         string server_name = (conf_params.IsServerSet()?   conf_params.server:
                                                            params.GetServerName());
@@ -901,11 +1025,41 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
         else if (params.GetParam("is_pooled") == "default") {
             act_params.SetParam("is_pooled", "false");
         }
+        if (conf_params.IsPoolMinSizeSet())
+            act_params.SetParam("pool_minsize", conf_params.pool_minsize);
+        else if (params.GetParam("pool_minsize") == "default") {
+            act_params.SetParam("pool_minsize", "0");
+        }
         if (conf_params.IsPoolMaxSizeSet())
             act_params.SetParam("pool_maxsize", conf_params.pool_maxsize);
         else if (params.GetParam("pool_maxsize") == "default") {
             act_params.SetParam("pool_maxsize", "");
         }
+        if (conf_params.IsPoolIdleTimeSet())
+            act_params.SetParam("pool_idle_time", conf_params.pool_idle_time);
+        else if (params.GetParam("pool_idle_time") == "default") {
+            act_params.SetParam("pool_idle_time", "");
+        }
+        if (conf_params.IsPoolWaitTimeSet())
+            act_params.SetParam("pool_wait_time", conf_params.pool_wait_time);
+        else if (params.GetParam("pool_wait_time") == "default") {
+            act_params.SetParam("pool_wait_time", "0");
+        }
+        if (conf_params.IsPoolAllowTempOverflowSet()) {
+            if (conf_params.pool_allow_temp_overflow.empty()) {
+                act_params.SetParam("pool_allow_temp_overflow", "false");
+            }
+            else {
+                act_params.SetParam
+                    ("pool_allow_temp_overflow", 
+                     NStr::BoolToString(
+                         NStr::StringToBool(
+                             conf_params.pool_allow_temp_overflow)));
+            }
+        }
+        else if (params.GetParam("pool_allow_temp_overflow") == "default") {
+            act_params.SetParam("pool_allow_temp_overflow", "false");
+        }
 
         s_TransformLoginData(server_name, user_name, db_name, password);
         act_params.SetServerName(server_name);
@@ -914,13 +1068,13 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
         act_params.SetPassword(password);
 
         CRef<IDBConnectionFactory> factory = CDbapiConnMgr::Instance().GetConnectionFactory();
-        t_con = factory->MakeDBConnection(*this, act_params);
+        t_con.reset(factory->MakeDBConnection(*this, act_params));
 
-        if((!t_con && act_params.GetParam("do_not_connect") == "true")) {
-            return NULL;
-        }
+        if (t_con.get() == NULL) {
+            if (act_params.GetParam("do_not_connect") == "true") {
+                return NULL;
+            }
 
-        if (!t_con) {
             string err;
             err += "Cannot connect to the server '" + act_params.GetServerName();
             err += "' as user '" + act_params.GetUserName() + "'";
@@ -947,12 +1101,12 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
     SetTimeout(was_timeout);
     SetLoginTimeout(was_login_timeout);
 
-    return t_con;
+    return t_con.release();
 }
 
 void CDriverContext::CloseConnsForPool(const string& pool_name)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(m_PoolMutex);
 
     ITERATE(TConnPool, it, m_InUse) {
         CConnection* t_con(*it);
@@ -969,6 +1123,40 @@ void CDriverContext::CloseConnsForPool(const string& pool_name)
     }
 }
 
+
+void CDriverContext::CloseOldIdleConns(unsigned int max_closings,
+                                       const string& pool_name)
+{
+    CMutexGuard mg(m_PoolMutex);
+    if (max_closings == 0) {
+        return;
+    }
+
+    set<string> at_min;
+    CTime now(CTime::eCurrent);
+    ERASE_ITERATE (TConnPool, it, m_NotInUse) {
+        const string& pool_name_2 = (*it)->PoolName();
+        if (pool_name_2.empty()
+            ||  ( !pool_name.empty()  &&  pool_name != pool_name_2)
+            ||  at_min.find(pool_name_2) != at_min.end()
+            ||  (*it)->m_CleanupTime.IsEmpty()
+            ||  (*it)->m_CleanupTime > now) {
+            continue;
+        }
+        unsigned int n = NofConnections(TSvrRef(), pool_name_2);
+        if (n > (*it)->m_PoolMinSize) {
+            delete *it;
+            m_NotInUse.erase(it);
+            if (--max_closings == 0) {
+                break;
+            }
+        } else {
+            at_min.insert(pool_name_2);
+        }
+    }
+}
+
+
 void CDriverContext::DestroyConnImpl(CConnection* impl)
 {
     if (impl) {
@@ -979,7 +1167,7 @@ void CDriverContext::DestroyConnImpl(CConnection* impl)
 
 void CDriverContext::SetClientCharset(const string& charset)
 {
-    CMutexGuard mg(m_CtxMtx);
+    CMutexGuard mg(x_GetCtxMtx());
 
     m_ClientCharset = charset;
     m_ClientEncoding = eEncoding_Unknown;
@@ -1019,7 +1207,7 @@ void CDriverContext::UpdateConnTimeout(void) const
 }
 
 
-void CDriverContext::UpdateConnMaxTextImageSize(void) const
+void CDriverContext::UpdateConnMaxBlobSize(void) const
 {
     // Do not protect this method. It is protected.
 
@@ -1027,14 +1215,14 @@ void CDriverContext::UpdateConnMaxTextImageSize(void) const
         CConnection* t_con = *it;
         if (!t_con) continue;
 
-        t_con->SetTextImageSize(GetMaxTextImageSize());
+        t_con->SetBlobSize(GetMaxBlobSize());
     }
 
     ITERATE(TConnPool, it, m_InUse) {
         CConnection* t_con = *it;
         if (!t_con) continue;
 
-        t_con->SetTextImageSize(GetMaxTextImageSize());
+        t_con->SetBlobSize(GetMaxBlobSize());
     }
 }
 
