@@ -1,4 +1,4 @@
-/*  $Id: seq_annot_info.cpp 444088 2014-08-21 13:34:39Z ivanov $
+/*  $Id: seq_annot_info.cpp 507368 2016-07-18 21:33:41Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -58,6 +58,7 @@
 #include <objects/seq/Annot_descr.hpp>
 #include <objects/seqtable/seqtable__.hpp>
 #include <objects/seqfeat/seqfeat__.hpp>
+#include <objects/seqalign/Seq_align_set.hpp>
 #include <objmgr/seq_feat_handle.hpp>
 
 
@@ -70,13 +71,15 @@ NCBI_DEFINE_ERR_SUBCODE_X(9);
 BEGIN_SCOPE(objects)
 
 
-CSeq_annot_Info::CSeq_annot_Info(CSeq_annot& annot)
+CSeq_annot_Info::CSeq_annot_Info(CSeq_annot& annot, int chunk_id)
+    : m_ChunkId(chunk_id)
 {
     x_SetObject(annot);
 }
 
 
 CSeq_annot_Info::CSeq_annot_Info(CSeq_annot_SNP_Info& snp_annot)
+    : m_ChunkId(0)
 {
     x_SetSNP_annot_Info(snp_annot);
 }
@@ -84,8 +87,13 @@ CSeq_annot_Info::CSeq_annot_Info(CSeq_annot_SNP_Info& snp_annot)
 
 CSeq_annot_Info::CSeq_annot_Info(const CSeq_annot_Info& info,
                                  TObjectCopyMap* copy_map)
-    : TParent(info, copy_map)
+    : TParent(info, copy_map),
+      m_ChunkId(info.GetChunkId())
 {
+    if ( !copy_map ) {
+        info.x_UpdateComplete();
+        m_ChunkId = 0;
+    }
     x_SetObject(info, copy_map);
 }
 
@@ -165,7 +173,6 @@ void CSeq_annot_Info::x_DSUnmapObject(CConstRef<TObject> obj, CDataSource& ds)
 
 void CSeq_annot_Info::x_TSEAttachContents(CTSE_Info& tse)
 {
-    SetBioObjectId(tse.x_RegisterBioObject(*this));
     CRef<CSeq_annot_SNP_Info> snp_info = tse.x_GetSNP_Info(m_Object);
     if ( snp_info ) {
         _ASSERT(!m_SNP_Info);
@@ -175,6 +182,7 @@ void CSeq_annot_Info::x_TSEAttachContents(CTSE_Info& tse)
         x_AttachObject(*snp_info);
     }
     TParent::x_TSEAttachContents(tse);
+    SetBioObjectId(tse.x_RegisterBioObject(*this));
     x_UpdateName();
     x_SetDirtyAnnotIndex();
     if ( m_SNP_Info ) {
@@ -459,6 +467,8 @@ void CSeq_annot_Info::x_InitLocsList(TLocs& objs)
 
 void CSeq_annot_Info::x_InitFeatTable(TSeq_table& table)
 {
+    m_Table_Info = new CSeqTableInfo(table);
+
     _ASSERT(m_ObjectIndex.GetInfos().empty());
     if ( !CSeqTableInfo::IsGoodFeatTable(table) ) {
         // index whole Seq-table
@@ -467,14 +477,18 @@ void CSeq_annot_Info::x_InitFeatTable(TSeq_table& table)
         _ASSERT(m_ObjectIndex.GetInfos().size() == 1u);
     }
     else {
+        SAnnotTypeSelector type = m_Table_Info->GetType();
+
+        if ( IsSortedTable() ) {
+            // sorted table of small elements,
+            // index it as a single entry,
+            // then use sorted position for lookup
+            m_ObjectIndex.AddInfo(CAnnotObject_Info(*this, kWholeAnnotIndex, type));
+            return;
+        }
+
         // index each row separately
         TAnnotIndex rows = table.GetNum_rows();
-        SAnnotTypeSelector type
-            (SAnnotTypeSelector::TFeatType(table.GetFeat_type()));
-        if ( table.IsSetFeat_subtype() ) {
-            type.SetFeatSubtype
-                (SAnnotTypeSelector::TFeatSubtype(table.GetFeat_subtype()));
-        }
         for ( TAnnotIndex index = 0; index < rows; ++index ) {
             m_ObjectIndex.AddInfo(CAnnotObject_Info(*this, index, type));
         }
@@ -508,7 +522,6 @@ void CSeq_annot_Info::x_InitAnnotList(const CSeq_annot_Info& info)
         data.SetIds() = src_data.GetIds();
         break;
     case C_Data::e_Seq_table:
-        //data.SetSeq_table(src_data.S
         x_InitFeatTable(data.SetSeq_table());
         break;
     default:
@@ -824,38 +837,36 @@ void CSeq_annot_Info::x_InitGraphKeys(CTSE_Info& tse)
 }
 
 
-void CSeq_annot_Info::x_InitAlignKeys(CTSE_Info& tse)
+void CSeq_annot_Info::x_AddAlignKeys(CAnnotObject_Info& info,
+                                     const CSeq_align& align,
+                                     const CMasterSeqSegments* master,
+                                     CTSEAnnotObjectMapper& mapper)
 {
-    _ASSERT(m_ObjectIndex.GetInfos().size() >= m_Object->GetData().GetAlign().size());
-    size_t object_count = m_ObjectIndex.GetInfos().size();
-    m_ObjectIndex.ReserveMapSize(object_count);
-
-    SAnnotObject_Key key;
-    SAnnotObject_Index index;
-    CConstRef<CMasterSeqSegments> master = tse.GetMasterSeqSegments();
-    vector<CHandleRangeMap> hrmaps;
-
-    CTSEAnnotObjectMapper mapper(tse, GetName());
-
-    NON_CONST_ITERATE ( SAnnotObjectsIndex::TObjectInfos, it,
-                        m_ObjectIndex.GetInfos() ) {
-        CAnnotObject_Info& info = *it;
-        if ( info.IsRemoved() ) {
-            continue;
+    if ( align.GetSegs().IsDisc() ) {
+        // discontinued alignments should be indexed separately
+        // add keys for each sub-alignment
+        const CSeq_align_set::Tdata& sub_aligns = 
+            align.GetSegs().GetDisc().Get();
+        for ( auto& align_ref : sub_aligns ) {
+            x_AddAlignKeys(info, *align_ref, master, mapper);
         }
-        size_t keys_begin = m_ObjectIndex.GetKeys().size();
-        index.m_AnnotObject_Info = &info;
+    }
+    else {
+        vector<CHandleRangeMap> hrmaps;
+        CAnnotObject_Info::x_ProcessAlign(hrmaps, align, master);
 
-        info.GetMaps(hrmaps, master);
+        SAnnotObject_Index index;
+        index.m_AnnotObject_Info = &info;
         index.m_AnnotLocationIndex = 0;
 
         ITERATE ( vector<CHandleRangeMap>, hrmit, hrmaps ) {
             ITERATE ( CHandleRangeMap, hrit, *hrmit ) {
                 const CHandleRange& hr = hrit->second;
+                SAnnotObject_Key key;
                 key.m_Range = hr.GetOverlappingRange();
                 if ( key.m_Range.Empty() ) {
-                    ERR_POST_X(3, "Empty region in "<<GetDescription()<<" "<<
-                               MSerial_AsnText<<info.GetAlign());
+                    ERR_POST_X(3, "Empty region in "<<GetDescription()<<
+                               " "<<MSerial_AsnText<<info.GetAlign());
                     continue;
                 }
                 key.m_Handle = hrit->first;
@@ -871,6 +882,27 @@ void CSeq_annot_Info::x_InitAlignKeys(CTSE_Info& tse)
             }
             ++index.m_AnnotLocationIndex;
         }
+    }
+}
+
+
+void CSeq_annot_Info::x_InitAlignKeys(CTSE_Info& tse)
+{
+    _ASSERT(m_ObjectIndex.GetInfos().size() >= m_Object->GetData().GetAlign().size());
+    size_t object_count = m_ObjectIndex.GetInfos().size();
+    m_ObjectIndex.ReserveMapSize(object_count);
+
+    CConstRef<CMasterSeqSegments> master = tse.GetMasterSeqSegments();
+    CTSEAnnotObjectMapper mapper(tse, GetName());
+
+    NON_CONST_ITERATE ( SAnnotObjectsIndex::TObjectInfos, it,
+                        m_ObjectIndex.GetInfos() ) {
+        CAnnotObject_Info& info = *it;
+        if ( info.IsRemoved() ) {
+            continue;
+        }
+        size_t keys_begin = m_ObjectIndex.GetKeys().size();
+        x_AddAlignKeys(info, info.GetAlign(), master, mapper);
         x_UpdateObjectKeys(info, keys_begin);
     }
 }
@@ -966,9 +998,6 @@ bool CSeq_annot_Info::IsTableFeatPartial(const CAnnotObject_Info& info) const
 
 void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
 {
-    const CSeq_table& feat_table = m_Object->GetData().GetSeq_table();
-    m_Table_Info = new CSeqTableInfo(feat_table);
-    
     CConstRef<CMasterSeqSegments> master = tse.GetMasterSeqSegments();
 
     if ( !m_Table_Info->IsFeatTable() ) {
@@ -1024,7 +1053,6 @@ void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
     }
 
     size_t object_count = m_ObjectIndex.GetInfos().size();
-    _ASSERT(object_count == size_t(feat_table.GetNum_rows()));
     m_ObjectIndex.ReserveMapSize(object_count);
 
     SAnnotObject_Key key;
@@ -1036,6 +1064,21 @@ void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
         m_ObjectIndex.GetInfos().begin();
     for ( size_t row = 0; row < object_count; ++row, ++it ) {
         CAnnotObject_Info& info = *it;
+        if ( info.GetAnnotIndex() == kWholeAnnotIndex ) {
+            size_t keys_begin = m_ObjectIndex.GetKeys().size();
+            index.m_AnnotObject_Info = &info;
+            index.m_AnnotLocationIndex = 0;
+            index.m_Flags = index.fStrand_both;
+            index.m_HandleRange.Reset();
+            CConstRef<CSeq_loc> loc = m_Table_Info->GetTableLocation();
+            _ASSERT(loc && loc->IsInt()); // checked in x_InitFeatTable()
+            key.m_Handle = CSeq_id_Handle::GetHandle(loc->GetInt().GetId());
+            key.m_Range = loc->GetTotalRange();
+            x_Map(mapper, key, index);
+            x_UpdateObjectKeys(info, keys_begin);
+            continue;
+        }
+
         if ( info.IsRemoved() ) {
             continue;
         }
@@ -1908,5 +1951,95 @@ void CSeq_annot_Info::ClearFeatIds(TAnnotIndex index,
 }
 
 
+bool CSeq_annot_Info::IsSortedTable(void) const
+{
+    return m_Table_Info && m_Table_Info->IsSorted();
+}
+
+
+bool CSeq_annot_Info::TableFeat_HasLabel(TAnnotIndex index) const
+{
+    if ( m_SNP_Info ) {
+        return m_SNP_Info->HasLabel(index);
+    }
+    else {
+        _ASSERT(m_Table_Info);
+        return m_Table_Info->HasLabel(index);
+    }
+}
+
+
+string CSeq_annot_Info::TableFeat_GetLabel(TAnnotIndex index) const
+{
+    if ( m_SNP_Info ) {
+        return m_SNP_Info->GetLabel(index);
+    }
+    else {
+        _ASSERT(m_Table_Info);
+        return m_Table_Info->GetLabel(index);
+    }
+}
+
+
+CSeq_annot_SortedIter
+CSeq_annot_Info::StartSortedIterator(CRange<TSeqPos> range) const
+{
+    CSeq_annot_SortedIter iter;
+    _ASSERT(IsSortedTable());
+    TSeqPos max_len = m_Table_Info->GetSortedMaxLength();
+    _ASSERT(max_len > 0);
+    TSeqPos min_pos = max(range.GetFrom(), max_len-1) - (max_len-1);
+    size_t num_rows = m_Table_Info->GetNumRows();
+    size_t a = 0, b = num_rows;
+    while ( b-a > 1 ) {
+        size_t m = a + (b-a)/2;
+        TSeqPos pos = m_Table_Info->GetLocationFrom(m);
+        if ( pos < min_pos ) {
+            a = m;
+        }
+        else {
+            b = m;
+        }
+    }
+    iter.m_Table_Info = m_Table_Info;
+    iter.m_RequestRange = range;
+    iter.m_ObjectRow = a;
+    iter.m_NumRows = num_rows;
+    iter.x_Settle();
+    return iter;
+}
+
+
+inline
+bool CSeq_annot_SortedIter::x_Valid(void)
+{
+    if ( m_Table_Info->RowIsDisabled(m_ObjectRow) ) {
+        return false;
+    }
+    m_ObjectRange = m_Table_Info->GetLocationRange(m_ObjectRow);
+    if ( m_ObjectRange.GetFrom() >= m_RequestRange.GetToOpen() ) {
+        m_NumRows = m_ObjectRow;
+        return true;
+    }
+    return m_ObjectRange.GetToOpen() > m_RequestRange.GetFrom();
+}
+
+
+void CSeq_annot_SortedIter::x_Settle(void)
+{
+    while ( *this && !x_Valid() ) {
+        ++m_ObjectRow;
+    }
+}
+
+
+bool CSeq_annot_Info::MatchBitFilter(const SAnnotSelector& sel,
+                                     const CSeq_annot_SortedIter& iter) const
+{
+    _ASSERT(IsSortedTable());
+    return m_Table_Info->MatchBitFilter(sel, iter.GetRow());
+}
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
