@@ -1,4 +1,4 @@
-/*  $Id: snp_annot_info.cpp 497230 2016-04-05 13:28:50Z ivanov $
+/*  $Id: snp_annot_info.cpp 500967 2016-05-10 14:39:30Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -782,6 +782,26 @@ void SSNP_Info::UpdateSeq_feat(CRef<CSeq_feat>& feat_ref,
 }
 
 
+string SSNP_Info::GetLabel(const CSeq_annot_SNP_Info& annot) const
+{
+    CNcbiOstrstream str;
+    size_t count = 0;
+    for (; count < kMax_AllelesCount; ++count) {
+        TAlleleIndex allele_index = m_AllelesIndices[count];
+        if ( allele_index == kNo_AlleleIndex ) {
+            break;
+        }
+        str << (count == 0? '/': ' ');
+        str << "replace";
+        const string& allele = annot.x_GetAllele(allele_index);
+        if ( !allele.empty() ) {
+            str << '=' << allele;
+        }
+    }
+    return CNcbiOstrstreamToString(str);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // CSeq_annot_SNP_Info
 /////////////////////////////////////////////////////////////////////////////
