@@ -1,4 +1,4 @@
-/*  $Id: writedb_impl.cpp 495464 2016-03-17 16:30:45Z ivanov $
+/*  $Id: writedb_impl.cpp 513290 2016-09-09 11:34:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,11 +30,6 @@
 /// @file writedb_impl.cpp
 /// Implementation for the CWriteDB_Impl class.
 /// class for WriteDB.
-
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: writedb_impl.cpp 495464 2016-03-17 16:30:45Z ivanov $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
-
 #include <ncbi_pch.hpp>
 #include <objtools/blast/seqdb_writer/writedb_error.hpp>
 #include <objtools/blast/seqdb_reader/seqdbexpert.hpp>
@@ -648,13 +643,26 @@ x_SetDeflinesFromBinary(const string                   & bin_hdr,
 
 static bool s_UseFastaReaderDeflines(CConstRef<CBioseq> & bioseq, CConstRef<CBlast_def_line_set> & deflines)
 {
+    CMetaRegistry::SEntry sentry =
+        CMetaRegistry::Load("ncbi", CMetaRegistry::eName_RcOrIni);
+    bool long_seqid = sentry.registry ?
+        sentry.registry->HasEntry("BLAST", "LONG_SEQID") : false;
+
 	if(deflines.Empty())
 		return false;
 
 	const CSeq_id * bioseq_id = bioseq->GetNonLocalId();
 
-	if(bioseq_id == NULL)
-		return true;
+	if(bioseq_id == NULL ||
+       // For bare pdb and prf ids go with the one from defline.
+       // This is to parse bare ids as local ones. The bare pdb ids are pdb in
+       // bioseq (parsed by CFastaReader), but local in deflines (parsed by
+       // CSeq_id).
+       (!long_seqid && (bioseq_id->IsPdb() ||
+                        bioseq_id->IsPrf() ||
+                        bioseq_id->IsPir()))) {
+        return true;
+       }
 
 	// Bioseq has non-local id, make sure at least one id is non-local from CFastaReader
 	// defline
@@ -1562,6 +1570,11 @@ x_GetFastaReaderDeflines(const CBioseq                  & bioseq,
         return;
     }
 
+    CMetaRegistry::SEntry sentry =
+        CMetaRegistry::Load("ncbi", CMetaRegistry::eName_RcOrIni);
+    bool long_seqids = sentry.registry ?
+        sentry.registry->HasEntry("BLAST", "LONG_SEQID") : false;
+
     string fasta;
 
     // Scan the CBioseq for the CFastaReader user object.
@@ -1681,11 +1694,29 @@ x_GetFastaReaderDeflines(const CBioseq                  & bioseq,
 
             // Parse '|' seperated ids.
             list< CRef<CSeq_id> > seqids;
-            if ( ids.find('|') == NPOS
-              || !isalpha((unsigned char)(ids[0]))) {
+            if ( (ids.find('|') == NPOS && long_seqids)
+                 || !isalpha((unsigned char)(ids[0]))) {
+
                  seqids.push_back(CRef<CSeq_id> (new CSeq_id(CSeq_id::e_Local, ids)));
             } else {
                  CSeq_id::ParseFastaIds(seqids, ids);
+
+                 if (!long_seqids) {
+
+                     // If accession's molecule type is different than
+                     // expected, change sequence id to local. CFastaReader
+                     // cannot distingush between bare pir protein ids genbank
+                     // nucleotide ids.
+                     for (auto& it: seqids) {
+                         CSeq_id::EAccessionInfo info = it->IdentifyAccession();
+                         if (!it->IsLocal() && !it->IsGi() &&
+                             bioseq.IsAa() == !!(info & CSeq_id::fAcc_nuc)) {
+
+                             string label = it->GetSeqIdString();
+                             it.Reset(new CSeq_id(CSeq_id::e_Local, label));
+                         }
+                     }
+                 }
             }
 
             // Build the actual defline.
