@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.cpp 458457 2015-02-05 13:19:52Z ivanov $
+/*  $Id: bed_reader.cpp 503967 2016-06-09 18:00:51Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -84,6 +84,7 @@
 #include <objtools/readers/read_util.hpp>
 #include <objtools/readers/reader_exception.hpp>
 #include <objtools/readers/line_error.hpp>
+#include <objtools/readers/track_data.hpp>
 #include <objtools/readers/message_listener.hpp>
 #include <objtools/readers/bed_reader.hpp>
 #include <objtools/error_codes.hpp>
@@ -122,7 +123,7 @@ CBedReader::~CBedReader()
 CRef< CSeq_annot >
 CBedReader::ReadSeqAnnot(
     CNcbiIstream& istr,
-    IMessageListener* pMessageListener ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------
 {
     CStreamLineReader lr( istr );
@@ -133,11 +134,9 @@ CBedReader::ReadSeqAnnot(
 CRef< CSeq_annot >
 CBedReader::ReadSeqAnnot(
     ILineReader& lr,
-    IMessageListener* pEC ) 
+    ILineErrorListener* pEC ) 
 //  ----------------------------------------------------------------------------                
 {
-    const size_t MAX_RECORDS = 100000;
-
     xProgressInit(lr);
 
     CRef<CSeq_annot> annot;
@@ -177,7 +176,7 @@ CBedReader::ReadSeqAnnot(
 
         //  parse
         vector<string> fields;
-        NStr::Tokenize( record_copy, " \t", fields, NStr::eMergeDelims );
+        NStr::Split( record_copy, " \t", fields, NStr::eMergeDelims );
         try {
             xCleanColumnValues(fields);
         }
@@ -189,9 +188,6 @@ CBedReader::ReadSeqAnnot(
             ++featureCount;
             continue;
         }
-        if (tbl.size() >= MAX_RECORDS) {
-            break;
-        }
     }
     //  Only return a valid object if there was at least one feature
     if (0 == featureCount) {
@@ -216,7 +212,7 @@ CBedReader::ReadSeqAnnot(
 CRef< CSerialObject >
 CBedReader::ReadObject(
     ILineReader& lr,
-    IMessageListener* pMessageListener ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------                
 { 
     CRef<CSerialObject> object( 
@@ -228,7 +224,7 @@ CBedReader::ReadObject(
 bool
 CBedReader::xParseTrackLine(
     const string& strLine,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     if ( ! NStr::StartsWith( strLine, "track" ) ) {
@@ -280,7 +276,7 @@ bool CBedReader::xParseFeature(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int featureCount,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     static int count = 0;
@@ -304,6 +300,9 @@ bool CBedReader::xParseFeature(
     if (m_iFlags & CBedReader::fThreeFeatFormat) {
         return xParseFeatureThreeFeatFormat(fields, annot, 3*featureCount, pEC);
     }
+    else if (m_iFlags & CBedReader::fDirectedFeatureModel) {
+        return xParseFeatureGeneModelFormat(fields, annot, 3*featureCount, pEC);
+    }
     else {
         return xParseFeatureUserFormat(fields, annot, pEC);
     }
@@ -314,7 +313,7 @@ bool CBedReader::xParseFeatureThreeFeatFormat(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int baseId,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     if (!xAppendFeatureChrom(fields, annot, baseId, pEC)) {
@@ -331,12 +330,34 @@ bool CBedReader::xParseFeatureThreeFeatFormat(
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CBedReader::xParseFeatureGeneModelFormat(
+    const vector<string>& fields,
+    CRef<CSeq_annot>& annot,
+    unsigned int baseId,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    if (!xAppendFeatureGene(fields, annot, baseId, pEC)) {
+        return false;
+    }
+    if (xContainsCdsFeature(fields)  &&  
+            !xAppendFeatureCds(fields, annot, baseId, pEC)) {
+        return false;
+    }
+    if (xContainsRnaFeature(fields)  &&
+            !xAppendFeatureRna(fields, annot, baseId, pEC)) {
+        return false;
+    }
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 bool CBedReader::xAppendFeatureChrom(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int baseId,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     CSeq_annot::C_Data::TFtable& ftable = annot->SetData().SetFtable();
@@ -358,12 +379,39 @@ bool CBedReader::xAppendFeatureChrom(
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CBedReader::xAppendFeatureGene(
+    const vector<string>& fields,
+    CRef<CSeq_annot>& annot,
+    unsigned int baseId,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    CSeq_annot::C_Data::TFtable& ftable = annot->SetData().SetFtable();
+    CRef<CSeq_feat> feature;
+    feature.Reset(new CSeq_feat);
+    try {
+        ////xSetFeatureTitle(feature, fields);
+        xSetFeatureLocationGene(feature, fields);
+        xSetFeatureIdsGene(feature, fields, baseId);
+        xSetFeatureBedData(feature, fields);
+    }
+    catch(CObjReaderLineException& err) {
+        //m_currentId.clear();
+        ProcessError(err, pEC);
+        return false;
+    }
+    ftable.push_back(feature);
+    m_currentId = fields[0];
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 bool CBedReader::xAppendFeatureThick(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int baseId,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     CSeq_annot::C_Data::TFtable& ftable = annot->SetData().SetFtable();
@@ -385,18 +433,44 @@ bool CBedReader::xAppendFeatureThick(
 }
 
 //  ----------------------------------------------------------------------------
-bool CBedReader::xAppendFeatureBlock(
+bool CBedReader::xAppendFeatureCds(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int baseId,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     CSeq_annot::C_Data::TFtable& ftable = annot->SetData().SetFtable();
     CRef<CSeq_feat> feature;
     feature.Reset(new CSeq_feat);
     try {
-        //xSetFeatureTitle(feature, fields);
+        ////xSetFeatureTitle(feature, fields);
+        xSetFeatureLocationCds(feature, fields);
+        xSetFeatureIdsCds(feature, fields, baseId);
+        xSetFeatureBedData(feature, fields);
+    }
+    catch(CObjReaderLineException& err) {
+        //m_currentId.clear();
+        ProcessError(err, pEC);
+        return false;
+    }
+    ftable.push_back(feature);
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+bool CBedReader::xAppendFeatureBlock(
+    const vector<string>& fields,
+    CRef<CSeq_annot>& annot,
+    unsigned int baseId,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    CSeq_annot::C_Data::TFtable& ftable = annot->SetData().SetFtable();
+    CRef<CSeq_feat> feature;
+    feature.Reset(new CSeq_feat);
+    try {
+        ////xSetFeatureTitle(feature, fields);
         xSetFeatureLocationBlock(feature, fields);
         xSetFeatureIdsBlock(feature, fields, baseId);
         xSetFeatureBedData(feature, fields);
@@ -409,11 +483,39 @@ bool CBedReader::xAppendFeatureBlock(
     ftable.push_back(feature);
     return true;
 }
+
+//  ----------------------------------------------------------------------------
+bool CBedReader::xAppendFeatureRna(
+    const vector<string>& fields,
+    CRef<CSeq_annot>& annot,
+    unsigned int baseId,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    CSeq_annot::C_Data::TFtable& ftable = annot->SetData().SetFtable();
+    CRef<CSeq_feat> feature;
+    feature.Reset(new CSeq_feat);
+    try {
+        //xSetFeatureTitle(feature, fields);
+        xSetFeatureLocationRna(feature, fields);
+        xSetFeatureIdsRna(feature, fields, baseId);
+        xSetFeatureBedData(feature, fields);
+    }
+    catch(CObjReaderLineException& err) {
+        //m_currentId.clear();
+        ProcessError(err, pEC);
+        return false;
+    }
+    ftable.push_back(feature);
+    return true;
+}
+
+
 //  ----------------------------------------------------------------------------
 bool CBedReader::xParseFeatureUserFormat(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     //  assign
@@ -507,6 +609,21 @@ void CBedReader::xSetFeatureLocationChrom(
     exts.push_back(pBed);
 }
 
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureLocationGene(
+    CRef<CSeq_feat>& feature,
+    const vector<string>& fields)
+//  ----------------------------------------------------------------------------
+{
+    x_SetFeatureLocation(feature, fields);
+
+    CRef<CUser_object> pBed(new CUser_object());
+    pBed->SetType().SetStr("BED");
+    pBed->AddField("location", "chrom");
+    CSeq_feat::TExts& exts = feature->SetExts();
+    exts.push_back(pBed);
+}
+
 //  ----------------------------------------------------------------------------
 void CBedReader::xSetFeatureLocationThick(
     CRef<CSeq_feat>& feature,
@@ -569,6 +686,68 @@ void CBedReader::xSetFeatureLocationThick(
     exts.push_back(pBed);
 }
 
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureLocationCds(
+    CRef<CSeq_feat>& feature,
+    const vector<string>& fields)
+//  ----------------------------------------------------------------------------
+{
+    CRef<CSeq_loc> location(new CSeq_loc);
+    int from, to;
+    from = to = -1;
+
+    //already established: We got at least three columns
+    try {
+        from = NStr::StringToInt(fields[6]);
+    }
+    catch (std::exception&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad \"ThickStart\" value." ) );
+        pErr->Throw();
+    }
+    try {
+        to = NStr::StringToInt(fields[7]);
+    }
+    catch (std::exception&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad \"ThickStop\" value.") );
+        pErr->Throw();
+    }
+    if (from == to) {
+        location->SetPnt().SetPoint(from);
+    }
+    else if (from < to) {
+        location->SetInt().SetFrom(from);
+        location->SetInt().SetTo(to);
+    }
+    else if (from > to) {
+        //below: flip commenting to switch from null locations to impossible 
+        // intervals
+        //location->SetInt().SetFrom(from);
+        //location->SetInt().SetTo(to);
+        location->SetNull();
+    }
+
+    if (!location->IsNull()) {
+        location->SetStrand(xGetStrand(fields));
+    }
+    CRef<CSeq_id> id = CReadUtil::AsSeqId(fields[0], m_iFlags, false);
+    location->SetId(*id);
+    feature->SetLocation(*location);
+
+    CRef<CUser_object> pBed(new CUser_object());
+    pBed->SetType().SetStr("BED");
+    pBed->AddField("location", "thick");
+    CSeq_feat::TExts& exts = feature->SetExts();
+    exts.push_back(pBed);
+}
+
 //  ----------------------------------------------------------------------------
 ENa_strand CBedReader::xGetStrand(
     const vector<string>& fields) const
@@ -605,7 +784,7 @@ void CBedReader::xSetFeatureLocationBlock(
     {{
         blockSizes.reserve(blockCount);
         vector<string> vals; 
-        NStr::Tokenize(fields[10], ",", vals);
+        NStr::Split(fields[10], ",", vals);
         if (vals.back() == "") {
             vals.erase(vals.end()-1);
         }
@@ -635,7 +814,105 @@ void CBedReader::xSetFeatureLocationBlock(
         blockStarts.reserve(blockCount);
         vector<string> vals; 
         size_t baseStart = NStr::StringToUInt(fields[1]);
-        NStr::Tokenize(fields[11], ",", vals);
+        NStr::Split(fields[11], ",", vals);
+        if (vals.back() == "") {
+            vals.erase(vals.end()-1);
+        }
+        if (vals.size() != blockCount) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Invalid data line: Bad value count in \"blockStarts\"." ) );
+            pErr->Throw();
+        }
+        try {
+            for (size_t i=0; i < blockCount; ++i) {
+                blockStarts.push_back(baseStart + NStr::StringToUInt(vals[i]));
+            }
+        }
+        catch (std::exception&) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Invalid data line: Malformed \"blockStarts\" column." ) );
+            pErr->Throw();
+        }
+    }}
+
+    CPacked_seqint& location = feature->SetLocation().SetPacked_int();
+    
+    ENa_strand strand = xGetStrand(fields);
+    CRef<CSeq_id> pId = CReadUtil::AsSeqId(fields[0], m_iFlags, false);
+
+    bool negative = fields[5] == "-";
+
+    CPacked_seqint::Tdata& blocks = location.Set();
+
+    for (size_t i=0; i < blockCount; ++i) {
+        CRef<CSeq_interval> pInterval(new CSeq_interval);
+        pInterval->SetId(*pId);
+        pInterval->SetFrom(blockStarts[i]);
+        pInterval->SetTo(blockStarts[i] + blockSizes[i]);
+        pInterval->SetStrand(strand);
+        if (negative)
+            blocks.insert(blocks.begin(), pInterval);
+        else
+            blocks.push_back(pInterval);
+    }
+
+    CRef<CUser_object> pBed(new CUser_object());
+    pBed->SetType().SetStr("BED");
+    pBed->AddField("location", "block");
+    CSeq_feat::TExts& exts = feature->SetExts();
+    exts.push_back(pBed);
+}
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureLocationRna(
+    CRef<CSeq_feat>& feature,
+    const vector<string>& fields)
+//  ----------------------------------------------------------------------------
+{
+    //already established: there are sufficient columns to do this
+    size_t blockCount = NStr::StringToUInt(fields[9]);
+    vector<size_t> blockSizes;
+    vector<size_t> blockStarts;
+    {{
+        blockSizes.reserve(blockCount);
+        vector<string> vals; 
+        NStr::Split(fields[10], ",", vals);
+        if (vals.back() == "") {
+            vals.erase(vals.end()-1);
+        }
+        if (vals.size() != blockCount) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Invalid data line: Bad value count in \"blockSizes\"." ) );
+            pErr->Throw();
+        }
+        try {
+            for (size_t i=0; i < blockCount; ++i) {
+                blockSizes.push_back(NStr::StringToUInt(vals[i]));
+            }
+        }
+        catch (std::exception&) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Invalid data line: Malformed \"blockSizes\" column." ) );
+            pErr->Throw();
+        }
+    }}
+    {{
+        blockStarts.reserve(blockCount);
+        vector<string> vals; 
+        size_t baseStart = NStr::StringToUInt(fields[1]);
+        NStr::Split(fields[11], ",", vals);
         if (vals.back() == "") {
             vals.erase(vals.end()-1);
         }
@@ -717,6 +994,17 @@ void CBedReader::xSetFeatureIdsChrom(
     }
 }
 
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureIdsGene(
+    CRef<CSeq_feat>& feature,
+    const vector<string>& fields,
+    unsigned int baseId)
+//  ----------------------------------------------------------------------------
+{
+    baseId++; //0-based to 1-based
+    feature->SetId().SetLocal().SetId(baseId);
+}
+
 //  ----------------------------------------------------------------------------
 void CBedReader::xSetFeatureIdsThick(
     CRef<CSeq_feat>& feature,
@@ -742,6 +1030,31 @@ void CBedReader::xSetFeatureIdsThick(
     }
 }
 
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureIdsCds(
+    CRef<CSeq_feat>& feature,
+    const vector<string>& fields,
+    unsigned int baseId)
+//  ----------------------------------------------------------------------------
+{
+    baseId++; //0-based to 1-based
+    feature->SetId().SetLocal().SetId(baseId+1);
+
+    //CRef<CFeat_id> pIdChrom(new CFeat_id);
+    //pIdChrom->SetLocal().SetId(baseId);
+    //CRef<CSeqFeatXref> pXrefChrom(new CSeqFeatXref);
+    //pXrefChrom->SetId(*pIdChrom);  
+    //feature->SetXref().push_back(pXrefChrom);
+
+    if (xContainsBlockFeature(fields)) {
+        CRef<CFeat_id> pIdBlock(new CFeat_id);
+        pIdBlock->SetLocal().SetId(baseId+2);
+        CRef<CSeqFeatXref> pXrefBlock(new CSeqFeatXref);
+        pXrefBlock->SetId(*pIdBlock);  
+        feature->SetXref().push_back(pXrefBlock);   
+    }
+}
+
 //  ----------------------------------------------------------------------------
 void CBedReader::xSetFeatureIdsBlock(
     CRef<CSeq_feat>& feature,
@@ -758,11 +1071,38 @@ void CBedReader::xSetFeatureIdsBlock(
     pXrefChrom->SetId(*pIdChrom);  
     feature->SetXref().push_back(pXrefChrom);
 
-    CRef<CFeat_id> pIdBlock(new CFeat_id);
-    pIdBlock->SetLocal().SetId(baseId+1);
-    CRef<CSeqFeatXref> pXrefBlock(new CSeqFeatXref);
-    pXrefBlock->SetId(*pIdBlock);  
-    feature->SetXref().push_back(pXrefBlock);   
+    if (xContainsThickFeature(fields)) {
+        CRef<CFeat_id> pIdThick(new CFeat_id);
+        pIdThick->SetLocal().SetId(baseId+1);
+        CRef<CSeqFeatXref> pXrefBlock(new CSeqFeatXref);
+        pXrefBlock->SetId(*pIdThick);  
+        feature->SetXref().push_back(pXrefBlock);   
+    }
+}
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureIdsRna(
+    CRef<CSeq_feat>& feature,
+    const vector<string>& fields,
+    unsigned int baseId)
+//  ----------------------------------------------------------------------------
+{
+    baseId++; //0-based to 1-based
+    feature->SetId().SetLocal().SetId(baseId+2);
+
+    CRef<CFeat_id> pIdChrom(new CFeat_id);
+    pIdChrom->SetLocal().SetId(baseId);
+    CRef<CSeqFeatXref> pXrefChrom(new CSeqFeatXref);
+    pXrefChrom->SetId(*pIdChrom);  
+    feature->SetXref().push_back(pXrefChrom);
+
+    //if (xContainsThickFeature(fields)) {
+    //    CRef<CFeat_id> pIdThick(new CFeat_id);
+    //    pIdThick->SetLocal().SetId(baseId+1);
+    //    CRef<CSeqFeatXref> pXrefBlock(new CSeqFeatXref);
+    //    pXrefBlock->SetId(*pIdThick);  
+    //    feature->SetXref().push_back(pXrefBlock);   
+    //}
 }
 
 //  ----------------------------------------------------------------------------
@@ -779,6 +1119,214 @@ void CBedReader::xSetFeatureTitle(
     }
 }
 
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureScore(
+    CRef<CUser_object> pDisplayData,
+    const vector<string>& fields )
+//  ----------------------------------------------------------------------------
+{
+    string trackUseScore = m_pTrackDefaults->ValueOf("useScore");
+    if (fields.size() < 5  || trackUseScore == "1") {
+        //record does not carry score information
+        return;
+    }
+
+    int int_score = NStr::StringToInt(fields[4], NStr::fConvErr_NoThrow );
+    double d_score = 0;
+
+    if (int_score == 0 && fields[4].compare("0") != 0) {
+        try {
+            d_score = NStr::StringToDouble(fields[4]);
+        }
+        catch(std::exception&) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Invalid data line: Bad \"score\" value.") );
+            pErr->Throw();
+        }
+    }
+
+    if (d_score < 0 || int_score < 0) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad \"score\" value.") );
+        pErr->Throw();
+    }
+    else if (d_score > 0) {
+        pDisplayData->AddField("score", d_score);
+    }
+    else {
+        pDisplayData->AddField("score", int_score);
+    }
+}
+
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureColor(
+    CRef<CUser_object> pDisplayData,
+    const vector<string>& fields )
+//  ----------------------------------------------------------------------------
+{
+    //1: if track line itemRgb is set, try that first:
+    string trackItemRgb = m_pTrackDefaults->ValueOf("itemRgb");
+    if (trackItemRgb == "On"  &&  fields.size() >= 9) {
+        string featItemRgb = fields[8];
+        if (featItemRgb != ".") {
+            xSetFeatureColorFromItemRgb(pDisplayData, featItemRgb);
+            return;
+        }
+    }
+
+    //2: if track useScore is set, try that next:
+    string trackUseScore = m_pTrackDefaults->ValueOf("useScore");
+    if (trackUseScore == "1"  && fields.size() >= 5) {
+        string featScore = fields[4];
+        if (featScore != ".") {    
+            xSetFeatureColorFromScore(pDisplayData, featScore);
+            return; 
+        }
+    }
+
+    //3: if track colorByStrand is set, try that next:
+    string trackColorByStrand = m_pTrackDefaults->ValueOf("colorByStrand");
+    if (!trackColorByStrand.empty()  && fields.size() >= 6) {
+        ENa_strand strand = 
+            (fields[5] == "-") ? eNa_strand_minus : eNa_strand_plus;
+        xSetFeatureColorByStrand(pDisplayData, trackColorByStrand, strand);
+        return;
+    }
+    //4: if none of the track color attributes are set, attempt feature itemRgb:
+    if (fields.size() >= 9) {
+        string featItemRgb = fields[8];
+        if (featItemRgb != ".") {
+            xSetFeatureColorFromItemRgb(pDisplayData, featItemRgb);
+            return;
+        }
+    }
+    
+    //5: if still here, assign default color:
+    xSetFeatureColorDefault(pDisplayData);
+}
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureColorDefault(
+    CRef<CUser_object> pDisplayData)
+//  ----------------------------------------------------------------------------
+{
+    const string colorDefault("0 0 0");
+    pDisplayData->AddField("color", colorDefault);
+}
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureColorByStrand(
+    CRef<CUser_object> pDisplayData,
+    const string& trackColorByStrand,
+    ENa_strand strand)
+//  ----------------------------------------------------------------------------
+{
+    try {
+        string colorPlus, colorMinus;
+        NStr::SplitInTwo(trackColorByStrand, " ", colorPlus, colorMinus);
+        string useColor = (strand == eNa_strand_minus) ? colorMinus : colorPlus;
+        xSetFeatureColorFromItemRgb(pDisplayData, useColor);
+    }
+    catch (std::exception&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid track line: Bad colorByStrand value.") );
+        pErr->Throw();
+    }
+}
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureColorFromScore(
+    CRef<CUser_object> pDisplayData,
+    const string& featScore )
+//  ----------------------------------------------------------------------------
+{
+    int score;
+    try {
+        score = NStr::StringToInt(featScore);
+    }
+    catch (const std::exception&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad score value to be used for color.") );
+        pErr->Throw();
+    }
+    if (score < 0  ||  1000 < score) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad score value to be used for color.") );
+        pErr->Throw();
+    }
+    string greyValue  = NStr::IntToString(255 - (score/4));
+    vector<string> srgb{ greyValue, greyValue, greyValue};
+    string rgbValue = NStr::Join(srgb, " ");
+    pDisplayData->AddField("color", rgbValue);
+}
+
+//  ----------------------------------------------------------------------------
+void CBedReader::xSetFeatureColorFromItemRgb(
+    CRef<CUser_object> pDisplayData,
+    const string& itemRgb )
+//  ----------------------------------------------------------------------------
+{
+    vector<string> srgb;
+    if (itemRgb == "0") {
+        srgb.push_back("0");
+        srgb.push_back("0");
+        srgb.push_back("0");
+    }
+    else {
+        NStr::Split(itemRgb, ",", srgb);
+    }
+    if (srgb.size() != 3) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad color value.") );
+        pErr->Throw();
+    }
+    try {
+        for (int i=0; i < 3; i++)
+        {
+           int x = NStr::StringToInt(srgb[i]);
+           if (x<0 || x>255) {
+                AutoPtr<CObjReaderLineException> pErr(
+                    CObjReaderLineException::Create(
+                    eDiag_Error,
+                    0,
+                    "Invalid data line: Bad color value.") );
+                pErr->Throw();
+           }
+
+        }
+    }
+    catch(const std::exception&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad color value.") );
+        pErr->Throw();
+    }
+    string rgbValue = NStr::Join(srgb, " ");
+    pDisplayData->AddField("color", rgbValue);
+}
+
 //  ----------------------------------------------------------------------------
 void CBedReader::xSetFeatureBedData(
     CRef<CSeq_feat>& feature,
@@ -794,99 +1342,13 @@ void CBedReader::xSetFeatureBedData(
 	}
     
     CRef<CUser_object> pDisplayData(new CUser_object());
-    if (fields.size() < 5  ||  fields[4] == ".") {
-        return;
-    }
 
     CSeq_feat::TExts& exts = feature->SetExts();
     pDisplayData->SetType().SetStr("DisplaySettings");
     exts.push_front(pDisplayData);
 
-    int int_score = NStr::StringToInt(fields[4], NStr::fConvErr_NoThrow );
-    double d_score = 0;
-
-    if (int_score == 0 && fields[4].compare("0") != 0)
-    {
-        try {
-            d_score = NStr::StringToDouble(fields[4]);
-        }
-        catch(std::exception&) {
-            AutoPtr<CObjReaderLineException> pErr(
-                CObjReaderLineException::Create(
-                eDiag_Error,
-                0,
-                "Invalid data line: Bad \"score\" value.") );
-            pErr->Throw();
-        }
-    }
-
-    if (d_score < 0 || int_score < 0)
-    {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Invalid data line: Bad \"score\" value.") );
-        pErr->Throw();
-    }
-    else
-    if (d_score > 0)
-    {
-        pDisplayData->AddField("score", d_score);
-    }
-    else
-    {
-        pDisplayData->AddField("score", int_score);
-    }
-
-    if (fields.size() < 9) {
-        return;
-    }
-    vector<string> srgb;
-    if (fields[8] == "0"  ||  fields[8] == ".") {
-        srgb.push_back("0");
-        srgb.push_back("0");
-        srgb.push_back("0");
-    }
-    else {
-        NStr::Tokenize(fields[8], ",", srgb);
-    }
-    if (srgb.size() != 3)
-    {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Invalid data line: Bad color value.") );
-        pErr->Throw();
-    }
-    try
-    {
-        for (int i=0; i < 3; i++)
-        {
-           int x = NStr::StringToInt(srgb[i]);
-           if (x<0 || x>255) {
-                AutoPtr<CObjReaderLineException> pErr(
-                    CObjReaderLineException::Create(
-                    eDiag_Error,
-                    0,
-                    "Invalid data line: Bad color value.") );
-                pErr->Throw();
-           }
-
-        }
-    }
-    catch(const std::exception&)
-    {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Invalid data line: Bad color value.") );
-        pErr->Throw();
-    }
-    string rgbValue = NStr::Join(srgb, " ");
-    pDisplayData->AddField("color", rgbValue);
+    xSetFeatureScore(pDisplayData, fields);
+    xSetFeatureColor(pDisplayData, fields);
 }
 
 //  ----------------------------------------------------------------------------
@@ -1008,7 +1470,7 @@ bool
 CBedReader::ReadTrackData(
     ILineReader& lr,
     CRawBedTrack& rawdata,
-    IMessageListener* pMessageListener)
+    ILineErrorListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
     if (m_CurBatchSize == m_MaxBatchSize) {
@@ -1037,7 +1499,7 @@ bool
 CBedReader::xReadBedRecordRaw(
     const string& line,
     CRawBedRecord& record,
-    IMessageListener* pMessageListener)
+    ILineErrorListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
     if (line == "browser"  || NStr::StartsWith(line, "browser ")
@@ -1054,7 +1516,7 @@ CBedReader::xReadBedRecordRaw(
 	NStr::TruncateSpacesInPlace(linecopy);
 
     //  parse
-    NStr::Tokenize( linecopy, " \t", columns, NStr::eMergeDelims );
+    NStr::Split( linecopy, " \t", columns, NStr::eMergeDelims );
     try {
         xCleanColumnValues(columns);
     }
@@ -1143,7 +1605,59 @@ CBedReader::xContainsThickFeature(
     const vector<string>& fields) const
 //  ----------------------------------------------------------------------------
 {
-    return (fields.size() >= 8);
+    if (fields.size() < 8) {
+        return false;
+    }
+
+    int start = -1, from = -1, to = -1;
+    try {
+        start = NStr::StringToInt(fields[1]);
+        from = NStr::StringToInt(fields[6]);
+        to = NStr::StringToInt(fields[7]);
+    }
+    catch (std::exception&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad \"Start/ThickStart/ThickStop\" values." ) );
+        pErr->Throw();
+    }
+    if (start == from  &&  from == to) {
+        return false;
+    }
+    return true;
+}
+
+
+//  ----------------------------------------------------------------------------
+bool
+CBedReader::xContainsCdsFeature(
+    const vector<string>& fields) const
+//  ----------------------------------------------------------------------------
+{
+    if (fields.size() < 8) {
+        return false;
+    }
+
+    int start = -1, from = -1, to = -1;
+    try {
+        start = NStr::StringToInt(fields[1]);
+        from = NStr::StringToInt(fields[6]);
+        to = NStr::StringToInt(fields[7]);
+    }
+    catch (std::exception&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad \"Start/ThickStart/ThickStop\" values." ) );
+        pErr->Throw();
+    }
+    if (start == from  &&  from == to) {
+        return false;
+    }
+    return true;
 }
 
 
@@ -1157,12 +1671,22 @@ CBedReader::xContainsBlockFeature(
 }
 
 
+//  ----------------------------------------------------------------------------
+bool
+CBedReader::xContainsRnaFeature(
+    const vector<string>& fields) const
+//  ----------------------------------------------------------------------------
+{
+    return (fields.size() >= 12);
+}
+
+
 //  ----------------------------------------------------------------------------
 bool
 CBedReader::xReadBedDataRaw(
     ILineReader& lr,
     CRawBedTrack& rawdata,
-    IMessageListener* pMessageListener)
+    ILineErrorListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
     rawdata.Reset();
