@@ -1,4 +1,4 @@
-/*  $Id: gff2_data.cpp 448853 2014-10-09 16:29:29Z ivanov $
+/*  $Id: gff2_data.cpp 511581 2016-08-23 15:29:20Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -63,6 +63,21 @@ BEGIN_NCBI_SCOPE
 
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
+//  ----------------------------------------------------------------------------
+CCdregion::EFrame FramePlusToMinus(
+    CCdregion::EFrame frame)
+//  ----------------------------------------------------------------------------
+{
+    static map<CCdregion::EFrame, CCdregion::EFrame> mFramePlusToMinus;
+    if (mFramePlusToMinus.empty()) {
+        mFramePlusToMinus[CCdregion::eFrame_not_set] = CCdregion::eFrame_not_set;
+        mFramePlusToMinus[CCdregion::eFrame_one] = CCdregion::eFrame_three;
+        mFramePlusToMinus[CCdregion::eFrame_two] = CCdregion::eFrame_two;
+        mFramePlusToMinus[CCdregion::eFrame_three] = CCdregion::eFrame_one;
+    }
+    return mFramePlusToMinus[frame];
+}
+
 //  ----------------------------------------------------------------------------
 CRef<CCode_break> s_StringToCodeBreak(
     const string& str,
@@ -71,7 +86,7 @@ CRef<CCode_break> s_StringToCodeBreak(
 //  ----------------------------------------------------------------------------
 {
     const string cdstr_start = "(pos:";
-    const string cdstr_div = ",aa=";
+    const string cdstr_div = ",aa:";
     const string cdstr_end = ")";
     
     CRef<CCode_break> pCodeBreak;
@@ -180,13 +195,29 @@ CGff2Record::~CGff2Record()
     delete m_pePhase; 
 };
 
-void CGff2Record::TokenizeGFF(vector<CTempString>& columns, const CTempString& in_line)
+
+unsigned int CGff2Record::m_nextId(0);
+//  -----------------------------------------------------------------------------
+unsigned int CGff2Record::NextId()
+//  -----------------------------------------------------------------------------
+{
+    return ++m_nextId;
+}
+
+//  -----------------------------------------------------------------------------
+void CGff2Record::ResetId()
+//  -----------------------------------------------------------------------------
+{
+    m_nextId = 0;
+}
+
+void CGff2Record::TokenizeGFF(vector<CTempStringEx>& columns, const CTempStringEx& in_line)
 {
     columns.clear();
     columns.reserve(9);
     size_t index;
     // first try to split just using tabs
-    NStr::Tokenize(in_line, "\t", columns, NStr::eMergeDelims);
+    NStr::Tokenize(in_line, "\t", columns, NStr::fSplit_MergeDelims);
     if (columns.size() == 9)
         return;
     columns.clear();
@@ -196,17 +227,19 @@ void CGff2Record::TokenizeGFF(vector<CTempString>& columns, const CTempString& i
     const CTempString digits("0123456789");
 
     size_t current = 0;
-    while (current != CTempString::npos && columns.size()<8 && (index = in_line.find_first_of(space_tab_delim, current)) != CTempString::npos)
-    {
-        CTempString next = in_line.substr(current, index-current);
+    while (current != CTempStringEx::npos && columns.size()<8 && 
+            (index = in_line.find_first_of(space_tab_delim, current)) != CTempStringEx::npos) {
+        CTempStringEx next = in_line.substr(current, index-current);
         current = in_line.find_first_not_of(space_tab_delim, index);
-        // looking for the numeric columns at 3 & 4 but not at 5
-        if (columns.size() == 5)
-        {
-            if (next.find_first_of(digits) != CTempString::npos ||
-                columns[4].find_first_not_of(digits) != CTempString::npos ||
-                columns[3].find_first_not_of(digits) != CTempString::npos)
-            {
+        if (columns.size() == 5) {
+            // reminder:
+            // columns [3] and [4] are positions and must always be numeric
+            // column [5] is a score (floating point or "." if absent)
+            bool isNumericCol3 = columns[3].find_first_not_of(digits) == CTempStringEx::npos;
+            bool isNumericCol4 = columns[4].find_first_not_of(digits) == CTempStringEx::npos;
+            bool isNumericNext = next.find_first_not_of(digits) == CTempStringEx::npos;
+
+            if (!isNumericCol3  &&  isNumericCol4  &&  isNumericNext) {
                 // merge col2 with col3 and shift subsequent columns
                 // operations with iterators are legal since all belong to the same memory space
                 size_t sizeof_col1 = (columns[2].begin() + columns[2].size() - columns[1].begin());
@@ -220,7 +253,7 @@ void CGff2Record::TokenizeGFF(vector<CTempString>& columns, const CTempString& i
         }
         columns.push_back(next); 
     }
-    if (current != CTempString::npos)
+    if (current != CTempStringEx::npos)
         columns.push_back(in_line.substr(current));
 }
 //  ----------------------------------------------------------------------------
@@ -228,7 +261,7 @@ bool CGff2Record::AssignFromGff(
     const string& strRawInput )
 //  ----------------------------------------------------------------------------
 {
-    vector< CTempString > columns;
+    vector< CTempStringEx > columns;
     
     TokenizeGFF(columns, strRawInput);
     if ( columns.size() < 9 ) {
@@ -242,11 +275,23 @@ bool CGff2Record::AssignFromGff(
     }
     //  to do: more sanity checks
 
-    m_strId = columns[0];
-    m_strSource = columns[1];
-    m_strType = columns[2];
+    columns[0].Copy(m_strId, 0, CTempString::npos);
+    columns[1].Copy(m_strSource, 0, CTempString::npos);
+    columns[2].Copy(m_strType, 0, CTempString::npos);
+
+    try {
     m_uSeqStart = NStr::StringToUInt( columns[3] ) - 1;
     m_uSeqStop = NStr::StringToUInt( columns[4] ) - 1;
+    }
+    catch (const CException&) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Bad data line: Both \"start\" and \"stop\" must be positive integers.",
+            ILineError::eProblem_FeatureBadStartAndOrStop));
+        pErr->Throw();
+    }
     if (m_uSeqStop < m_uSeqStart) {
         AutoPtr<CObjReaderLineException> pErr(
             CObjReaderLineException::Create(
@@ -262,33 +307,44 @@ bool CGff2Record::AssignFromGff(
         m_pdScore = new double( NStr::StringToDouble( columns[5] ) );
     }
 
-    if ( columns[6] == "+" ) {
-        m_peStrand = new ENa_strand( eNa_strand_plus );
-    }
-    else
-    if ( columns[6] == "-" ) {
-        m_peStrand = new ENa_strand( eNa_strand_minus );
-    }
-    else
-    if ( columns[6] == "?" ) {
-        m_peStrand = new ENa_strand( eNa_strand_unknown );
+    enum ENa_strand strand;
+    switch (columns[6][0]) {
+    default: 
+        strand = objects::eNa_strand_unknown;
+        break;
+    case '+':
+        strand = objects::eNa_strand_plus;
+        break;
+    case '-':
+        strand = objects::eNa_strand_minus;
+        break;
+    case '.':
+        strand = objects::eNa_strand_both;
+        break;
     }
+    m_peStrand = new ENa_strand(strand);
+
+
+    enum CCdregion::EFrame frame = CCdregion::eFrame_not_set;
 
     if ( columns[7] == "0" ) {
-        m_pePhase = new TFrame( CCdregion::eFrame_one );
+        frame = CCdregion::eFrame_one;
     }
     else
     if ( columns[7] == "1" ) {
-        m_pePhase = new TFrame( CCdregion::eFrame_two );
+        frame = CCdregion::eFrame_two;
     }
     else
     if ( columns[7] == "2" ) {
-        m_pePhase = new TFrame( CCdregion::eFrame_three );
+        frame = CCdregion::eFrame_three;
     }
 
-    m_strAttributes = columns[8];
+    if (frame != CCdregion::eFrame_not_set)
+        m_pePhase = new TFrame(frame);
+
+    columns[8].Copy(m_strAttributes, 0, CTempString::npos);
     
-    return x_AssignAttributesFromGff( m_strAttributes );
+    return x_AssignAttributesFromGff(m_strType, columns[8]);
 }
 
 //  ----------------------------------------------------------------------------
@@ -312,11 +368,12 @@ bool CGff2Record::GetAttribute(
     list<string>& values ) const
 //  ----------------------------------------------------------------------------
 {
-    string strValues;
-    if (!GetAttribute(strKey, strValues)) {
+    values.clear();
+    TAttrCit it = m_Attributes.find(strKey);
+    if (it == m_Attributes.end()) {
         return false;
     }
-    NStr::Split(strValues, ",", values);
+    NStr::Split(it->second, ",", values, 0);
     return !values.empty();
 }
 
@@ -337,57 +394,140 @@ CRef<CSeq_loc> CGff2Record::GetSeqLoc(
     pLocation->SetInt().SetId(*GetSeqId(flags));
     pLocation->SetInt().SetFrom(SeqStart());
     pLocation->SetInt().SetTo(SeqStop());
-    if (IsSetStrand()) {
+        if (IsSetStrand()) {
         pLocation->SetInt().SetStrand(Strand());
-    }
+        }
     return pLocation;
+    }
+
+//  ----------------------------------------------------------------------------
+string CGff2Record::xNormalizedAttributeKey(
+    const CTempString& strRawKey )
+//  ----------------------------------------------------------------------------
+{
+    return NStr::TruncateSpaces_Unsafe(strRawKey);
 }
 
 //  ----------------------------------------------------------------------------
-string CGff2Record::x_NormalizedAttributeKey(
-    const string& strRawKey )
+string CGff2Record::xNormalizedAttributeValue(
+    const CTempString& strRawValue )
 //  ----------------------------------------------------------------------------
 {
-    string strKey = NStr::TruncateSpaces( strRawKey );
-    return strKey;
-}
-
-//  ----------------------------------------------------------------------------
-string CGff2Record::x_NormalizedAttributeValue(
-    const string& strRawValue )
-//  ----------------------------------------------------------------------------
-{
-    string strValue = NStr::TruncateSpaces( strRawValue );
+    CTempString strValue = NStr::TruncateSpaces_Unsafe(strRawValue);
     if ( NStr::StartsWith( strValue, "\"" ) ) {
         strValue = strValue.substr( 1, string::npos );
     }
     if ( NStr::EndsWith( strValue, "\"" ) ) {
         strValue = strValue.substr( 0, strValue.length() - 1 );
-    }
-    return NStr::URLDecode(strValue);
+    }   
+    return NStr::URLDecode(strValue, NStr::eUrlDec_Percent);
 }
 
+
 //  ----------------------------------------------------------------------------
+CTempString x_GetNextAttribute(CTempString& input)
+{
+    CTempString result;
+    bool inQuotes = false;
+    size_t i = 0;
+    for (; i < input.length(); i++)
+    {
+        if (inQuotes) {
+            if (input[i] == '\"') {
+                inQuotes = false;
+            }
+        }
+        else { // not in quotes
+            if (input[i] == ';') {
+                result = NStr::TruncateSpaces_Unsafe(input.substr(0, i));
+                if (!result.empty())
+                {
+                    input = input.substr(i+1);
+                    return result;
+                }
+            }
+            else {
+                if (input[i] == '\"') {
+                    inQuotes = true;
+                }
+            }
+        }
+    }
+    result = NStr::TruncateSpaces_Unsafe(input);
+    input.clear();
+    return result;
+}
+
+bool x_GetNextAttribute(CTempString& input, CTempString& key, CTempString& value)
+{
+    size_t semicolon = CTempString::npos;
+    size_t space = CTempString::npos;
+    size_t equal = CTempString::npos;
+
+    key.clear();
+    value.clear();
+
+    bool inQuotes = false;
+    size_t i = 0;
+    for (; i < input.length(); i++)
+    {
+        if (inQuotes) {
+            if (input[i] == '\"') {
+                inQuotes = false;
+            }
+        }
+        else { // not in quotes
+            switch (input[i])
+            {
+            case ';':
+                semicolon = i;
+                break;
+            case ' ':
+                if (space == CTempString::npos && equal == CTempString::npos)
+                    space = i;
+                continue;
+            case '"':
+                inQuotes = true;
+                continue;
+            case '=':
+                if (equal == CTempString::npos)
+                    equal = i;
+                continue;
+            default:
+                continue;
+            }
+            break;
+        }
+    }
+    
+    if (semicolon == CTempString::npos)
+        semicolon = input.length();
+
+    if (equal == CTempString::npos)
+        equal = min(space, semicolon);
+
+    key = NStr::TruncateSpaces_Unsafe(input.substr(0, equal));
+    value = NStr::TruncateSpaces_Unsafe(input.substr(equal + 1, semicolon - equal - 1));
+
+    input = NStr::TruncateSpaces_Unsafe(input.substr(semicolon+1));
+
+    return !key.empty();
+}
+
 bool CGff2Record::x_AssignAttributesFromGff(
+    const string& strType,
     const string& strRawAttributes )
 //  ----------------------------------------------------------------------------
 {
-    vector< string > attributes;
-    x_SplitGffAttributes(strRawAttributes, attributes);
-	for ( size_t u=0; u < attributes.size(); ++u ) {
-        string strKey;
-        string strValue;
-        if ( ! NStr::SplitInTwo( attributes[u], "=", strKey, strValue ) ) {
-            if ( ! NStr::SplitInTwo( attributes[u], " ", strKey, strValue ) ) {
-                return false;
-            }
-        }
-		if ( strKey.empty() && strValue.empty() ) {
-            // Probably due to trailing "; ". Sequence Ontology generates such
-            // things. 
-            continue;
-        }
-        m_Attributes[ strKey ] = strValue;        
+    m_Attributes.clear();
+    CTempString input(strRawAttributes);
+    CTempString next;
+    CTempString strKey;
+    CTempString strValue;
+
+    while (!input.empty() && x_GetNextAttribute(input, strKey, strValue))
+    {
+        m_Attributes[strKey] = strValue;
     }
     return true;
 }
@@ -437,7 +577,7 @@ bool CGff2Record::InitializeFeature(
 {
     return (
         x_InitFeatureLocation(flags, pFeature)  &&
-        x_InitFeatureData(flags, pFeature)  &&
+        xInitFeatureData(flags, pFeature)  &&
         x_MigrateId(pFeature)  &&
         x_MigrateStartStopStrand(pFeature)  &&
         x_MigrateType(pFeature)  &&
@@ -452,6 +592,13 @@ bool CGff2Record::UpdateFeature(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
+    // mss-582:
+    //  if the parent feature is a gene then don't mess with the gene's location
+    //
+    CSeqFeatData::ESubtype subtype = pFeature->GetData().GetSubtype();
+    if (subtype == CSeqFeatData::eSubtype_gene) {
+        return true;
+    }
     const CSeq_loc& target = pFeature->GetLocation();
     CRef<CSeq_loc> pAddLoc = GetSeqLoc(flags);
 
@@ -463,16 +610,75 @@ bool CGff2Record::UpdateFeature(
     }
     else {
         // indicates the feature location is already under construction
-        pFeature->SetLocation(*pFeature->SetLocation().Add(*pAddLoc, CSeq_loc::fSortAndMerge_All, 0));
-    }        
+        pFeature->SetLocation(*pFeature->SetLocation().Add(
+            *pAddLoc, CSeq_loc::fSortAndMerge_All, 0));
+        if (pFeature->GetLocation().IsInt()) {
+            CRef<CSeq_loc> pOld(new CSeq_loc);
+            pOld->Assign(pFeature->GetLocation());
+            pFeature->SetLocation().SetMix().AddSeqLoc(*pOld);
+        }
+    }
+    if (!this->xUpdateFeatureData(flags, pFeature)) {
+        return false;
+    }
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CGff2Record::xUpdateFeatureData(
+    int flags,
+    CRef<CSeq_feat> pFeature) const
+    //  ----------------------------------------------------------------------------
+{
+    const CSeq_loc& target = pFeature->GetLocation();
+    CSeqFeatData::ESubtype subtype = pFeature->GetData().GetSubtype();
+    CRef<CSeq_loc> pAddLoc = GetSeqLoc(flags);
+
+    switch(subtype) {
+        default: {
+            return true;
+        }
+        case CSeqFeatData::eSubtype_cdregion: {
+            // if incoming piece is new feature start then it will provide the
+            //  frame
+            //
+            if (!pAddLoc->GetInt().CanGetStrand()) {
+                return true;
+            }
+            if (pAddLoc->GetInt().GetStrand() == eNa_strand_plus) {
+                size_t curStart = target.GetStart(eExtreme_Positional);
+                size_t newStart = pAddLoc->GetStart(eExtreme_Positional);
+                if (curStart == newStart) {
+                    pFeature->SetData().SetCdregion().SetFrame(Phase());
+                }
+                return true;
+            }
+            if (pAddLoc->GetInt().GetStrand() == eNa_strand_minus) {
+                size_t curStop = target.GetStop(eExtreme_Positional);
+                size_t newStop = pAddLoc->GetStop(eExtreme_Positional);
+                if (curStop == newStop) {
+                    pFeature->SetData().SetCdregion().SetFrame(Phase());
+                }
+                return true;
+            }
+        }
+    }
+    return true;
+}
+    
+
 //  ----------------------------------------------------------------------------
 bool CGff2Record::x_MigrateId(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
+    unsigned int featId = NextId();
+//    if (!GetAttribute("ID", featIdStr)) {
+//        featIdStr = NextId();
+//    }
+    CRef<CFeat_id> pFeatId(new CFeat_id);
+    pFeatId->SetLocal().SetId(featId);
+    pFeature->SetId(*pFeatId);
     return true;
 }
 
@@ -520,17 +726,17 @@ bool CGff2Record::x_MigrateAttributes(
 
     it = attrs_left.find("Note");
     if (it != attrs_left.end()) {
-        pFeature->SetComment(x_NormalizedAttributeValue(it->second));
+        pFeature->SetComment(xNormalizedAttributeValue(it->second));
         attrs_left.erase(it);
     }
 
     it = attrs_left.find("Dbxref");
     if (it != attrs_left.end()) {
         vector<string> dbxrefs;
-        NStr::Tokenize(it->second, ",", dbxrefs, NStr::eMergeDelims);
+        NStr::Split(it->second, ",", dbxrefs, NStr::eMergeDelims);
         for (vector<string>::iterator it1 = dbxrefs.begin(); it1 != dbxrefs.end();
                 ++it1 ) {
-            string dbtag = x_NormalizedAttributeValue(*it1);
+            string dbtag = xNormalizedAttributeValue(*it1);
             pFeature->SetDbxref().push_back(CGff2Reader::x_ParseDbtag(dbtag));
         }
         attrs_left.erase(it);
@@ -546,18 +752,21 @@ bool CGff2Record::x_MigrateAttributes(
         }
     }
 
+    it = attrs_left.find("Parent");
+    if (it != attrs_left.end()) {
+        if (Type() != "CDS") {
+            xMigrateAttributeSingle(
+                attrs_left, "Parent", pFeature, "Parent", flags);
+        }
+        else attrs_left.erase(it);
+    }
+
     it = attrs_left.find("Name");
     if (it != attrs_left.end()) {
         if (0 == NStr::CompareNocase(Type(), "cds")) {
-            CRef<CSeqFeatData> pData(new CSeqFeatData);
-            pData->SetProt().SetName().push_back(it->second);
-            CRef<CSeqFeatXref> pName(new CSeqFeatXref);
-            pName->SetData(*pData);
-            pFeature->SetXref().push_back(pName);
             attrs_left.erase(it);
         }
         if (0 == NStr::CompareNocase(Type(), "mRNA")) {
-            pFeature->SetData().SetRna().SetExt().SetName(it->second);
             attrs_left.erase(it);
         }
     }
@@ -586,7 +795,8 @@ bool CGff2Record::x_MigrateAttributes(
     it = attrs_left.find("description");
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsGene()) {
-            pFeature->SetData().SetGene().SetDesc(it->second);
+            string description = xNormalizedAttributeValue(it->second);
+            pFeature->SetData().SetGene().SetDesc(description);
             attrs_left.erase(it);
         }
     }
@@ -594,7 +804,7 @@ bool CGff2Record::x_MigrateAttributes(
     it = attrs_left.find("exception");
     if (it != attrs_left.end()) {
         pFeature->SetExcept(true);
-        pFeature->SetExcept_text(x_NormalizedAttributeValue(it->second));
+        pFeature->SetExcept_text(xNormalizedAttributeValue(it->second));
         attrs_left.erase(it);
     }
 
@@ -611,7 +821,7 @@ bool CGff2Record::x_MigrateAttributes(
     if (it != attrs_left.end()) {
         const string strExperimentDefault(
             "experimental evidence, no additional details recorded" );
-        string value = x_NormalizedAttributeValue(it->second);
+        string value = xNormalizedAttributeValue(it->second);
         if (value == strExperimentDefault) {
             pFeature->SetExp_ev(CSeq_feat::eExp_ev_experimental);
         }
@@ -632,7 +842,20 @@ bool CGff2Record::x_MigrateAttributes(
     it = attrs_left.find("gene");
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsGene()) {
-            pFeature->SetData().SetGene().SetLocus(it->second);
+            list<string> geneValues;
+            NStr::Split(it->second, ",", geneValues, 0);
+            string value;
+            list<string>::const_iterator cit = geneValues.begin();
+            if (cit != geneValues.end()) {
+                value = xNormalizedAttributeValue(*cit);
+                pFeature->SetData().SetGene().SetLocus(value);
+                cit++;
+                while (cit != geneValues.end()) {
+                    value = xNormalizedAttributeValue(*cit);
+                    pFeature->SetData().SetGene().SetSyn().push_back(value);
+                    cit++;
+                }
+            }
             attrs_left.erase(it);
         }
     }
@@ -649,22 +872,22 @@ bool CGff2Record::x_MigrateAttributes(
     it = attrs_left.find("gene_synonym");
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsGene()) {
-        vector<string> synonyms;
-        NStr::Tokenize(it->second, ",", synonyms, NStr::eMergeDelims);
-        for (vector<string>::iterator it1 = synonyms.begin(); it1 != synonyms.end();
-                ++it1 ) {
-            string synonym = x_NormalizedAttributeValue(*it1);
-            pFeature->SetData().SetGene().SetSyn().push_back(synonym);
-        }
-            attrs_left.erase(it);
+            vector<string> synonyms;
+            NStr::Split(it->second, ",", synonyms, NStr::eMergeDelims);
+            for (vector<string>::iterator it1 = synonyms.begin(); it1 != synonyms.end();
+                    ++it1 ) {
+                string synonym = xNormalizedAttributeValue(*it1);
+                pFeature->SetData().SetGene().SetSyn().push_back(synonym);
+            }
         }
+        attrs_left.erase(it);
     }
 
     it = attrs_left.find("inference");
     if (it != attrs_left.end()) {
         const string strInferenceDefault(
             "non-experimental evidence, no additional details recorded" );
-       string value = x_NormalizedAttributeValue(it->second);
+       string value = xNormalizedAttributeValue(it->second);
         if (value == strInferenceDefault) {
             pFeature->SetExp_ev(CSeq_feat::eExp_ev_not_experimental);
         }
@@ -680,8 +903,8 @@ bool CGff2Record::x_MigrateAttributes(
     it = attrs_left.find("locus_tag");
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsGene()) {
-            pFeature->SetData().SetGene().SetLocus_tag(
-                x_NormalizedAttributeValue(it->second));
+            string tag = xNormalizedAttributeValue(it->second);
+            pFeature->SetData().SetGene().SetLocus_tag(tag);
         }
         attrs_left.erase(it);
     }
@@ -690,7 +913,7 @@ bool CGff2Record::x_MigrateAttributes(
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsGene()) {
             pFeature->SetData().SetGene().SetMaploc(
-                x_NormalizedAttributeValue(it->second));
+                xNormalizedAttributeValue(it->second));
         }
     }
 
@@ -698,7 +921,7 @@ bool CGff2Record::x_MigrateAttributes(
     if (it != attrs_left.end()) {
         if (pFeature->GetData().GetSubtype() == CSeqFeatData::eSubtype_ncRNA) {
             pFeature->SetData().SetRna().SetExt().SetGen().SetClass(
-                x_NormalizedAttributeValue(it->second));
+                xNormalizedAttributeValue(it->second));
         }
         attrs_left.erase(it);
     }
@@ -709,25 +932,32 @@ bool CGff2Record::x_MigrateAttributes(
         attrs_left.erase(it);
     }
 
-    it = attrs_left.find("product");
-    if (it != attrs_left.end()) {
-        if (!pFeature->IsSetProduct()) {
-            CRef<CSeq_id> pId = CReadUtil::AsSeqId(it->second, flags);
-            CRef<CSeq_loc> pLoc( new CSeq_loc(CSeq_loc::e_Whole));
-            pLoc->SetId(*pId);
-            pFeature->SetProduct(*pLoc);
-        }
-        attrs_left.erase(it);
-    }
+//    it = attrs_left.find("product");
+//    if (it != attrs_left.end()) {
+//        if (!pFeature->IsSetProduct()) {
+//            CRef<CSeq_id> pId = CReadUtil::AsSeqId(it->second, flags);
+//            CRef<CSeq_loc> pLoc( new CSeq_loc(CSeq_loc::e_Whole));
+//            pLoc->SetId(*pId);
+//            pFeature->SetProduct(*pLoc);
+//        }
+//        xMigrateAttributeDefault(
+//            attrs_left, "product", pFeature, "product", flags);
+//    }
 
-    it = attrs_left.find("protein_id");
-    if (it != attrs_left.end()) {
-        CRef<CSeq_id> pId = CReadUtil::AsSeqId(it->second, flags);
-        CRef<CSeq_loc> pLoc( new CSeq_loc(CSeq_loc::e_Whole));
-        pLoc->SetId(*pId);
-        pFeature->SetProduct(*pLoc);
-        attrs_left.erase(it);
-    }
+//    xMigrateAttributeDefault(
+//        attrs_left, "Product", pFeature, "product", flags);
+
+//    it = attrs_left.find("protein_id");
+//    if (it != attrs_left.end()) {
+//        string protId = it->second;
+//        CRef<CSeq_id> pId = CReadUtil::AsSeqId(protId, flags);
+//        CRef<CSeq_loc> pLoc( new CSeq_loc(CSeq_loc::e_Whole));
+//        pLoc->SetId(*pId);
+//        pFeature->SetProduct(*pLoc);
+
+//        this->xMigrateAttributeDefault(
+//            attrs_left, "protein_id", pFeature, "protein_id", flags);
+//    }
 
     it = attrs_left.find("pseudo");
     if (it != attrs_left.end()) {
@@ -735,26 +965,31 @@ bool CGff2Record::x_MigrateAttributes(
         attrs_left.erase(it);
     }
 
-    it = attrs_left.find("transcript_id");
-    if (it != attrs_left.end()) {
-        if (!pFeature->IsSetProduct()) {
-            CRef<CSeq_id> pId = CReadUtil::AsSeqId(it->second, flags);
-            CRef<CSeq_loc> pLoc( new CSeq_loc(CSeq_loc::e_Whole));
-            pLoc->SetId(*pId);
-            pFeature->SetProduct(*pLoc);
-        }
-        //do not erase
-    }
+//    it = attrs_left.find("transcript_id");
+//    if (it != attrs_left.end()) {
+//        string transcriptId = it->second;
+//        if (!pFeature->IsSetProduct()) {
+//            CRef<CSeq_id> pId = CReadUtil::AsSeqId(transcriptId, flags);
+//            CRef<CSeq_loc> pLoc( new CSeq_loc(CSeq_loc::e_Whole));
+//            pLoc->SetId(*pId);
+//            pFeature->SetProduct(*pLoc);
+//        }
+        // mss-362: turn transcript_id attributes into transcript_id quailifiers.
+//        xMigrateAttributeSingle(
+//            attrs_left, "transcript_id", pFeature, "transcript_id", flags);
+//    }
 
     it = attrs_left.find("transl_except");
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsCdregion()) {
             vector<string> codebreaks;
-            NStr::Tokenize(it->second, ",", codebreaks, NStr::eMergeDelims);
+            NStr::Split(it->second, ",", codebreaks, NStr::eMergeDelims);
             for (vector<string>::iterator it1 = codebreaks.begin(); 
                     it1 != codebreaks.end(); ++it1 ) {
+                string breakData = xNormalizedAttributeValue(*it1);
+                CRef<CSeq_id> pBreakId = GetSeqId(flags);
                 CRef<CCode_break> pCodeBreak = s_StringToCodeBreak(
-                    x_NormalizedAttributeValue(*it1), *GetSeqId(flags), flags);
+                    breakData, *pBreakId, flags);
                 if (pCodeBreak) {
                     pFeature->SetData().SetCdregion().SetCode_break().push_back(
                         pCodeBreak);
@@ -786,19 +1021,64 @@ bool CGff2Record::x_MigrateAttributes(
     //
     //  Turn whatever is left into a gbqual:
     //
-    CRef<CGb_qual> pQual;
     while (!attrs_left.empty()) {
-        it = attrs_left.begin();
-        string qual = it->first;
-        pQual.Reset(new CGb_qual);
-        pQual->SetQual(qual);
-        pQual->SetVal(it->second);
-        pFeature->SetQual().push_back(pQual);
-        attrs_left.erase(it);
+        const string& key = attrs_left.begin()->first;
+        if (!xMigrateAttributeDefault(attrs_left, key, pFeature, key, flags)) {
+            return false;
+        }
     }
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CGff2Record::xMigrateAttributeSingle(
+    TAttributes& attributes,
+    const string& attrKey,
+    CRef<CSeq_feat> pFeature,
+    const string& qualKey,
+    int flags)
+    //  -----------------------------------------------------------------------------
+{
+    //retrieve GFF3 attribute as a single value,
+    //  turn unescaped value into gbqual of the same key.
+
+    TAttributes::iterator it = attributes.find(attrKey);
+    if (it == attributes.end()) {
+        return true;
+    }
+    string value = xNormalizedAttributeValue(it->second);
+    pFeature->AddQualifier(qualKey, value);
+    attributes.erase(it);
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+bool CGff2Record::xMigrateAttributeDefault(
+    TAttributes& attributes,
+    const string& attrKey,
+    CRef<CSeq_feat> pFeature,
+    const string& qualKey,
+    int flags)
+    //  -----------------------------------------------------------------------------
+{
+    //split GFF3 multi-value into individual values, create a gbqual of the
+    // same key for each of the individual values.
+    //
+    TAttributes::iterator it = attributes.find(attrKey);
+    if (it == attributes.end()) {
+        return true;
+    }
+    list<CTempStringEx> values;
+    NStr::Split(it->second, ",", values, 0);
+    for (list<CTempStringEx>::const_iterator cit = values.begin(); cit != values.end();
+            cit++) {
+        string value = xNormalizedAttributeValue(*cit);
+        pFeature->AddQualifier(qualKey, value);
+    }
+    attributes.erase(it);
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 bool CGff2Record::x_MigrateAttributesOrgName(
     int flags,
@@ -933,7 +1213,7 @@ bool CGff2Record::x_MigrateAttributesSubSource(
         }
         CRef<CSubSource> pSubSource(new CSubSource);
         pSubSource->SetSubtype(sit->second);
-        pSubSource->SetName(ait->second);
+        pSubSource->SetName(xNormalizedAttributeValue(ait->second));
         subType.push_back(pSubSource);
         attrs_left.erase(ait);
     }
@@ -951,17 +1231,11 @@ bool CGff2Record::x_InitFeatureLocation(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_InitFeatureData(
+bool CGff2Record::xInitFeatureData(
     int flags,
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
-    map<CCdregion::EFrame, CCdregion::EFrame> FramePlusToMinus;
-    FramePlusToMinus[CCdregion::eFrame_not_set] = CCdregion::eFrame_not_set;
-    FramePlusToMinus[CCdregion::eFrame_one] = CCdregion::eFrame_three;
-    FramePlusToMinus[CCdregion::eFrame_two] = CCdregion::eFrame_two;
-    FramePlusToMinus[CCdregion::eFrame_three] = CCdregion::eFrame_one;
-
     string gbkey;
     if (GetAttribute("gbkey", gbkey)) {
         if (gbkey == "Src") {
@@ -970,14 +1244,28 @@ bool CGff2Record::x_InitFeatureData(
         }
     }
 
-    CFeatListItem itemtype = SofaTypes().MapSofaTermToFeatListItem( Type());
-    switch( itemtype.GetType() ) {
-        default:
-            break;
+    //special cases:
+    if (xInitFeatureDataBond(flags, pFeature)) {
+        return true;
+    }
+    if (xInitFeatureDataNcrna(flags, pFeature)) {
+        return true;
+    }
+    if (xInitFeatureDataSpecialImp(flags, pFeature)) {
+        return true;
+    }
 
-        case CSeqFeatData::e_Gene:
+    //regular case:
+    // use 1<->1 SOFA map.
+    CFeatListItem itemtype = SofaTypes().MapSofaTermToFeatListItem(Type());
+    switch( itemtype.GetType() ) {
+        default: {
+            return true;
+        }
+        case CSeqFeatData::e_Gene: {
             pFeature->SetData().SetGene();
             return true;
+        }
 
         case CSeqFeatData::e_Cdregion: {
             //oh my --- phases again ---
@@ -985,9 +1273,6 @@ bool CGff2Record::x_InitFeatureData(
             if (frame == CCdregion::eFrame_not_set) {
                 frame = CCdregion::eFrame_one;
             }
-            else if (Strand() == eNa_strand_minus) {
-                frame = FramePlusToMinus[frame];
-            } 
             pFeature->SetData().SetCdregion();
             pFeature->SetData().SetCdregion().SetFrame(frame);
             return true;
@@ -999,6 +1284,9 @@ bool CGff2Record::x_InitFeatureData(
                 default:
                     rnaref.SetType(CRNA_ref::eType_unknown);
                     return true;
+               case CSeqFeatData::eSubtype_tmRNA:
+                    rnaref.SetType(CRNA_ref::eType_tmRNA);
+                    return true;
                case CSeqFeatData::eSubtype_mRNA:
                     rnaref.SetType(CRNA_ref::eType_mRNA);
                     return true;
@@ -1020,15 +1308,134 @@ bool CGff2Record::x_InitFeatureData(
             }
             return true;
         }
+        case CSeqFeatData::e_Imp: {
+            CSeqFeatData::TImp& imp = pFeature->SetData().SetImp();
+            CSeqFeatData::ESubtype subType = 
+                static_cast<CSeqFeatData::ESubtype>(itemtype.GetSubtype());
+            if (subType == CSeqFeatData::eSubtype_bad) {
+                if (Type() == ".") {
+                    imp.SetKey("misc_feature");
+                    return true;
+                }
+                imp.SetKey(Type());
+                return true;
+            }
+            const string& key = CSeqFeatData::SubtypeValueToName(
+                static_cast<CSeqFeatData::ESubtype>(itemtype.GetSubtype()));
+            imp.SetKey(key);
+            return true;
+        }
     }
-    pFeature->SetData().SetImp();  
-    if (Type() == ".") {
-        pFeature->SetData().SetImp().SetKey("misc_feature");
+}
+
+//  ----------------------------------------------------------------------------
+bool CGff2Record::xInitFeatureDataBond(
+    int flags,
+    CRef<CSeq_feat> pFeature) const
+//  ----------------------------------------------------------------------------
+{
+    string ftype = Type();
+    if (ftype == "cross_link") {
+        pFeature->SetData().SetBond(CSeqFeatData::eBond_xlink);
+        return true;
     }
-    else {
-        pFeature->SetData().SetImp().SetKey(Type());
+    if (ftype == "disulfide_bond") {
+        pFeature->SetData().SetBond(CSeqFeatData::eBond_disulfide);
+        return true;
     }
-    return true;
+    return false;
+}
+
+//  ----------------------------------------------------------------------------
+bool CGff2Record::xInitFeatureDataNcrna(
+    int flags,
+    CRef<CSeq_feat> pFeature) const
+//  ----------------------------------------------------------------------------
+{
+    typedef SStaticPair<const char*, const char*>  NCRNA_ENTRY;
+    static const NCRNA_ENTRY ncRnaMap_[] = {
+        { "antisense_RNA", "antisense_RNA" },
+        { "autocatalytically_spliced_intron", "autocatalytically_spliced_intron" },
+        { "guide_RNA", "guide_RNA" },
+        { "hammerhead_ribozyme", "hammerhead_ribozyme" },
+        { "lnc_RNA", "lncRNA" },
+        { "miRNA", "miRNA" },
+        { "ncRNA", "other" },
+        { "piRNA", "piRNA" },
+        { "rasiRNA", "rasiRNA" },
+        { "ribozyme", "ribozyme" },
+        { "RNase_MRP_RNA", "RNase_MRP_RNA" },
+        { "RNase_P_RNA", "RNase_P_RNA" },
+        { "scRNA", "scRNA" },
+        { "siRNA", "siRNA" },
+        { "snoRNA", "snoRNA" },
+        { "snRNA", "snRNA" },
+        { "SRP_RNA", "SRP_RNA" },
+        { "telomerase_RNA", "telomerase_RNA" },
+        { "vault_RNA", "vault_RNA" },
+        { "Y_RNA", "Y_RNA" }
+    };
+    typedef CStaticArrayMap<string, string, PNocase> NCRNA_MAP;
+    DEFINE_STATIC_ARRAY_MAP_WITH_COPY(NCRNA_MAP, ncRnaMap, ncRnaMap_);
+
+    string ftype = Type();
+    if (ftype == "ncRNA") {
+        pFeature->SetData().SetRna().SetType(CRNA_ref::eType_ncRNA);
+        string ncrna_class;
+        if (GetAttribute("ncrna_class", ncrna_class)) {
+            pFeature->SetData().SetRna().SetExt().SetGen().SetClass(ncrna_class);
+        }
+        else {
+            pFeature->SetData().SetRna().SetExt().SetGen().SetClass("other");
+        }
+        return true;
+    }
+    NCRNA_MAP::const_iterator cit = ncRnaMap.find(ftype);
+    if (cit != ncRnaMap.end()) {
+        pFeature->SetData().SetRna().SetType(CRNA_ref::eType_ncRNA);
+        pFeature->SetData().SetRna().SetExt().SetGen().SetClass(cit->second);
+        return true;
+    }
+    return false;
+}
+
+//  ----------------------------------------------------------------------------
+bool CGff2Record::xInitFeatureDataSpecialImp(
+    int flags,
+    CRef<CSeq_feat> pFeature) const
+//  ----------------------------------------------------------------------------
+{
+    typedef SStaticPair<const char*, const char*>  REGULATORY_ENTRY;
+    static const REGULATORY_ENTRY regulatoryMap_[] = {
+        { "attenuator", "attenuator" },
+        { "boundary_element", "insulator" },
+        { "CAAT_signal", "CAAT_signal" },
+        { "enhancer", "enhancer" },
+        { "GC_rich_promoter_region", "GC_signal" },
+        { "insulator", "enhancer_blocking_element" },
+        { "locus_control_region", "locus_control_region" },
+        { "minus_10_signal", "minus_10_signal" },
+        { "minus_35_signal", "minus_35_signal" },
+        { "polyA_signal_sequence", "polyA_signal_sequence" },
+        { "promoter", "promoter" },
+        { "regulatory_region", "other" },
+        { "riboswitch", "riboswitch" },
+        { "Shine_Dalgarno_sequence", "ribosome_binding_site" },
+        { "silencer", "silencer" },
+        { "TATA_box", "TATA_box" },
+        { "terminator", "terminator" },
+    };
+    typedef CStaticArrayMap<string, string, PNocase> REGULATORY_MAP;
+    DEFINE_STATIC_ARRAY_MAP_WITH_COPY(REGULATORY_MAP, regulatoryMap, regulatoryMap_);
+
+    string ftype = Type();
+    REGULATORY_MAP::const_iterator cit = regulatoryMap.find(ftype);
+    if (cit != regulatoryMap.end()) {
+        pFeature->SetData().SetImp().SetKey("regulatory");
+        pFeature->AddQualifier("regulatory_class", cit->second);
+        return true;
+    }
+    return false;
 }
 
 END_objects_SCOPE
