@@ -1,4 +1,4 @@
-/*  $Id: gff2_reader.cpp 448876 2014-10-09 16:36:40Z ivanov $
+/*  $Id: gff2_reader.cpp 512399 2016-08-30 17:45:18Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -85,6 +85,8 @@
 #include <objects/seqalign/Dense_seg.hpp>
 #include <objects/seqalign/Score.hpp>
 
+#include <objmgr/feat_ci.hpp>
+
 #include <objtools/readers/read_util.hpp>
 #include <objtools/readers/reader_exception.hpp>
 #include <objtools/readers/line_error.hpp>
@@ -105,23 +107,21 @@ BEGIN_NCBI_SCOPE
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
 //  ----------------------------------------------------------------------------
-bool CGff2Reader::s_GetAnnotId(
-    const CSeq_annot& annot,
-    string& strId )
+const string* CGff2Reader::s_GetAnnotId(
+    const CSeq_annot& annot)
 //  ----------------------------------------------------------------------------
 {
     if ( ! annot.CanGetId() || annot.GetId().size() != 1 ) {
         // internal error
-        return false;
+        return 0;
     }
     
     CRef< CAnnot_id > pId = *( annot.GetId().begin() );
     if ( ! pId->IsLocal() ) {
         // internal error
-        return false;
+        return 0;
     }
-    strId = pId->GetLocal().GetStr();
-    return true;
+    return &pId->GetLocal().GetStr();
 }
 
 //  ----------------------------------------------------------------------------
@@ -130,10 +130,8 @@ CGff2Reader::CGff2Reader(
     const string& name,
     const string& title):
 //  ----------------------------------------------------------------------------
-    CReaderBase(iFlags),
-    m_pErrors(0),
-    m_AnnotName(name),
-    m_AnnotTitle(title)
+    CReaderBase(iFlags, name, title),
+    m_pErrors(0)
 {
 }
 
@@ -143,17 +141,15 @@ CGff2Reader::~CGff2Reader()
 {
 }
 
-
 //  --------------------------------------------------------------------------- 
 void
 CGff2Reader::ReadSeqAnnots(
-    vector< CRef<CSeq_annot> >& annots,
+    TAnnotList& annots,
     CNcbiIstream& istr,
-    IMessageListener* pMessageListener )
+    ILineErrorListener* pMessageListener )
 //  ---------------------------------------------------------------------------
 {
     xReadInit();
-    xProgressInit(istr);
     CStreamLineReader lr( istr );
     ReadSeqAnnots( annots, lr, pMessageListener );
 }
@@ -161,31 +157,34 @@ CGff2Reader::ReadSeqAnnots(
 //  ---------------------------------------------------------------------------                       
 void
 CGff2Reader::ReadSeqAnnots(
-    vector< CRef<CSeq_annot> >& annots,
+    TAnnotList& annots,
     ILineReader& lr,
-    IMessageListener* pMessageListener )
+    ILineErrorListener* pMessageListener )
 //  ----------------------------------------------------------------------------
 {
+    xProgressInit(lr);
+
     if ( m_iFlags & fNewCode ) {
-        return ReadSeqAnnotsNew( annots, lr, pMessageListener );
+        ReadSeqAnnotsNew(annots, lr, pMessageListener);
     }
-    CRef< CSeq_entry > entry = ReadSeqEntry( lr, pMessageListener );
-    CTypeIterator<CSeq_annot> annot_iter( *entry );
-    for ( ;  annot_iter;  ++annot_iter) {
-        annots.push_back( CRef<CSeq_annot>( annot_iter.operator->() ) );
+    else {
+        CRef< CSeq_entry > entry = ReadSeqEntry(lr, pMessageListener);
+        CTypeIterator<CSeq_annot> annot_iter( *entry );
+        for (; annot_iter; ++annot_iter) {
+            annots.push_back(CRef<CSeq_annot>(annot_iter.operator->()));
+        }
     }
 }
 
 //  ---------------------------------------------------------------------------                       
 void
 CGff2Reader::ReadSeqAnnotsNew(
-    vector< CRef<CSeq_annot> >& annots,
+    TAnnots& annots,
     ILineReader& lr,
-    IMessageListener* pEC )
+    ILineErrorListener* pEC )
 //  ----------------------------------------------------------------------------
 {
     string line;
-    //int linecount = 0;
     while (xGetLine(lr, line)) {
         if (IsCanceled()) {
             AutoPtr<CObjReaderLineException> pErr(
@@ -219,8 +218,7 @@ CGff2Reader::ReadSeqAnnotsNew(
             ProcessError(err, pEC);
         }
     }
-    typedef vector< CRef<CSeq_annot> >::iterator ANNOTIT;
-    for (ANNOTIT it = annots.begin(); it != annots.end(); ++it) {
+    for (TAnnots::iterator it = annots.begin(); it != annots.end(); ++it) {
         try {
             xAnnotPostProcess(*it);
         }
@@ -235,16 +233,18 @@ CGff2Reader::ReadSeqAnnotsNew(
 CRef< CSeq_entry >
 CGff2Reader::ReadSeqEntry(
     ILineReader& lr,
-    IMessageListener* pMessageListener ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------                
 { 
-    vector<CRef<CSeq_annot> > annots;
+    xProgressInit(lr);
+
+    TAnnots annots;
     ReadSeqAnnotsNew( annots, lr, pMessageListener );
     
     CRef<CSeq_entry> pSeqEntry(new CSeq_entry());
     pSeqEntry->SetSet();
 
-    for (vector<CRef<CSeq_annot> >::iterator it = annots.begin(); 
+    for (TAnnots::iterator it = annots.begin();
             it != annots.end(); ++it) {
         CRef<CBioseq> pSeq( new CBioseq() );
         pSeq->SetAnnot().push_back(*it);
@@ -264,7 +264,7 @@ CGff2Reader::ReadSeqEntry(
 CRef< CSerialObject >
 CGff2Reader::ReadObject(
     ILineReader& lr,
-    IMessageListener* pMessageListener ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------                
 { 
     CRef<CSerialObject> object( 
@@ -313,10 +313,13 @@ bool CGff2Reader::x_ParseStructuredCommentGff(
 bool CGff2Reader::x_ParseDataGff(
     const string& strLine,
     TAnnots& annots,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-    if ( CGff2Reader::IsAlignmentData(strLine) ) {
+    if (CGff2Reader::IsAlignmentData(strLine)) {
+        if (m_iFlags&fGenbankMode) {
+            return true;
+        }
         return x_ParseAlignmentGff(strLine, annots);
     }
     return x_ParseFeatureGff(strLine, annots, pEC);
@@ -326,7 +329,7 @@ bool CGff2Reader::x_ParseDataGff(
 bool CGff2Reader::x_ParseFeatureGff(
     const string& strLine,
     TAnnots& annots,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     //
@@ -343,6 +346,19 @@ bool CGff2Reader::x_ParseFeatureGff(
         ProcessError(err, pEC);
         return false;
     }
+    string ftype = pRecord->Type();
+    if (xIsIgnoredFeatureType(ftype)) {
+        string message = string("GFF3 feature type \"") + ftype + 
+            string("\" not supported- ignored.");
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Warning,
+            0,
+            message,
+            ILineError::eProblem_FeatureNameNotAllowed));
+        ProcessError(*pErr, pEC);
+        return true;
+    }
 
     //
     //  Search annots for a pre-existing annot pertaining to the same ID:
@@ -350,11 +366,11 @@ bool CGff2Reader::x_ParseFeatureGff(
     TAnnotIt it = annots.begin();
     for ( /*NOOP*/; it != annots.end(); ++it ) {
         if (!(**it).IsFtable()) continue;
-        string strAnnotId;
-        if ( ! s_GetAnnotId( **it, strAnnotId ) ) {
+        const string* strAnnotId = s_GetAnnotId(**it);
+        if (strAnnotId == 0) {
             return false;
         }
-        if ( pRecord->Id() == strAnnotId ) {
+        if ( pRecord->Id() == *strAnnotId ) {
             break;
         }
     }
@@ -364,7 +380,7 @@ bool CGff2Reader::x_ParseFeatureGff(
     //  information:
     //
     if ( it != annots.end() ) {
-        if ( ! x_UpdateAnnotFeature( *pRecord, *it ) ) {
+        if ( ! x_UpdateAnnotFeature( *pRecord, *it, pEC ) ) {
             return false;
         }
     }
@@ -375,13 +391,11 @@ bool CGff2Reader::x_ParseFeatureGff(
     //
     else {
         CRef< CSeq_annot > pAnnot( new CSeq_annot );
-        if ( ! x_InitAnnot( *pRecord, pAnnot ) ) {
+        if ( ! x_InitAnnot( *pRecord, pAnnot, pEC ) ) {
             return false;
         }
-        CSeq_annot::TData::TFtable& table = pAnnot->SetData().SetFtable();
-        annots.push_back( pAnnot );      
+        annots.insert(annots.begin(), pAnnot );      
     }
- 
     return true; 
 };
 
@@ -406,11 +420,11 @@ bool CGff2Reader::x_ParseAlignmentGff(
     TAnnotIt it = annots.begin();
     for ( /*NOOP*/; it != annots.end(); ++it ) {
         if (!(**it).IsAlign()) continue;
-        string strAnnotId;
-        if ( ! s_GetAnnotId( **it, strAnnotId ) ) {
+        const string* strAnnotId = s_GetAnnotId(**it);
+        if (!strAnnotId) {
             return false;
         }
-        if ( pRecord->Id() == strAnnotId ) {
+        if ( pRecord->Id() == *strAnnotId ) {
             break;
         }
     }
@@ -434,9 +448,9 @@ bool CGff2Reader::x_ParseAlignmentGff(
         if ( ! x_InitAnnot( *pRecord, pAnnot ) ) {
             return false;
         }
-        annots.push_back( pAnnot );      
+        annots.insert(annots.begin(), pAnnot );      
     }
- 
+
     return true; 
 };
 
@@ -450,7 +464,7 @@ bool CGff2Reader::x_ParseBrowserLineGff(
         return false;
     }
     vector< string > columns;
-    NStr::Tokenize( strRawInput, " \t", columns, NStr::eMergeDelims );
+    NStr::Split( strRawInput, " \t", columns, NStr::eMergeDelims );
 
     if ( columns.size() <= 1 || 1 != ( columns.size() % 2 ) ) {
         // don't know how to unwrap this
@@ -490,7 +504,7 @@ bool CGff2Reader::x_ParseTrackLineGff(
         }
     }
     vector< string > columns;
-    NStr::Tokenize( strCookedInput, " \t", columns, NStr::eMergeDelims );
+    NStr::Split( strCookedInput, " \t", columns, NStr::eMergeDelims );
 
     if ( columns.size() <= 1 ) {
         pAnnotDesc.Reset();
@@ -523,7 +537,8 @@ bool CGff2Reader::x_ParseTrackLineGff(
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::x_InitAnnot(
     const CGff2Record& gff,
-    CRef< CSeq_annot > pAnnot )
+    CRef< CSeq_annot > pAnnot,
+    ILineErrorListener* pEC )
 //  ----------------------------------------------------------------------------
 {
     CRef< CAnnot_id > pAnnotId( new CAnnot_id );
@@ -554,7 +569,7 @@ bool CGff2Reader::x_InitAnnot(
     }
     else {
         pAnnot->SetData().SetFtable();
-        return x_UpdateAnnotFeature( gff, pAnnot );
+        return x_UpdateAnnotFeature( gff, pAnnot, pEC );
     }
 }
 
@@ -562,7 +577,7 @@ bool CGff2Reader::x_InitAnnot(
 bool CGff2Reader::x_UpdateAnnotFeature(
     const CGff2Record& gff,
     CRef< CSeq_annot > pAnnot,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     CRef< CSeq_feat > pFeature( new CSeq_feat );
@@ -582,10 +597,7 @@ bool CGff2Reader::x_UpdateAnnotFeature(
     if ( ! x_FeatureSetQualifiers( gff, pFeature ) ) {
         return false;
     }
-    if (!x_FeatureSetXref(gff, pFeature)) {
-        return false;
-    }
-    if (!x_AddFeatureToAnnot( pFeature, pAnnot )) {
+    if (!xAddFeatureToAnnot( pFeature, pAnnot )) {
         return false;
     }
     string strId;
@@ -629,20 +641,18 @@ bool CGff2Reader::xAlignmentSetSegment(
     if (!gff.GetAttribute("Target", targetInfo)) {
         return false;
     }
-    NStr::Tokenize(targetInfo, " ", targetParts);
+    NStr::Split(targetInfo, " ", targetParts);
     if (targetParts.size() != 4) {
         return false;
     }
     
     string gapInfo;
     vector<string> gapParts;
-    if (gff.GetAttribute("Gap", gapInfo)) 
-    {
-        NStr::Tokenize(gapInfo, " ", gapParts);
+    if (gff.GetAttribute("Gap", gapInfo)) {
+        NStr::Split(gapInfo, " ", gapParts);
     }
-    else
-    {
-        gapParts.push_back(string("M") + NStr::IntToString(gff.SeqStop()-gff.SeqStart()+1));
+    else {
+        gapParts.push_back(string("M") + NStr::NumericToString(gff.SeqStop()-gff.SeqStart()+1));
     }
     int gapCount = gapParts.size();
 
@@ -654,9 +664,9 @@ bool CGff2Reader::xAlignmentSetSegment(
 
     //ids
     denseg.SetIds().push_back(
-        CRef<CSeq_id>(new CSeq_id(targetParts[0])));
+        CReadUtil::AsSeqId(targetParts[0]));
     denseg.SetIds().push_back(
-        CRef<CSeq_id>(new CSeq_id(gff.Id())));
+        CReadUtil::AsSeqId(gff.Id()));
 
     //starts
     size_t targetOffset = 0;
@@ -784,35 +794,6 @@ bool CGff2Reader::x_FeatureSetId(
     return true;
 }
 
-//  ----------------------------------------------------------------------------
-bool CGff2Reader::x_FeatureSetXref(
-    const CGff2Record& record,
-    CRef< CSeq_feat > pFeature )
-//  ----------------------------------------------------------------------------
-{
-    string strParent;
-    list<string> parents;
-    if (!record.GetAttribute("Parent", parents)) {
-        return true;
-    }
-    for (list<string>::const_iterator cit = parents.begin(); cit != parents.end();
-            ++cit) {
-        CRef<CFeat_id> pFeatId(new CFeat_id);
-        pFeatId->SetLocal().SetStr(*cit);
-        IdToFeatureMap::iterator it = m_MapIdToFeature.find(*cit);
-        if (it == m_MapIdToFeature.end()) {
-            //return false;
-			continue;
-        }
-        CRef<CSeq_feat> pParent = it->second;
-        pParent->SetId(*pFeatId);
-        CRef< CSeqFeatXref > pXref(new CSeqFeatXref);
-        pXref->SetId(*pFeatId);  
-        pFeature->SetXref().push_back(pXref);
-    }
-    return true;
-}
-
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::x_FeatureSetLocation(
     const CGff2Record& record,
@@ -865,6 +846,18 @@ bool CGff2Reader::x_FeatureTrimQualifiers(
             it++;
             continue;
         }
+        if (qualKey == "old_locus_tag") {
+            it++;
+            continue;
+        }
+        if (qualKey == "product") {
+            it++;
+            continue;
+        }
+        if (qualKey == "protein_id") {
+            it++;
+            continue;
+        }
         const string& qualVal = (*it)->GetVal();
         string attrVal;
         if (!record.GetAttribute(qualKey, attrVal)) {
@@ -909,6 +902,20 @@ bool CGff2Reader::x_FeatureSetQualifiers(
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CGff2Reader::xFeatureSetQualifier(
+    const string& key,
+    const string& value,
+    CRef<CSeq_feat> pTargetFeature)
+//  ----------------------------------------------------------------------------
+{
+    if (!pTargetFeature) {
+        return false;
+    }
+    pTargetFeature->AddOrReplaceQualifier(key, value);
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::x_FeatureSetGffInfo(
     const CGff2Record& record,
@@ -929,7 +936,7 @@ bool CGff2Reader::x_FeatureSetGffInfo(
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::x_FeatureSetData(
     const CGff2Record& record,
-    CRef< CSeq_feat > pFeature )
+    CRef< CSeq_feat > pFeature)
 //  ----------------------------------------------------------------------------
 {
     //
@@ -937,20 +944,28 @@ bool CGff2Reader::x_FeatureSetData(
     //
 
     CSeqFeatData::ESubtype iGenbankType = SofaTypes().MapSofaTermToGenbankType(
-        record.Type() );
+        record.Type());
 
-    switch( iGenbankType ) {
+    switch(iGenbankType) {
     default:
-        return x_FeatureSetDataMiscFeature( record, pFeature );
+        return x_FeatureSetDataMiscFeature(record, pFeature);
 
     case CSeqFeatData::eSubtype_cdregion:
-        return x_FeatureSetDataCDS( record, pFeature );
+        return x_FeatureSetDataCDS(record, pFeature);
     case CSeqFeatData::eSubtype_exon:
-        return x_FeatureSetDataExon( record, pFeature );
+        return x_FeatureSetDataExon(record, pFeature);
     case CSeqFeatData::eSubtype_gene:
-        return x_FeatureSetDataGene( record, pFeature );
+        return x_FeatureSetDataGene(record, pFeature);
     case CSeqFeatData::eSubtype_mRNA:
-        return x_FeatureSetDataMRNA( record, pFeature );
+    case CSeqFeatData::eSubtype_rRNA:
+    case CSeqFeatData::eSubtype_ncRNA:
+    case CSeqFeatData::eSubtype_preRNA:
+    case CSeqFeatData::eSubtype_scRNA:
+    case CSeqFeatData::eSubtype_snRNA:
+    case CSeqFeatData::eSubtype_snoRNA:
+    case CSeqFeatData::eSubtype_tRNA:
+    case CSeqFeatData::eSubtype_tmRNA:
+        return x_FeatureSetDataRna(record, pFeature, iGenbankType);
     }    
     return true;
 }
@@ -966,14 +981,24 @@ bool CGff2Reader::x_FeatureSetDataGene(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Reader::x_FeatureSetDataMRNA(
+bool CGff2Reader::x_FeatureSetDataRna(
     const CGff2Record& record,
-    CRef< CSeq_feat > pFeature )
+    CRef< CSeq_feat > pFeature,
+    CSeqFeatData::ESubtype subType)
 //  ----------------------------------------------------------------------------
 {
     CRNA_ref& rnaRef = pFeature->SetData().SetRna();
-    rnaRef.SetType( CRNA_ref::eType_mRNA );
-
+    switch (subType){
+        default:
+            rnaRef.SetType(CRNA_ref::eType_miscRNA);
+            break;
+        case CSeqFeatData::eSubtype_mRNA:
+            rnaRef.SetType(CRNA_ref::eType_mRNA);
+            break;
+        case CSeqFeatData::eSubtype_rRNA:
+            rnaRef.SetType(CRNA_ref::eType_rRNA);
+            break;
+    }
     return true;
 }
 
@@ -1080,14 +1105,14 @@ bool CGff2Reader::IsCds(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Reader::x_AddFeatureToAnnot(
+bool CGff2Reader::xAddFeatureToAnnot(
     CRef< CSeq_feat > pFeature,
     CRef< CSeq_annot > pAnnot )
 //  ----------------------------------------------------------------------------
 {
     if (IsExon(pFeature)) {
         CRef< CSeq_feat > pParent;    
-        if (!x_GetParentFeature(*pFeature, pParent) ) {
+        if (!xGetParentFeature(*pFeature, pParent) ) {
             pAnnot->SetData().SetFtable().push_back(pFeature) ;
             return true;
         }
@@ -1127,7 +1152,7 @@ bool CGff2Reader::xGetExistingFeature(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Reader::x_GetParentFeature(
+bool CGff2Reader::xGetParentFeature(
     const CSeq_feat& feature,
     CRef< CSeq_feat >& pParent )
 //  ----------------------------------------------------------------------------
@@ -1224,9 +1249,110 @@ bool CGff2Reader::xAnnotPostProcess(
     CRef<CSeq_annot> pAnnot)
 //  ============================================================================
 {
+    if (!xGenerateParentChildXrefs(pAnnot)) {
+        return false;
+    }
     return true;
 }
 
+//  ============================================================================
+bool CGff2Reader::xGenerateParentChildXrefs(
+    CRef<CSeq_annot> pAnnot)
+//  ============================================================================
+{
+    typedef list<CRef<CSeq_feat> > FTABLE;
+    typedef list<string> PARENTS;
+
+    if (!pAnnot->IsFtable()) {
+        return true;
+    }
+    FTABLE& ftable = pAnnot->SetData().SetFtable();
+    for (FTABLE::iterator featIt = ftable.begin(); featIt != ftable.end(); ++featIt) {
+        CSeq_feat& feat = **featIt;
+        const string& parentStr = feat.GetNamedQual("Parent");
+        PARENTS parents;
+        NStr::Split(parentStr, ",", parents, 0);
+        for (PARENTS::iterator parentIt = parents.begin(); parentIt != parents.end(); ++parentIt) {
+            const string& parent = *parentIt; 
+            xSetAncestryLine(feat, parent);
+        }
+    }
+    return true;
+}
+
+//  ============================================================================
+void CGff2Reader::xSetAncestryLine(
+    CSeq_feat& feat,
+    const string& directParentStr)
+//  ============================================================================
+{
+    typedef list<string> PARENTS;
+
+    string ancestorStr(directParentStr);
+    CRef<CSeq_feat> pAncestor;
+    while (!ancestorStr.empty()) {
+        if (!x_GetFeatureById(ancestorStr, pAncestor)) {
+            return;
+        }
+        xSetAncestorXrefs(feat, *pAncestor);
+        ancestorStr = pAncestor->GetNamedQual("Parent");
+        PARENTS ancestors;
+        NStr::Split(ancestorStr, ",", ancestors, 0);
+        for (PARENTS::iterator it = ancestors.begin(); it != ancestors.end(); ++it) {
+            const string& ancestorStr = *it;
+            xSetAncestryLine(feat, ancestorStr);
+        }
+    }
+}
+
+//  ============================================================================
+bool sFeatureHasXref(
+    const CSeq_feat& feat,
+    const CFeat_id& featId)
+//  ============================================================================
+{
+    typedef vector<CRef<CSeqFeatXref> > XREFS;
+    if (!feat.IsSetXref()) {
+        return false;
+    }
+    int xrefId = featId.GetLocal().GetId();
+    const XREFS& xrefs = feat.GetXref();
+    for (XREFS::const_iterator cit = xrefs.begin(); cit != xrefs.end(); ++cit) {
+        const CSeqFeatXref& ref = **cit; 
+        int contentId = ref.GetId().GetLocal().GetId();
+        if (contentId == xrefId) {
+            return true;
+        }
+    }
+    return false;
+}
+
+//  ============================================================================
+void CGff2Reader::xSetAncestorXrefs(
+    CSeq_feat& descendent,
+    CSeq_feat& ancestor)
+//  ============================================================================
+{
+
+    //xref descendent->ancestor
+    if (!sFeatureHasXref(descendent, ancestor.GetId())) {
+        CRef<CFeat_id> pAncestorId(new CFeat_id);
+        pAncestorId->Assign(ancestor.GetId());
+        CRef<CSeqFeatXref> pAncestorXref(new CSeqFeatXref);
+        pAncestorXref->SetId(*pAncestorId);
+        descendent.SetXref().push_back(pAncestorXref);
+    }
+
+    //xref ancestor->descendent
+    if (!sFeatureHasXref(ancestor, descendent.GetId())) {
+        CRef<CFeat_id> pDescendentId(new CFeat_id);
+        pDescendentId->Assign(descendent.GetId());
+        CRef<CSeqFeatXref> pDescendentXref(new CSeqFeatXref);
+        pDescendentXref->SetId(*pDescendentId);
+        ancestor.SetXref().push_back(pDescendentXref);
+    }
+}
+
 //  ============================================================================
 bool CGff2Reader::xReadInit()
 //  ============================================================================
@@ -1242,16 +1368,25 @@ bool CGff2Reader::IsAlignmentData(
     const string& line)
 //  ============================================================================
 {
-    vector<CTempString> columns;
+    vector<CTempStringEx> columns;
     CGff2Record::TokenizeGFF(columns, line);
     if (columns.size() < 9) {
         return false;
     }
-    if (NStr::StartsWith(columns[2], "match")) {
+    if (NStr::StartsWith(columns[2], "match") || 
+        NStr::EndsWith(columns[2], "_match")) {
         return true;
     }
     return false;
 }
 
+//  ============================================================================
+bool CGff2Reader::xIsIgnoredFeatureType(
+    const string& type)
+//  ============================================================================
+{
+    return false;
+}
+
 END_objects_SCOPE
 END_NCBI_SCOPE
