@@ -1,4 +1,4 @@
-/*  $Id: gff3_reader.cpp 497078 2016-04-04 14:59:49Z ivanov $
+/*  $Id: gff3_reader.cpp 512480 2016-08-31 14:32:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -110,6 +110,46 @@ string CGff3Reader::xNextGenericId()
     return string("generic") + NStr::IntToString(msGenericIdCounter++);
 }
 
+//  ----------------------------------------------------------------------------
+bool CGff3ReadRecord::AssignFromGff(
+    const string& strRawInput )
+//  ----------------------------------------------------------------------------
+{
+    if (!CGff2Record::AssignFromGff(strRawInput)) {
+        return false;
+    }
+    if (m_strType == "pseudogene") {
+        m_strType = "gene";
+        m_Attributes["pseudo"] = "true";
+        return true;
+    }
+    if (m_strType == "pseudogenic_transcript") {
+        m_strType = "transcript";
+        m_Attributes["pseudo"] = "true";
+        return true;
+    }        
+    if (m_strType == "pseudogenic_tRNA") {
+        m_strType = "tRNA";
+        m_Attributes["pseudo"] = "true";
+        return true;
+    }
+    if (m_strType == "pseudogenic_rRNA") {
+        m_strType = "rRNA";
+        m_Attributes["pseudo"] = "true";
+        return true;
+    }
+    if (m_strType == "pseudogenic_exon") {
+        m_strType = "exon";
+        return true;
+    }
+    if (m_strType == "pseudogenic_CDS") {
+        m_strType = "CDS";
+        m_Attributes["pseudo"] = "true";
+        return true;
+    }
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 string CGff3ReadRecord::x_NormalizedAttributeKey(
     const string& strRawKey )
@@ -215,7 +255,7 @@ bool CGff3Reader::x_UpdateAnnotFeature(
 }
 
 //  ----------------------------------------------------------------------------
-void CGff3Reader::xVerifyExonLocation(
+bool CGff3Reader::xVerifyExonLocation(
     const string& mrnaId,
     const CGff2Record& exon,
     ILineErrorListener* pEC)
@@ -223,36 +263,27 @@ void CGff3Reader::xVerifyExonLocation(
 {
     map<string,CRef<CSeq_interval> >::const_iterator cit = mMrnaLocs.find(mrnaId);
     if (cit == mMrnaLocs.end()) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Bad data line: Exon record referring to non-existing mRNA parent.",
-            ILineError::eProblem_FeatureBadStartAndOrStop));
-        ProcessError(*pErr, pEC);
-        return; // can not continue without cit being valid, regardless of ProcessError() 
+        return false;
     }
     const CSeq_interval& containingInt = cit->second.GetObject();
     const CRef<CSeq_loc> pContainedLoc = exon.GetSeqLoc(m_iFlags);
     const CSeq_interval& containedInt = pContainedLoc->GetInt();
     bool failed = (containedInt.GetFrom() < containingInt.GetFrom())  ||
         (containedInt.GetTo() > containingInt.GetTo());
-    if (failed) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Bad data line: Exon record that lies outside its parent location.",
-            ILineError::eProblem_FeatureBadStartAndOrStop));
-        ProcessError(*pErr, pEC);
+    if (containedInt.GetFrom() < containingInt.GetFrom()) {
+        return false;
     }
+    if (containedInt.GetTo() > containingInt.GetTo()) {
+        return false;
+    }
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xUpdateAnnotExon(
     const CGff2Record& record,
-    CRef<CSeq_feat>,
-    CRef<CSeq_annot>,
+    CRef<CSeq_feat> pFeature,
+    CRef<CSeq_annot> pAnnot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
@@ -261,10 +292,32 @@ bool CGff3Reader::xUpdateAnnotExon(
         for (list<string>::const_iterator it = parents.begin(); it != parents.end(); 
                 ++it) {
             const string& parentId = *it; 
-            xVerifyExonLocation(parentId, record, pEC);
+            if (!xVerifyExonLocation(parentId, record, pEC)) {
+                //create a free agent "exon" feature
+                if (!record.InitializeFeature(m_iFlags, pFeature)) {
+                    return false;
+                }
+                CRef<CSeq_feat> pParent;
+                if (!xGetParentFeature(*pFeature, pParent)  ||  
+                        !pParent->GetData().IsGene()) {
+                    AutoPtr<CObjReaderLineException> pErr(
+                        CObjReaderLineException::Create(
+                        eDiag_Error,
+                        0,
+                        "Bad data line: Exon record referring to non-existing mRNA or gene parent.",
+                        ILineError::eProblem_FeatureBadStartAndOrStop));
+                    ProcessError(*pErr, pEC);
+                    return false;
+                }
+                if (! xAddFeatureToAnnot(pFeature, pAnnot)) {
+                    return false;
+                }
+                return true;
+            }
             IdToFeatureMap::iterator fit = m_MapIdToFeature.find(parentId);
             if (fit != m_MapIdToFeature.end()) {
-                if (!record.UpdateFeature(m_iFlags, fit->second)) {
+                CRef<CSeq_feat> pParent = fit->second;
+                if (!record.UpdateFeature(m_iFlags, pParent)) {
                     return false;
                 }
             }
@@ -273,6 +326,7 @@ bool CGff3Reader::xUpdateAnnotExon(
     return true;
 }
 
+
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xUpdateAnnotCds(
     const CGff2Record& record,
@@ -324,14 +378,23 @@ bool CGff3Reader::xUpdateAnnotCds(
         //update parent location:
         IdToFeatureMap::iterator featIt = m_MapIdToFeature.find(parentId);
         if (featIt != m_MapIdToFeature.end()) {
-            if (!record.UpdateFeature(m_iFlags, featIt->second)) {
+            CRef<CSeq_feat> pParent = featIt->second;
+            if (!record.UpdateFeature(m_iFlags, pParent)) {
                 return false;
             }
+            //rw-143:
+            // if parent type is miscRNA then change it to mRNA:
+            if (pParent->GetData().IsRna()  &&  
+                    pParent->GetData().GetRna().GetType()  ==  CRNA_ref::eType_other) {
+                pParent->SetData().SetRna().SetType(CRNA_ref::eType_mRNA);
+            }
         }
 
         //generate applicable CDS ID:
-        string siblingId("cds:");
-        siblingId += parentId;
+        string siblingId("cds");
+        if (!record.GetAttribute("ID", siblingId)  ||  IsInGenbankMode()) {
+            siblingId = string("cds:") + parentId;
+        }
         impliedCdsFeats[siblingId] = parentId;
     }
     // deal with unparented cds
@@ -469,11 +532,25 @@ bool CGff3Reader::xUpdateAnnotGeneric(
     if (featType == "stop_codon_read_through"  ||  featType == "selenocysteine") {
         string cdsParent;
         if (!record.GetAttribute("Parent", cdsParent)) {
-            cerr << "BAD!" << endl;
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Bad data line: Unassigned code break.",
+                ILineError::eProblem_GeneralParsingError) );
+            ProcessError(*pErr, pEC);
+            return false;
         }
         IdToFeatureMap::iterator it = m_MapIdToFeature.find(cdsParent);
         if (it == m_MapIdToFeature.end()) {
-            cerr << "BAD!" << endl;
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Bad data line: Code break assigned to missing feature.",
+                ILineError::eProblem_GeneralParsingError) );
+            ProcessError(*pErr, pEC);
+            return false;
         }
 
         CRef<CCode_break> pCodeBreak(new CCode_break); 
@@ -487,8 +564,6 @@ bool CGff3Reader::xUpdateAnnotGeneric(
         }
         pCodeBreak->SetAa().SetNcbieaa(
             (featType == "selenocysteine") ? 'U' : 'X');
-
-
         CRef<CSeq_feat> pCds = it->second;
         CCdregion& cdRegion = pCds->SetData().SetCdregion();
         list< CRef< CCode_break > >& codeBreaks = cdRegion.SetCode_break();
@@ -627,40 +702,19 @@ bool CGff3Reader::xReadInit()
 
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xIsIgnoredFeatureType(
-    const string& ftype)
+    const string& featureType)
 //  ----------------------------------------------------------------------------
 {
-    vector<string>::const_iterator cit;
+    typedef CStaticArraySet<string, PNocase> STRINGARRAY;
 
-    static vector<string> ignoredTypesAlways;
-    if (ignoredTypesAlways.empty()) {
-        ignoredTypesAlways.push_back("protein");
-    }
-    static vector<string> ignoredTypesGenbank;
-    if (ignoredTypesGenbank.empty()) {
-        ignoredTypesGenbank.push_back("replicon");
-        ignoredTypesGenbank.push_back("chromosome");
-        ignoredTypesGenbank.push_back("dna_chromosome");
-        ignoredTypesGenbank.push_back("rna_chromosome");
-        ignoredTypesGenbank.push_back("apicoplast_chromosome");
-        ignoredTypesGenbank.push_back("chloroplast_chromosome");
-        ignoredTypesGenbank.push_back("chromoplast_chromosome");
-        ignoredTypesGenbank.push_back("cyanelle_chromosome");
-        ignoredTypesGenbank.push_back("leucoplast_chromosome");
-        ignoredTypesGenbank.push_back("macronuclear_chromosome");
-        ignoredTypesGenbank.push_back("micronuclear_chromosome");
-        ignoredTypesGenbank.push_back("mitochondrial_chromosome");
-        ignoredTypesGenbank.push_back("nuclear_chromosome");
-        ignoredTypesGenbank.push_back("nucleomorphic_chromosome");
-        ignoredTypesGenbank.push_back("contig");
-        ignoredTypesGenbank.push_back("supercontig");
-        ignoredTypesGenbank.push_back("ultracontig");
-        ignoredTypesGenbank.push_back("partial_genomic_sequence_assembly");
-        ignoredTypesGenbank.push_back("sequence_assembly");
-        ignoredTypesGenbank.push_back("assembly");
-    }
+    string ftype(featureType);
+    NStr::ToLower(ftype);
 
-    cit = std::find(ignoredTypesAlways.begin(), ignoredTypesAlways.end(), ftype);
+    static const char* const ignoredTypesAlways_[] = {
+        "protein"
+    };
+    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, ignoredTypesAlways, ignoredTypesAlways_);    
+    STRINGARRAY::const_iterator cit = ignoredTypesAlways.find(ftype);
     if (cit != ignoredTypesAlways.end()) {
         return true;
     }
@@ -669,7 +723,71 @@ bool CGff3Reader::xIsIgnoredFeatureType(
     }
 
     /* -genbank mode:*/
-    cit = std::find(ignoredTypesGenbank.begin(), ignoredTypesGenbank.end(), ftype);
+    static const char* const specialTypesGenbank_[] = {
+        "antisense_RNA",
+        "autocatalytically_spliced_intron",
+        "guide_RNA",
+        "hammerhead_ribozyme",
+        "lnc_RNA",
+        "miRNA",
+        "piRNA",
+        "rasiRNA",
+        "ribozyme",
+        "RNase_MRP_RNA",
+        "RNase_P_RNA",
+        "scRNA",
+        "selenocysteine",
+        "siRNA",
+        "snoRNA",
+        "snRNA",
+        "SRP_RNA",
+        "stop_codon_read_through",
+        "telomerase_RNA",
+        "vault_RNA",
+        "Y_RNA"
+    };
+    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, specialTypesGenbank, specialTypesGenbank_);    
+
+    static const char* const ignoredTypesGenbank_[] = {
+        "apicoplast_chromosome",
+        "assembly",
+        "cDNA_match",
+        "chloroplast_chromosome",
+        "chromoplast_chromosome",
+        "chromosome",
+        "contig",
+        "cyanelle_chromosome",
+        "dna_chromosome",
+        "EST_match",
+        "expressed_sequence_match",
+        "intron",
+        "leucoplast_chromosome",
+        "macronuclear_chromosome",
+        "match",
+        "match_part",
+        "micronuclear_chromosome",
+        "mitochondrial_chromosome",
+        "nuclear_chromosome",
+        "nucleomorphic_chromosome",
+        "nucleotide_motif",
+        "nucleotide_to_protein_match",
+        "partial_genomic_sequence_assembly",
+        "protein_match",
+        "replicon",
+        "rna_chromosome",
+        "sequence_assembly",
+        "supercontig",
+        "translated_nucleotide_match",
+        "ultracontig",
+    };
+    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, ignoredTypesGenbank, ignoredTypesGenbank_);    
+
+    cit = specialTypesGenbank.find(ftype);
+    if (cit != specialTypesGenbank.end()) {
+        return false;
+    }
+
+    cit = ignoredTypesGenbank.find(ftype);
     if (cit != ignoredTypesGenbank.end()) {
         return true;
     }
