@@ -1,4 +1,4 @@
-/*  $Id: microarray_reader.cpp 448857 2014-10-09 16:31:19Z ivanov $
+/*  $Id: microarray_reader.cpp 491626 2016-02-08 14:58:45Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -101,8 +101,8 @@ CMicroArrayReader::CMicroArrayReader(
 //  ----------------------------------------------------------------------------
     : CReaderBase(flags),
       m_currentId(""),
-      m_usescore(false),
-      m_columncount(15)
+      m_columncount(15),
+      m_usescore(false)
 {
 }
 
@@ -116,7 +116,7 @@ CMicroArrayReader::~CMicroArrayReader()
 CRef< CSerialObject >
 CMicroArrayReader::ReadObject(
     ILineReader& lr,
-    IMessageListener* pMessageListener ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------                
 { 
     CRef<CSerialObject> object( 
@@ -128,7 +128,7 @@ CMicroArrayReader::ReadObject(
 CRef< CSeq_annot >
 CMicroArrayReader::ReadSeqAnnot(
     ILineReader& lr,
-    IMessageListener* pEC) 
+    ILineErrorListener* pEC) 
 //  ----------------------------------------------------------------------------                
 {
     const int MAX_RECORDS = 100000;
@@ -160,7 +160,7 @@ CMicroArrayReader::ReadSeqAnnot(
 
         //  parse
         vector<string> fields;
-        NStr::Tokenize( record_copy, " \t", fields, NStr::eMergeDelims );
+        NStr::Split( record_copy, " \t", fields, NStr::eMergeDelims );
         try {
             xCleanColumnValues(fields);
         }
@@ -207,7 +207,7 @@ CMicroArrayReader::ReadSeqAnnot(
 bool CMicroArrayReader::xParseFeature(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     const size_t columncount = 15;
@@ -305,7 +305,7 @@ void CMicroArrayReader::xSetFeatureDisplayData(
 //  ----------------------------------------------------------------------------
 bool CMicroArrayReader::xParseTrackLine(
     const string& strLine,
-    IMessageListener* pEC)
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     m_strExpNames = "";
