@@ -1,4 +1,4 @@
-/*  $Id: reader_base.cpp 167543 2009-08-04 11:54:16Z ludwigf $
+/*  $Id: reader_base.cpp 504304 2016-06-14 11:38:41Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -80,256 +80,326 @@
 #include <objects/seqfeat/Gb_qual.hpp>
 #include <objects/seqfeat/Feat_id.hpp>
 
+#include <objtools/readers/read_util.hpp>
 #include <objtools/readers/reader_exception.hpp>
 #include <objtools/readers/line_error.hpp>
-#include <objtools/readers/error_container.hpp>
+#include <objtools/readers/message_listener.hpp>
+#include <objtools/readers/track_data.hpp>
 #include <objtools/readers/reader_base.hpp>
 #include <objtools/readers/bed_reader.hpp>
 #include <objtools/readers/microarray_reader.hpp>
 #include <objtools/readers/wiggle_reader.hpp>
 #include <objtools/readers/gff3_reader.hpp>
+#include <objtools/readers/gvf_reader.hpp>
+#include <objtools/readers/vcf_reader.hpp>
+#include <objtools/readers/rm_reader.hpp>
+#include <objtools/readers/fasta.hpp>
+#include <objtools/readers/readfeat.hpp>
 #include <objtools/error_codes.hpp>
+#include <objtools/readers/ucscregion_reader.hpp>
 
 #include <algorithm>
+#include <ctime>
 
+#include "reader_data.hpp"
 
 #define NCBI_USE_ERRCODE_X   Objtools_Rd_RepMask
 
 BEGIN_NCBI_SCOPE
-
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
 //  ----------------------------------------------------------------------------
 CReaderBase*
 CReaderBase::GetReader(
     CFormatGuess::EFormat format,
-    int flags )
+    TReaderFlags flags )
 //  ----------------------------------------------------------------------------
 {
     switch ( format ) {
     default:
         return 0;
     case CFormatGuess::eBed:
-        return new CBedReader( flags );
+        return new CBedReader(flags);
     case CFormatGuess::eBed15:
-        return new CMicroArrayReader( flags );
+        return new CMicroArrayReader(flags);
     case CFormatGuess::eWiggle:
-        return new CWiggleReader( flags );
+        return new CWiggleReader(flags);
     case CFormatGuess::eGtf:
-        return new CGff3Reader( flags );
+    case CFormatGuess::eGtf_POISENED:
+        return new CGff3Reader(flags);
+    case CFormatGuess::eGff3:
+        return new CGff3Reader(flags);
+    case CFormatGuess::eGvf:
+        return new CGvfReader(flags);
+    case CFormatGuess::eVcf:
+        return new CVcfReader(flags);
+    case CFormatGuess::eRmo:
+        return new CRepeatMaskerReader(flags);
+    case CFormatGuess::eFasta:
+        return new CFastaReader(flags);
+    case CFormatGuess::eFiveColFeatureTable:
+        return new CFeature_table_reader(flags);
+    case CFormatGuess::eUCSCRegion:
+        return new CUCSCRegionReader(flags);
     }
 }
 
+//  ----------------------------------------------------------------------------
+CReaderBase::CReaderBase(
+    TReaderFlags flags,
+    const string& annotName,
+    const string& annotTitle) :
+//  ----------------------------------------------------------------------------
+    m_uLineNumber(0),
+    m_uProgressReportInterval(0),
+	m_uNextProgressReport(0),
+    m_iFlags(flags),
+    m_AnnotName(annotName),
+    m_AnnotTitle(annotTitle),
+    m_pReader(0),
+    m_pCanceler(0)
+{
+    m_pTrackDefaults = new CTrackData;
+}
+
+//  ----------------------------------------------------------------------------
+CReaderBase::~CReaderBase()
+//  ----------------------------------------------------------------------------
+{
+    delete m_pTrackDefaults;
+}
+
 //  ----------------------------------------------------------------------------
 CRef< CSerialObject >
 CReaderBase::ReadObject(
     CNcbiIstream& istr,
-    IErrorContainer* pErrorContainer ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------
 {
     CStreamLineReader lr( istr );
-    return ReadObject( lr, pErrorContainer );
+    return ReadObject( lr, pMessageListener );
 }
 
 //  ----------------------------------------------------------------------------
 CRef< CSeq_annot >
 CReaderBase::ReadSeqAnnot(
     CNcbiIstream& istr,
-    IErrorContainer* pErrorContainer ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------
 {
     CStreamLineReader lr( istr );
-    return ReadSeqAnnot( lr, pErrorContainer );
+    return ReadSeqAnnot( lr, pMessageListener );
 }
 
 //  ----------------------------------------------------------------------------
 CRef< CSeq_annot >
 CReaderBase::ReadSeqAnnot(
-    ILineReader&,
-    IErrorContainer* ) 
+    ILineReader& lr,
+    ILineErrorListener* ) 
 //  ----------------------------------------------------------------------------
 {
-    CRef<CSeq_annot> object( new CSeq_annot() );
-    return object;
+    xProgressInit(lr);
+    return CRef<CSeq_annot>();
 }
                 
+//  --------------------------------------------------------------------------- 
+void
+CReaderBase::ReadSeqAnnots(
+    TAnnots& annots,
+    CNcbiIstream& istr,
+    ILineErrorListener* pMessageListener )
+//  ---------------------------------------------------------------------------
+{
+    CStreamLineReader lr( istr );
+    ReadSeqAnnots( annots, lr, pMessageListener );
+}
+ 
+//  ---------------------------------------------------------------------------                       
+void
+CReaderBase::ReadSeqAnnots(
+    TAnnots& annots,
+    ILineReader& lr,
+    ILineErrorListener* pMessageListener )
+//  ----------------------------------------------------------------------------
+{
+    xProgressInit(lr);
+    CRef<CSeq_annot> annot = ReadSeqAnnot(lr, pMessageListener);
+    while (annot) {
+        //xAssignTrackData(annot);
+        annots.push_back(annot);
+        annot = ReadSeqAnnot(lr, pMessageListener);
+    }
+}
+                        
 //  ----------------------------------------------------------------------------
 CRef< CSeq_entry >
 CReaderBase::ReadSeqEntry(
     CNcbiIstream& istr,
-    IErrorContainer* pErrorContainer ) 
+    ILineErrorListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------
 {
     CStreamLineReader lr( istr );
-    return ReadSeqEntry( lr, pErrorContainer );
+    CRef<CSeq_entry> pResult = ReadSeqEntry( lr, pMessageListener );
+    return pResult;
 }
 
 //  ----------------------------------------------------------------------------
 CRef< CSeq_entry >
 CReaderBase::ReadSeqEntry(
-    ILineReader&,
-    IErrorContainer* ) 
+    ILineReader& lr,
+    ILineErrorListener* ) 
 //  ----------------------------------------------------------------------------
 {
-    CRef<CSeq_entry> object( new CSeq_entry() );
-    return object;
+    xProgressInit(lr);
+    return CRef<CSeq_entry>();
 }
-                
+               
 //  ----------------------------------------------------------------------------
-bool CReaderBase::SplitLines( 
-    const char* pcBuffer, 
-    size_t uSize,
-    list<string>& lines )
+void
+CReaderBase::ProcessError(
+    CObjReaderLineException& err,
+    ILineErrorListener* pContainer )
 //  ----------------------------------------------------------------------------
 {
-    //
-    //  Make sure the given data is ASCII before checking potential line breaks:
-    //
-    const size_t MIN_HIGH_RATIO = 20;
-    size_t high_count = 0;
-    for ( size_t i=0; i < uSize; ++i ) {
-        if ( 0x80 & pcBuffer[i] ) {
-            ++high_count;
-        }
+    err.SetLineNumber( m_uLineNumber );
+    if (!pContainer) {
+        err.Throw();
     }
-    if ( 0 < high_count && uSize / high_count < MIN_HIGH_RATIO ) {
-        return false;
+    if (!pContainer->PutError(err)) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Critical,
+            0,
+            "Error allowance exceeded",
+            ILineError::eProblem_GeneralParsingError) );
+        pErr->Throw();
+        //err.Throw();
     }
+}
 
-    //
-    //  Let's expect at least one line break in the given data:
-    //
-    string data( pcBuffer, uSize );
-    
-    lines.clear();
-    if ( NStr::Split( data, "\r\n", lines ).size() > 1 ) {
-        return true;
+//  ----------------------------------------------------------------------------
+void
+CReaderBase::ProcessWarning(
+    CObjReaderLineException& err,
+    ILineErrorListener* pContainer )
+//  ----------------------------------------------------------------------------
+{
+    err.SetLineNumber( m_uLineNumber );
+    if (!pContainer) {
+        cerr << m_uLineNumber << ": " << err.SeverityStr() << err.Message() 
+            << endl;
+        return;
     }
-    lines.clear();
-    if ( NStr::Split( data, "\r", lines ).size() > 1 ) {
-        return true;
+    if (!pContainer->PutError(err)) {
+        err.Throw();
     }
-    lines.clear();
-    if ( NStr::Split( data, "\n", lines ).size() > 1 ) {
-        return true;
-    }
-    
-    //
-    //  Suspicious for non-binary files. Unfortunately, it can actually happen
-    //  for Newick tree files.
-    //
-    lines.clear();
-    lines.push_back( data );
-    return true;
 }
 
 //  ----------------------------------------------------------------------------
 void
 CReaderBase::ProcessError(
-    CObjReaderLineException& err,
-    IErrorContainer* pContainer )
+    CLineError& err,
+    ILineErrorListener* pContainer )
 //  ----------------------------------------------------------------------------
 {
-    err.SetLineNumber( m_uLineNumber );
-    if ( 0 == pContainer ) {
-        throw( err );
+    if (!pContainer  ||  !pContainer->PutError(err)) {
+        err.Throw();
     }
-    if ( ! pContainer->PutError( err ) )
-    {
-        throw( err );
-    }
-}
+ }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_SetBrowserRegion(
+void
+CReaderBase::ProcessWarning(
+    CLineError& err,
+    ILineErrorListener* pContainer )
+//  ----------------------------------------------------------------------------
+{
+    if (!pContainer) {
+        cerr << m_uLineNumber << ": " << err.SeverityStr() << err.Message() 
+            << endl;
+        return;
+    }
+    if (!pContainer->PutError(err)) {
+        err.Throw();
+    }
+ }
+
+//  ----------------------------------------------------------------------------
+void CReaderBase::xSetBrowserRegion(
     const string& strRaw,
-    CAnnot_descr& desc )
+    CAnnot_descr& desc,
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     CRef<CSeq_loc> location( new CSeq_loc );
-    CSeq_interval& interval = location->SetInt();
 
     string strChrom;
     string strInterval;
     if ( ! NStr::SplitInTwo( strRaw, ":", strChrom, strInterval ) ) {
-        CObjReaderLineException err(
-            eDiag_Error,
-            0,
-            "Bad browser: cannot parse browser position" );
-        throw( err );
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Bad browser line: cannot parse browser position" ) );
+        ProcessError(*pErr, pEC);
     }
     CRef<CSeq_id> id( new CSeq_id( CSeq_id::e_Local, strChrom ) );
-    location->SetId( *id );
 
-    string strFrom;
-    string strTo;
-    if ( ! NStr::SplitInTwo( strInterval, "-", strFrom, strTo ) ) {
-        CObjReaderLineException err(
-            eDiag_Error,
-            0,
-            "Bad browser: cannot parse browser position" );
-        throw( err );
-    }    
-    interval.SetFrom( NStr::StringToInt( strFrom ) - 1);
-    interval.SetTo( NStr::StringToInt( strTo ) - 1 );
-    interval.SetStrand( eNa_strand_unknown );
-
-    CRef<CAnnotdesc> region( new CAnnotdesc() );
-    region->SetRegion( *location );
-    desc.Set().push_back( region );
-}
-    
-//  ----------------------------------------------------------------------------
-void CReaderBase::x_GetTrackValues(
-    const string& strLine,
-    map<string, string>& values )
-//  ----------------------------------------------------------------------------
-{
-    string strTemp( strLine );
-    
-    //
-    //  Hide blanks inside of string literals
-    //
-    bool bInString = false;
-    for ( size_t u=0; u < strTemp.size(); ++u ) {
-        switch ( strTemp[u] ) {
-        default:
-            break;
-        case '\"':
-            bInString = !bInString;
-            break;
-        case ' ':
-            if ( bInString ) {
-                strTemp[u] = '\"';
-            }
-            break;
-        }
+    if (NStr::Compare(strInterval, "start-stop") == 0 )
+    {
+        location->SetWhole(*id);
     }
-    vector<string> fields;
-    NStr::Tokenize( strTemp, " \t", fields, NStr::eMergeDelims );
-    for ( vector<string>::size_type i=1; i < fields.size(); ++i ) {
-        vector<string> splits;
-        NStr::Tokenize( fields[i], "=", splits, NStr::eMergeDelims );        
-        if ( splits.size() != 2 ) {
-            CObjReaderLineException err(
-                eDiag_Warning,
+    else
+    {
+        string strFrom;
+        string strTo;
+        if ( ! NStr::SplitInTwo( strInterval, "-", strFrom, strTo ) ) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
                 0,
-                "Bad track line: key=value pair expected" );
-            throw( err );
+                "Bad browser line: cannot parse browser position" ) );
+            ProcessError(*pErr, pEC);
+        }  
+        try
+        {
+            int n_from,n_to;
+
+            n_from = NStr::StringToInt(strFrom, NStr::fAllowCommas);
+            n_to   = NStr::StringToInt(strTo, NStr::fAllowCommas);
+
+            CSeq_interval& interval = location->SetInt();
+            interval.SetFrom(n_from-1);
+            interval.SetTo(n_to-1);
+            interval.SetStrand( eNa_strand_unknown );
+            location->SetId( *id );
+
         }
-        string strKey = splits[0];
-        //  Restore the hidden blanks, remove quotes     
-        string strValue = splits[1];
-        NStr::ReplaceInPlace( strValue, "\"", " " );
-        NStr::TruncateSpacesInPlace( strValue );
-        values[ strKey ] = strValue;
+        catch (const CStringException&)
+        {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Bad browser line: cannot parse browser position" ) );
+            ProcessError(*pErr, pEC);
+            location.Release();
+        }
+    }
+
+    if (location.NotEmpty())
+    {
+        CRef<CAnnotdesc> region( new CAnnotdesc() );
+        region->SetRegion( *location );
+        desc.Set().push_back( region );
     }
 }
-
+    
 //  ----------------------------------------------------------------------------
-bool CReaderBase::x_ParseBrowserLine(
+bool CReaderBase::xParseBrowserLine(
     const string& strLine,
-    CRef<CSeq_annot>& annot )
+    CRef<CSeq_annot>& annot,
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     if ( ! NStr::StartsWith( strLine, "browser" ) ) {
@@ -338,55 +408,87 @@ bool CReaderBase::x_ParseBrowserLine(
     CAnnot_descr& desc = annot->SetDesc();
     
     vector<string> fields;
-    NStr::Tokenize( strLine, " \t", fields, NStr::eMergeDelims );
+    NStr::Split( strLine, " \t", fields, NStr::eMergeDelims );
     for ( vector<string>::iterator it = fields.begin(); it != fields.end(); ++it ) {
         if ( *it == "position" ) {
             ++it;
             if ( it == fields.end() ) {
-                CObjReaderLineException err(
+                AutoPtr<CObjReaderLineException> pErr(
+                    CObjReaderLineException::Create(
                     eDiag_Error,
                     0,
-                    "Bad browser line: incomplete position directive" );
-                throw( err );
+                    "Bad browser line: incomplete position directive" ) );
+                ProcessError(*pErr, pEC);
             }
-            x_SetBrowserRegion( *it, desc );
-            continue;
+            xSetBrowserRegion(*it, desc, pEC);
         }
     }
 
     return true;
 }
 
-
 //  ----------------------------------------------------------------------------
-bool CReaderBase::x_ParseTrackLine(
-    const string& strLine,
+void CReaderBase::xAssignTrackData(
     CRef<CSeq_annot>& annot )
 //  ----------------------------------------------------------------------------
 {
-    if ( ! NStr::StartsWith( strLine, "track" ) ) {
-        return false;
+    if (!m_AnnotName.empty()) {
+        annot->SetNameDesc(m_AnnotName);
+    }
+    if (!m_AnnotTitle.empty()) {
+        annot->SetTitleDesc(m_AnnotTitle);
+    }
+    if (!m_pTrackDefaults->ContainsData()) {
+        return;
     }
     CAnnot_descr& desc = annot->SetDesc();
-
     CRef<CUser_object> trackdata( new CUser_object() );
-    trackdata->SetType().SetStr( "Track Data" );    
-    
-    map<string, string> values;
-    x_GetTrackValues( strLine, values );
-    for ( map<string,string>::iterator it = values.begin(); it != values.end(); ++it ) {
-        x_SetTrackData( annot, trackdata, it->first, it->second );
+    trackdata->SetType().SetStr( "Track Data" );
+   
+    if ( !m_pTrackDefaults->Description().empty() ) {
+        annot->SetTitleDesc(m_pTrackDefaults->Description());
+    }
+    if ( !m_pTrackDefaults->Name().empty() ) {
+        annot->SetNameDesc(m_pTrackDefaults->Name());
+    }
+    map<const string,string>::const_iterator cit = m_pTrackDefaults->Values().begin();
+    while ( cit != m_pTrackDefaults->Values().end() ) {
+        trackdata->AddField( cit->first, cit->second );
+        ++cit;
     }
     if ( trackdata->CanGetData() && ! trackdata->GetData().empty() ) {
         CRef<CAnnotdesc> user( new CAnnotdesc() );
         user->SetUser( *trackdata );
         desc.Set().push_back( user );
     }
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xParseTrackLine(
+    const string& strLine,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    vector<string> parts;
+    CReadUtil::Tokenize( strLine, " \t", parts );
+    if ( !CTrackData::IsTrackData( parts ) ) {
+        return false;
+    }
+    m_pTrackDefaults->ParseLine( parts );
     return true;
 }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_SetTrackData(
+bool CReaderBase::xParseBrowserLine(
+    const string& strLine,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+void CReaderBase::xSetTrackData(
     CRef<CSeq_annot>& annot,
     CRef<CUser_object>& trackdata,
     const string& strKey,
@@ -397,56 +499,214 @@ void CReaderBase::x_SetTrackData(
 }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_AddConversionInfo(
+bool CReaderBase::xParseComment(
+    const CTempString& record,
+    CRef<CSeq_annot>& annot ) /* throws CObjReaderLineException */
+//  ----------------------------------------------------------------------------
+{
+    if (NStr::StartsWith(record, "#")) {
+        return true;
+    }
+    return false;
+}
+ 
+//  ----------------------------------------------------------------------------
+void CReaderBase::xAddConversionInfo(
     CRef<CSeq_annot >& annot,
-    IErrorContainer *pErrorContainer )
+    ILineErrorListener *pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    if ( !annot || !pErrorContainer ) {
+    if (!annot || !pMessageListener) {
+        return;
+    }
+    if (0 == pMessageListener->LevelCount(eDiag_Critical)  &&
+        0 == pMessageListener->LevelCount(eDiag_Error) &&
+        0 == pMessageListener->LevelCount(eDiag_Warning) &&
+        0 == pMessageListener->LevelCount(eDiag_Info)) {
         return;
     }
-    CAnnot_descr& desc = annot->SetDesc();
 
-    CRef<CUser_object> conversioninfo( new CUser_object() );
-    conversioninfo->SetType().SetStr( "Conversion Info" );    
-    conversioninfo->AddField( 
-        "critical errors", int ( pErrorContainer->LevelCount( eDiag_Critical ) ) );
-    conversioninfo->AddField( 
-        "errors", int ( pErrorContainer->LevelCount( eDiag_Error ) ) );
-    conversioninfo->AddField( 
-        "warnings", int ( pErrorContainer->LevelCount( eDiag_Warning ) ) );
-    conversioninfo->AddField( 
-        "notes", int ( pErrorContainer->LevelCount( eDiag_Info ) ) );
-    CRef<CAnnotdesc> user( new CAnnotdesc() );
-    user->SetUser( *conversioninfo );
-    desc.Set().push_back( user );
+
+    CRef<CAnnotdesc> user(new CAnnotdesc());
+    user->SetUser(*xMakeAsnConversionInfo(pMessageListener));
+    annot->SetDesc().Set().push_back(user);
 }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_AddConversionInfo(
+void CReaderBase::xAddConversionInfo(
     CRef<CSeq_entry >& entry,
-    IErrorContainer *pErrorContainer )
+    ILineErrorListener *pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    if ( !entry || !pErrorContainer ) {
+    if (!entry || !pMessageListener) {
         return;
     }
-    CSeq_descr& descr = entry->SetDescr();
-
-    CRef<CUser_object> conversioninfo( new CUser_object() );
-    conversioninfo->SetType().SetStr( "Conversion Info" );    
-    conversioninfo->AddField( 
-        "critical errors", int ( pErrorContainer->LevelCount( eDiag_Critical ) ) );
-    conversioninfo->AddField( 
-        "errors", int ( pErrorContainer->LevelCount( eDiag_Error ) ) );
-    conversioninfo->AddField( 
-        "warnings", int ( pErrorContainer->LevelCount( eDiag_Warning ) ) );
-    conversioninfo->AddField( 
-        "notes", int ( pErrorContainer->LevelCount( eDiag_Info ) ) );
-    CRef<CSeqdesc> user( new CSeqdesc() );
-    user->SetUser( *conversioninfo );
-    descr.Set().push_back( user );
+    CRef<CSeqdesc> user(new CSeqdesc());
+    user->SetUser(*xMakeAsnConversionInfo(pMessageListener));
+    entry->SetDescr().Set().push_back(user);
 }
 
+//  ----------------------------------------------------------------------------
+CRef<CUser_object> CReaderBase::xMakeAsnConversionInfo(
+    ILineErrorListener* pMessageListener )
+//  ----------------------------------------------------------------------------
+{
+    CRef<CUser_object> conversioninfo(new CUser_object());
+    conversioninfo->SetType().SetStr("Conversion Info");    
+    conversioninfo->AddField( 
+        "critical errors", int(pMessageListener->LevelCount(eDiag_Critical)));
+    conversioninfo->AddField( 
+        "errors", int(pMessageListener->LevelCount(eDiag_Error)));
+    conversioninfo->AddField( 
+        "warnings", int(pMessageListener->LevelCount(eDiag_Warning)));
+    conversioninfo->AddField( 
+        "notes", int(pMessageListener->LevelCount(eDiag_Info)));
+    return conversioninfo;
+}
+
+//  ----------------------------------------------------------------------------
+void CReaderBase::SetProgressReportInterval(
+    unsigned int intv)
+//  ----------------------------------------------------------------------------
+{
+    m_uProgressReportInterval = intv;
+    m_uNextProgressReport = (unsigned int)time(0) + intv;
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsReportingProgress() const
+//  ----------------------------------------------------------------------------
+{
+    if (0 == m_uProgressReportInterval) {
+        return false;
+    }
+    if (0 == m_pReader) {
+        return false;
+    }
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+void CReaderBase::xReportProgress(
+    ILineErrorListener* pProgress)
+//  ----------------------------------------------------------------------------
+{
+    if (!xIsReportingProgress()) { // progress reports disabled
+        return;
+    }
+    unsigned int uCurrentTime = (unsigned int)time(0);
+    if (uCurrentTime < m_uNextProgressReport) { // not time yet
+        return;
+    }
+    // report something
+    ios::streampos curPos = m_pReader->GetPosition();
+    pProgress->PutProgress("Progress", Uint8(curPos), 0);
+
+    m_uNextProgressReport += m_uProgressReportInterval;
+}
+
+//  ============================================================================
+bool CReaderBase::xReadInit()
+//  ============================================================================
+{
+    return true;
+}
+
+//  ============================================================================
+bool CReaderBase::xProgressInit(
+    ILineReader& lr)
+//  ============================================================================
+{
+    if (0 == m_uProgressReportInterval) {
+        return true;
+    }
+    m_pReader = &lr;
+    return true;
+}
+
+//  ============================================================================
+void CReaderBase::SetCanceler(
+    ICanceled* pCanceler)
+//  ============================================================================
+{
+    m_pCanceler = pCanceler;
+}
+
+//  ============================================================================
+bool CReaderBase::xIsOperationCanceled() const
+//  ============================================================================
+{
+    if (!m_pCanceler) {
+        return false;
+    }
+    return m_pCanceler->IsCanceled();
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsCommentLine(
+    const CTempString& strLine)
+//  ----------------------------------------------------------------------------
+{
+    if (strLine.empty()) {
+        return true;
+    }
+    return (strLine[0] == '#' && strLine[1] != '#');
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsTrackLine(
+    const CTempString& strLine)
+//  ----------------------------------------------------------------------------
+{
+    if (NStr::StartsWith(strLine, "track ")) {
+        return true;
+    }
+    return NStr::StartsWith(strLine, "track\t");
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsTrackTerminator(
+    const CTempString& strLine)
+//  ----------------------------------------------------------------------------
+{
+    return (NStr::TruncateSpaces(strLine) == "###");
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsBrowserLine(
+    const CTempString& strLine)
+//  ----------------------------------------------------------------------------
+{
+    return NStr::StartsWith(strLine, "browser ");
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xGetLine(
+    ILineReader& lr,
+    string& line)
+//  ----------------------------------------------------------------------------
+{
+    CTempString temp;
+    while (!lr.AtEOF()) {
+        temp = *++lr;
+        ++m_uLineNumber;
+        temp = NStr::TruncateSpaces_Unsafe(temp);
+        if (!xIsCommentLine(temp)) {
+            line = temp;
+            return true;
+        }
+    }
+    return false;
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xUngetLine(
+    ILineReader& lr)
+//  ----------------------------------------------------------------------------
+{
+    lr.UngetLine();
+    --m_uLineNumber;
+    return true;
+}
 END_objects_SCOPE
 END_NCBI_SCOPE
