@@ -1,4 +1,4 @@
-/*  $Id: enumtype.cpp 413810 2013-09-18 15:28:07Z gouriano $
+/*  $Id: enumtype.cpp 485707 2015-11-25 13:58:19Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -158,7 +158,15 @@ void CEnumDataType::PrintXMLSchema(CNcbiOstream& out,
                 use = "optional";
             }
         }
+#if _DATATOOL_USE_SCHEMA_STYLE_COMMENTS
+        PrintASNNewLine(out, indent) << tmp;
+        if (!Comments().PrintSchemaComments(out, indent+1)) {
+            out << '>';
+        }
+        ++indent;
+#else
         opentag.push_back(tmp + ">");
+#endif
         closetag.push_front("</xs:element>");
         opentag.push_back("<xs:complexType>");
         closetag.push_front("</xs:complexType>");
@@ -171,8 +179,8 @@ void CEnumDataType::PrintXMLSchema(CNcbiOstream& out,
         }
     }
     string tmp = "<xs:attribute name=\"" + value + "\" use=\"" + use + "\"" + form;
+    const CDataMember* mem = GetDataMember();
     if (!inAttlist) {
-        const CDataMember* mem = GetDataMember();
         if (mem && mem->Optional() && mem->GetDefault()) {
             tmp += " default=\"" + GetXmlValueName(mem->GetDefault()->GetXmlString()) + "\"";
         }
@@ -195,14 +203,25 @@ void CEnumDataType::PrintXMLSchema(CNcbiOstream& out,
         }
     }
     if ( haveComments ) {
-        out << "\n<!--\n";
+#if _DATATOOL_USE_SCHEMA_STYLE_COMMENTS
+        PrintASNNewLine(out, indent) << "<xs:annotation><xs:documentation>";
         ITERATE ( TValues, i, m_Values ) {
             if ( !i->GetComments().Empty() ) {
-                i->GetComments().Print(out, "    "+i->GetName()+"\t- ",
-                                       "\n        ", "\n");
+                i->GetComments().Print(out, "\n "+i->GetName()+"\t- ",
+                                            "\n ", "");
             }
         }
-        out << "-->";
+        PrintASNNewLine(out, indent) << "</xs:documentation></xs:annotation>";
+#else
+        out << "\n<!--";
+        ITERATE ( TValues, i, m_Values ) {
+            if ( !i->GetComments().Empty() ) {
+                i->GetComments().Print(out, "\n    "+i->GetName()+"\t- ",
+                                       "\n        ", "");
+            }
+        }
+        out << "\n-->";
+#endif
     }
     ITERATE ( TValues, i, m_Values ) {
         PrintASNNewLine(out, indent) <<
@@ -314,7 +333,7 @@ bool CEnumDataType::CheckValue(const CDataValue& value) const
             if ( i->GetValue() == intValue->GetValue() )
                 return true;
         }
-        value.Warning("illegal INTEGER value: " + intValue->GetValue(), 14);
+        value.Warning("illegal INTEGER value: " + NStr::NumericToString(intValue->GetValue()), 14);
         return false;
     }
 
@@ -430,6 +449,20 @@ AutoPtr<CTypeStrings> CEnumDataType::GetRefCType(void) const
 
 AutoPtr<CTypeStrings> CEnumDataType::GetFullCType(void) const
 {
+    ITERATE ( TValues, i, m_Values ) {
+        string id(Identifier( i->GetEnumId(), false ));
+        string value;
+        value = GetVar(id + "._hidename", false);
+        if (!value.empty()) {
+            if (NStr::StringToBool(value)) {
+                i->SetFlag(CEnumDataTypeValue::eHideName);
+            }
+        }
+        value = GetVar( id, false);
+        if (!value.empty()) {
+            i->SetEnumId(value);
+        }
+    }
 // in case client wants std type instead of enum.
 // I must be accurate here to not to mess with GetEnumCInfo()
     string type = GetAndVerifyVar("_type");
