@@ -1,4 +1,4 @@
-/*  $Id: xsdparser.cpp 432804 2014-04-18 13:01:30Z gouriano $
+/*  $Id: xsdparser.cpp 486439 2015-12-04 15:06:23Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -160,6 +160,7 @@ TToken XSDParser::GetNextToken(void)
             ParseError("Unexpected data", "attribute (name=\"value\")");
         }
         NStr::TruncateSpacesInPlace(str1);
+        NStr::TruncateSpacesInPlace(data2);
 // attribute
         data = str1;
         if (NStr::SplitInTwo(data, ":", str1, str2)) {
@@ -255,8 +256,10 @@ bool XSDParser::DefineElementType(DTDElement& node)
     if (IsValue("string") || IsValue("token") ||
         IsValue("normalizedString") ||
         IsValue("duration") || IsValue("language") ||
-        IsValue("anyType") || IsValue("anyURI") || IsValue("QName") ||
+        IsValue("anyType") || IsValue("anyURI") ||
+        IsValue("Name") ||IsValue("NCName") || IsValue("QName") ||
         IsValue("dateTime") || IsValue("time") || IsValue("date") ||
+        IsValue("gYearMonth") || IsValue("gYear") || IsValue("gMonthDay") || IsValue("gMonth") || IsValue("gDay") || 
         IsValue("anySimpleType")) {
         node.SetType(DTDElement::eString);
     } else if (IsValue("double") || IsValue("float") || IsValue("decimal")) {
@@ -338,7 +341,7 @@ void XSDParser::ParseHeader()
         tok = GetNextToken();
     } else {
         if (m_SrcType == eSchema) {
-            ERR_POST_X(4, "LINE " << Location() << " XML declaration is missing");
+            ERR_POST_X(4, GetLocation() << " XML declaration is missing");
         }
     }
 // schema    
@@ -532,7 +535,7 @@ DTDElement::EOccurrence XSDParser::ParseMinOccurs( DTDElement::EOccurrence occNo
                 occNew = DTDElement::eZeroOrMore;
             }
         } else if (m > 1) {
-            ERR_POST_X(8, Warning << "Unsupported element minOccurs= " << m);
+            ERR_POST_X(8, Warning << GetLocation() << "Unsupported element minOccurs= " << m);
             occNew = DTDElement::eOneOrMore;
         }
     }
@@ -546,7 +549,7 @@ DTDElement::EOccurrence XSDParser::ParseMaxOccurs( DTDElement::EOccurrence occNo
         int m = IsValue("unbounded") ? -1 : NStr::StringToInt(m_Value);
         if (m == -1 || m > 1) {
             if (m > 1) {
-                ERR_POST_X(8, Warning << "Unsupported element maxOccurs= " << m);
+                ERR_POST_X(8, Warning << GetLocation() << "Unsupported element maxOccurs= " << m);
             }
             if (occNow == DTDElement::eOne) {
                 occNew = DTDElement::eOneOrMore;
@@ -571,6 +574,9 @@ string XSDParser::ParseElementContent(DTDElement* owner, int emb)
     tok = GetRawAttributeSet();
 
     if (GetAttribute("ref")) {
+        if (!owner) {
+            ParseError("ref attribute not allowed for root elements", "");
+        }
         if (IsValue("schema") &&
             GetElementNamespace(m_ValuePrefix) == eSchemaNamespace) {
             name = CreateTmpEmbeddedName(owner->GetName(), emb);
@@ -1036,6 +1042,7 @@ void XSDParser::ParseAny(DTDElement& node)
 void XSDParser::ParseUnion(DTDElement& node)
 {
     ERR_POST_X(9, Warning
+        << GetLocation()
         << "Unsupported element type: union; in node "
         << node.GetName());
     node.SetType(DTDElement::eString);
@@ -1050,6 +1057,7 @@ void XSDParser::ParseUnion(DTDElement& node)
 void XSDParser::ParseList(DTDElement& node)
 {
     ERR_POST_X(10, Warning
+        << GetLocation()
         << "Unsupported element type: list; in node "
         << node.GetName());
     node.SetType(DTDElement::eString);
@@ -1191,6 +1199,7 @@ void XSDParser::ParseEnumeration(DTDAttribute& att)
 void XSDParser::ParseUnion(DTDAttribute& att)
 {
     ERR_POST_X(9, Warning
+        << GetLocation()
         << "Unsupported attribute type: union; in attribute "
         << att.GetName());
     att.SetType(DTDAttribute::eString);
@@ -1204,6 +1213,7 @@ void XSDParser::ParseUnion(DTDAttribute& att)
 void XSDParser::ParseList(DTDAttribute& att)
 {
     ERR_POST_X(10, Warning
+        << GetLocation()
         << "Unsupported attribute type: list; in attribute "
         << att.GetName());
     att.SetType(DTDAttribute::eString);
@@ -1251,7 +1261,7 @@ string XSDParser::CreateEntityId(
         break;
     }
     if (prefix) {
-        if (m_PrefixToNamespace.find(*prefix) == m_PrefixToNamespace.end()) {
+        if (m_PrefixToNamespace.find(*prefix) == m_PrefixToNamespace.end() && !prefix->empty()) {
             string msg("Namespace prefix not defined: ");
             msg += *prefix;
             ParseError(msg.c_str(), "namespace declaration");
@@ -1363,6 +1373,7 @@ void XSDParser::ParseTypeDefinition(DTDEntity& ent)
     data += '\n';
     data += m_Raw;
     ent.SetData(data);
+    m_Comments = 0;
 }
 
 void XSDParser::ProcessNamedTypes(void)
@@ -1438,6 +1449,8 @@ void XSDParser::ProcessNamedTypes(void)
                     }
                 }
                 processed.insert(i->second.GetName() + i->second.GetNamespaceName());
+            } else {
+                node.SetTypeIfUnknown(node.GetContent().empty() ? DTDElement::eEmpty : DTDElement::eString);
             }
         }
     } while (found);
@@ -1458,9 +1471,7 @@ void XSDParser::ProcessNamedTypes(void)
                             found = true;
                             PushEntityLexer(a->GetTypeName());
                             ParseContent(*a);
-                            if (a->GetType() == DTDAttribute::eUnknown) {
-                                a->SetType(DTDAttribute::eString);
-                            }
+                            a->SetTypeIfUnknown(DTDAttribute::eString);
                         } else if ( a->GetType() == DTDAttribute::eUnknownGroup) {
                             found = true;
                             PushEntityLexer(a->GetTypeName());
@@ -1468,6 +1479,8 @@ void XSDParser::ProcessNamedTypes(void)
                             ParseAttributeGroupRef(node);
                             break;
                         }
+                    } else {
+                        a->SetTypeIfUnknown(DTDAttribute::eString);
                     }
                 }
             }
