@@ -1,4 +1,4 @@
-/*  $Id: bytesrc.cpp 355801 2012-03-08 16:20:00Z ivanovp $
+/*  $Id: bytesrc.cpp 506679 2016-07-11 12:10:32Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -203,6 +203,9 @@ void CStreamByteSourceReader::Seekg(CNcbiStreampos pos)
 {
     m_Stream->clear();
     m_Stream->seekg(pos);
+    if (m_Stream->fail()) {
+        NCBI_THROW(CIOException, eRead, "Failed to set read position");
+    }
 }
 
 
@@ -510,7 +513,13 @@ size_t CMemoryByteSourceReader::Read(char* buffer, size_t bufferLength)
 
 bool CMemoryByteSourceReader::EndOfData(void) const
 {
-    return !m_CurrentChunk;
+    if (!m_CurrentChunk) {
+        return true;
+    }
+    if (GetCurrentChunkAvailable() != 0) {
+        return false;
+    }
+    return !(m_CurrentChunk->GetNextChunk());
 }
 
 
