@@ -1,4 +1,4 @@
-/*  $Id: format_guess.cpp 497436 2016-04-06 17:56:51Z ivanov $
+/*  $Id: format_guess.cpp 509869 2016-08-09 17:31:55Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -167,6 +167,7 @@ int CFormatGuess::s_CheckOrder[] =
     eGvf,
     eGff3,
     eGtf,
+    eGffAugustus,
     eGff2,
     eGlimmer3,
     eAgp,
@@ -188,6 +189,7 @@ int CFormatGuess::s_CheckOrder[] =
     eTable,
     eBinaryASN,
     eUCSCRegion,
+    eJSON
 };
 
 
@@ -229,7 +231,9 @@ const char* const CFormatGuess::sm_FormatNames[CFormatGuess::eFormat_max] =
     "SRA",
     "BAM",
     "VCF",
-    "UCSC Region"
+    "UCSC Region",
+    "GFF Augustus",
+    "JSON"
 };
 
 const char*
@@ -321,7 +325,7 @@ CFormatGuess::SequenceType(const char* str, unsigned length,
 
 
 //  ----------------------------------------------------------------------------
-CFormatGuess::EFormat CFormatGuess::Format(const string& path, EOnError onerror)
+CFormatGuess::EFormat CFormatGuess::Format(const string& path, EOnError /*onerror*/)
 {
     CNcbiIfstream input(path.c_str(), IOS_BASE::in | IOS_BASE::binary);
     return Format(input);
@@ -505,6 +509,10 @@ bool CFormatGuess::x_TestFormat(EFormat format, EMode mode)
         return TestFormatVcf( mode );
     case eUCSCRegion:
         return false;
+    case eGffAugustus:
+        return TestFormatAugustus( mode );
+    case eJSON:
+        return TestFormatJson( mode );
     default:
         NCBI_THROW( CCoreException, eInvalidArg,
             "CFormatGuess::x_TestFormat(): Unsupported format ID (" +
@@ -797,6 +805,44 @@ CFormatGuess::TestFormatGff3(
 }
 
 
+//  -----------------------------------------------------------------------------
+bool
+CFormatGuess::TestFormatAugustus(
+    EMode /*not used*/)
+{
+    if ( ! EnsureTestBuffer() || ! EnsureSplitLines() ) {
+        return false;
+    }
+
+    unsigned int uGffLineCount = 0;
+    list<string>::iterator it = m_TestLines.begin();
+
+    for ( ;  it != m_TestLines.end();  ++it) {
+        //
+        //  Make sure to ignore any UCSC track and browser lines prior to the
+        //  start of data
+        //
+        if (!uGffLineCount && NStr::StartsWith(*it, "##gff-version 3")) {
+            return false;
+        }
+        if ( it->empty() || (*it)[0] == '#' ) {
+            continue;
+        }
+        if ( !uGffLineCount && NStr::StartsWith( *it, "browser " ) ) {
+            return false;
+        }
+        if ( !uGffLineCount && NStr::StartsWith( *it, "track " ) ) {
+            return false;
+        }
+        if ( !IsLineAugustus( *it ) ) {
+            return false;
+        }
+        ++uGffLineCount;
+    }
+    return (uGffLineCount != 0);
+}
+
+
 //  -----------------------------------------------------------------------------
 bool
 CFormatGuess::TestFormatGff2(
@@ -1085,10 +1131,7 @@ CFormatGuess::TestFormatFiveColFeatureTable(
             continue;
         }
 
-        if (it->find(">Feature ") != 0) {
-            return false;
-        }
-        if (it->find_first_of(" \t", 9) != string::npos) {
+        if (it->find(">Feature ") != 0 && it->find(">Features ") != 0) {
             return false;
         }
         break;
@@ -1697,10 +1740,10 @@ bool CFormatGuess::TestFormatBam(EMode mode)
             &&  m_pTestBuffer[12] == 'B'  &&  m_pTestBuffer[13] == 'C');
 }
 
+
 //  ----------------------------------------------------------------------------
 bool CFormatGuess::TestFormatVcf(
     EMode)
-//  ----------------------------------------------------------------------------
 {
     // Currently, only look for the header line identifying the VCF version.
     // Waive requirement this be the first line, but still expect it to by
@@ -1716,6 +1759,338 @@ bool CFormatGuess::TestFormatVcf(
     }
     return false;
 }
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+void CFormatGuess::x_StripJsonStrings(string& testString) const
+{
+    list<size_t> limits;
+    x_FindJsonStringLimits(testString, limits);
+
+    // If no strings found
+    if ( limits.empty() ) {
+        return;
+    }
+
+    if (limits.size()%2 == 1) { 
+        // Perhaps testString ends on an open string 
+        // Tack on an additional set of quotes at the end
+        testString += "\"";
+        limits.push_back(testString.size()-1);       
+    }
+    // The length of the limits container is now even
+
+    // Iterate over string start and stop sites
+    // Strip strings and copy what remains to complement
+    string complement = "";
+
+    auto it = limits.begin();
+    size_t comp_interval_start = 0;
+    while (it != limits.end()) {
+        const size_t string_start = *it++;
+        if (string_start > comp_interval_start) {
+            const size_t comp_interval_length = string_start-comp_interval_start;
+            complement += testString.substr(comp_interval_start, comp_interval_length);
+        }
+
+        const size_t string_stop = *it++;
+        comp_interval_start = string_stop+1;
+    }
+
+    if (comp_interval_start < testString.size()) {
+        complement += testString.substr(comp_interval_start);
+    }
+
+    testString = complement;
+    return;
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+void CFormatGuess::x_FindJsonStringLimits(const string& input, list<size_t>& limits) const
+{
+    limits.clear();
+    const string& double_quotes = R"(")";
+
+    bool is_start = true;
+    size_t pos = NStr::Find(input, double_quotes);
+    // List all string start and stop positions
+    while ( pos != NPOS ) {
+        limits.push_back(pos);
+        if (is_start) {
+            pos = x_FindNextJsonStringStop(input, pos+1);
+        } else {
+            pos = NStr::Find(input, double_quotes, pos+1);
+        }
+        is_start = !is_start;
+    }
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+size_t s_GetPrecedingFslashCount(const string& input, const size_t pos)
+{
+    if (pos == 0 ||
+        pos >= input.size() ||
+        NStr::IsBlank(input) ) 
+    {
+        return 0;
+    }
+
+    int current_pos = pos-1;
+    size_t num_fslash = 0;
+    while  ( current_pos >= 0 && input[current_pos] == '\\' ) {
+        ++num_fslash;
+        --current_pos;
+    }
+    return num_fslash;
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+size_t CFormatGuess::x_FindNextJsonStringStop(const string& input, const size_t from_pos) const
+{
+    const string& double_quotes = R"(")";
+    size_t pos = NStr::Find(input, double_quotes, from_pos);
+
+    // Double quotes immediately preceded by an odd number of forward 
+    // slashes, for example, /", ///", are escaped
+    while (pos != NPOS) {
+        const size_t num_fslash = s_GetPrecedingFslashCount(input, pos);
+        // If the number of forward slashes is even,
+        // return the position of the double quotes
+        if (num_fslash%2 == 0) {
+            break;
+        }
+        pos = NStr::Find(input, double_quotes, pos+1);
+    }   
+    return pos;
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+bool CFormatGuess::x_CheckStripJsonNumbers(string& testString) const
+{
+    if (NStr::IsBlank(testString)) {
+        return true;
+    }
+
+    list<string> subStrings;
+    // Split on white space
+    NStr::Split(testString, " \r\t\n", subStrings, NStr::fSplit_MergeDelims);
+
+    for (auto it = subStrings.cbegin(); it != subStrings.cend(); ++it) {
+        const string subString = *it;
+
+        if (!x_IsNumber(subString)) { // The last substring might be a truncated number or keyword
+           ++it;
+           if (it == subStrings.cend()) {
+               testString = subString;
+               return true;
+           }
+           return false;
+        }
+    }
+
+    testString.clear();
+    return true;
+}
+//  ----------------------------------------------------------------------------
+
+
+// -----------------------------------------------------------------------------
+bool CFormatGuess::x_IsTruncatedJsonNumber(const string& testString) const
+{
+    // Truncation of a JSON number may result strings of the following type:
+    //  1.1e
+    //  1.1E
+    //  1.7E-
+    //  +
+    //  -
+    // NStr::StringToDouble cannot handle such truncations, but we can "fix"
+    // the truncation by appending zero ("0") to the truncated string
+
+    const string extendedString = testString + "0"; 
+
+    return x_IsNumber(extendedString);
+}
+// -----------------------------------------------------------------------------
+
+
+// -----------------------------------------------------------------------------
+bool CFormatGuess::x_IsNumber(const string& testString) const 
+{
+    try {
+        NStr::StringToDouble(testString);
+    } 
+    catch (...) {
+        return false;
+    }
+    return true;
+}
+// -----------------------------------------------------------------------------
+
+
+// -----------------------------------------------------------------------------
+bool CFormatGuess::x_IsTruncatedJsonKeyword(const string& testString) const
+{
+    const size_t stringSize = testString.size();
+    // nul, tru, fals
+    if (stringSize > 4) {
+        return false;
+    }
+
+    const string nullString("null");
+    const string trueString("true");
+    const string falseString("false");
+
+    if (testString == nullString.substr(0, stringSize) ||
+        testString == trueString.substr(0, stringSize) ||
+        testString == falseString.substr(0, stringSize)) {
+        return true;
+    }
+
+    return false;
+}
+// -----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+bool CFormatGuess::x_IsBlankOrNumbers(const string& testString) const 
+{
+    if (NStr::IsBlank(testString)) {
+        return true;
+    }
+
+    list<string> numStrings;
+    // Split on white space
+    NStr::Split(testString, " \r\t\n", numStrings, NStr::fSplit_MergeDelims);
+
+    for (auto numString : numStrings) {
+        if (!x_IsNumber(numString)) {
+            return false;
+        }
+    }
+
+    return true;
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+bool CFormatGuess::x_CheckStripJsonPunctuation(string& testString) const 
+{
+    // Parentheses are prohibited
+    if (testString.find_first_of("()") != string::npos) {
+        return false;
+    }
+
+    const size_t punctuation_threshold = 4;
+
+    // Reject if the number of punctuation characters falls below some threshold value.
+    // In this case, the threshold is hardcoded to 4.
+    if (x_StripJsonPunctuation(testString) < punctuation_threshold) {
+        return false;
+    }
+
+    return true;
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+size_t CFormatGuess::x_StripJsonPunctuation(string& testString) const 
+{
+    size_t initial_len = testString.size();
+
+    NStr::ReplaceInPlace(testString, "{", "");
+    NStr::ReplaceInPlace(testString, "}", "");
+    NStr::ReplaceInPlace(testString, "[", "");
+    NStr::ReplaceInPlace(testString, "]", "");
+    NStr::ReplaceInPlace(testString, ":", "");
+    NStr::ReplaceInPlace(testString, ",", "");
+
+    return testString.size() - initial_len;
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+void CFormatGuess::x_StripJsonKeywords(string& testString) const 
+{
+    NStr::ReplaceInPlace(testString, "true", "");
+    NStr::ReplaceInPlace(testString, "false", "");
+    NStr::ReplaceInPlace(testString, "null", "");
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+bool CFormatGuess::x_CheckJsonStart(const string& testString) const 
+{
+    if (NStr::StartsWith(testString, "{")) {
+        // Next character must begin a string
+        const auto next_pos = testString.find_first_not_of("( \t\r\n",1); 
+        if (next_pos != NPOS && testString[next_pos] == '\"') {
+            return true;
+        }
+    } 
+    else
+    if (NStr::StartsWith(testString, "[")) {
+        return true;
+    }
+
+    return false;
+}
+//  ----------------------------------------------------------------------------
+
+
+//  ----------------------------------------------------------------------------
+bool CFormatGuess::TestFormatJson(
+        EMode)
+{
+
+    // Convert the test-buffer character array to a string
+    string testString(m_pTestBuffer, m_iTestDataSize);
+
+    if ( NStr::IsBlank(testString) ) {
+        return false;
+    }
+
+    NStr::TruncateSpacesInPlace(testString, NStr::eTrunc_Begin);
+
+    if (!x_CheckJsonStart(testString)) {
+        return false;
+    }
+
+    x_StripJsonStrings(testString);
+
+    if ( !x_CheckStripJsonPunctuation(testString) ) {
+        return false;
+    }
+
+    x_StripJsonKeywords(testString);
+
+    if (!x_CheckStripJsonNumbers(testString)) {
+        return false;
+    }
+
+    if ( NStr::IsBlank(testString) ) {
+        return true;
+    }
+
+    // What remains is either a truncated number
+    // or a truncated keyword
+    return x_IsTruncatedJsonNumber(testString) | 
+           x_IsTruncatedJsonKeyword(testString);
+}
+//  ----------------------------------------------------------------------------
+
 
 //  ----------------------------------------------------------------------------
 bool CFormatGuess::IsInputRepeatMaskerWithHeader()
@@ -2259,6 +2634,99 @@ bool CFormatGuess::IsLineGff3(
 }
 
 
+//  ----------------------------------------------------------------------------
+bool CFormatGuess::IsLineAugustus(
+    const string& line )
+{
+    vector<string> tokens;
+    string remaining(line), head, tail;
+
+    //column 0: ID, string
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)) {
+        return false;
+    }
+    remaining = tail;
+
+    //column 1: method, most likely "AUGUSTUS" but don't want to rely on this
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)) {
+        return false;
+    }
+    remaining = tail;
+
+    //column 2: feature type, controlled vocabulary
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)) {
+        return false;
+    }
+    remaining = tail;
+    string featureType = head;
+
+    //column 3: start, integer
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)  ||  !s_IsTokenPosInt(head)) {
+        return false;
+    }
+    remaining = tail;
+
+    //column 4: stop, integer
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)  ||  !s_IsTokenPosInt(head)) {
+        return false;
+    }
+    remaining = tail;
+
+    //column 5: score, double
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)  ||  !s_IsTokenDouble(head)) {
+        return false;
+    }
+    remaining = tail;
+
+    //column 6: strand, one in "+-.?"
+    const string legalStrands{"+-.?"};
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)  ||  !head.size() == 1  ||  
+            string::npos == legalStrands.find(head)) {
+        return false;
+    }
+    remaining = tail;
+
+    //column 7: phase, one in ".0123"
+    const string legalPhases{".0123"};
+    if (!NStr::SplitInTwo(remaining, " \t", head, tail)  ||  !head.size() == 1  ||  
+            string::npos == legalPhases.find(head)) {
+        return false;
+    }
+    remaining = tail;
+
+    //everything else: attributes, format depends on featureType
+    if (remaining.empty()) {
+        return false;
+    }
+
+    if (featureType == "gene") {
+        if (NPOS != NStr::Find(remaining, ";")) {
+            return false;
+        }
+        if (NPOS != NStr::Find(remaining, " ")) {
+            return false;
+        }
+        return true;
+    }
+    if (featureType == "transcript") {
+        if (NPOS != NStr::Find(remaining, ";")) {
+            return false;
+        }
+        if (NPOS != NStr::Find(remaining, " ")) {
+            return false;
+        }
+        return true;
+    }
+    if (NPOS == NStr::Find(remaining, "transcript_id")) {
+        return false;
+    }
+    if (NPOS == NStr::Find(remaining, "gene_id")) {
+        return false;
+    }
+    return true;
+}
+
+
 //  ----------------------------------------------------------------------------
 bool CFormatGuess::IsLineGff2(
     const string& line )
@@ -2399,6 +2867,7 @@ CFormatGuess::IsAsnComment(
 }
 
 //  ----------------------------------------------------------------------------
+
 bool
 CFormatGuess::EnsureSplitLines()
 //  ----------------------------------------------------------------------------
